Package: docorator
Title: Docorate (Decorate + Output) Displays
Version: 0.7.0
Authors@R: 
    c(
      person(given = "Shannon", family = "Haughton", email = "shannon.l.haughton@gsk.com", role = c("aut", "cre")),
      person(given = "Becca", family = "Krouse", email = "becca.z.krouse@gsk.com", role = c("aut")),
      person(given = "Seongbin", family = "Hong", email = "seongbin.x.hong@gsk.com", role = c("aut")),
      person("Dragoș", "Moldovan-Grünfeld", , "dragos.v.moldovan-grunfeld@gsk.com", role = "aut"),
      person("GlaxoSmithKline Research & Development Limited", role = c("cph", "fnd")))
Description: A framework for creating production outputs. Users can frame a table, listing, or figure with headers and footers and save to an output file. Stores an intermediate 'docorator' object for reproducibility and rendering to multiple output types.
License: Apache License 2.0
Encoding: UTF-8
Imports: gt (>= 1.2.0), rmarkdown, rlang, cli, dplyr, rstudioapi,
        purrr, stringr, stringi, tidyr, lifecycle, png, knitr, withr,
        quarto, officer, polish, xml2
Suggests: rprojroot, testthat (>= 3.0.0), tfrmt, pdftools, ggplot2
Config/testthat/edition: 3
VignetteBuilder: knitr
URL: https://GSK-Biostatistics.github.io/docorator/,
        https://github.com/GSK-Biostatistics/docorator
BugReports: https://github.com/GSK-Biostatistics/docorator/issues
Depends: R (>= 4.1.0)
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-08 12:34:00 UTC; slh95484
Author: Shannon Haughton [aut, cre],
  Becca Krouse [aut],
  Seongbin Hong [aut],
  Dragoș Moldovan-Grünfeld [aut],
  GlaxoSmithKline Research & Development Limited [cph, fnd]
Maintainer: Shannon Haughton <shannon.l.haughton@gsk.com>
Repository: CRAN
Date/Publication: 2026-07-08 14:20:24 UTC
Built: R 4.5.2; ; 2026-07-08 16:27:07 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(apply_scale,default)
S3method(apply_scale,gt_group)
S3method(apply_scale,gt_tbl)
S3method(apply_scale,list)
S3method(fancywrap,default)
S3method(fancywrap,docorator)
S3method(fancywrap,fancyfoot)
S3method(fancywrap,fancyhead)
S3method(fancywrap,fancyrow)
S3method(hf_height,default)
S3method(hf_height,fancyhdr)
S3method(hf_process,character)
S3method(hf_process,default)
S3method(hf_process,fancyfoot)
S3method(hf_process,fancyhead)
S3method(prep_obj_docx,PNG)
S3method(prep_obj_docx,default)
S3method(prep_obj_docx,gt_tbl)
S3method(prep_obj_docx,list)
S3method(prep_obj_rtf,PNG)
S3method(prep_obj_rtf,character)
S3method(prep_obj_rtf,default)
S3method(prep_obj_rtf,ggplot)
S3method(prep_obj_rtf,gt_tbl)
S3method(prep_obj_rtf,list)
S3method(prep_obj_tex,PNG)
S3method(prep_obj_tex,character)
S3method(prep_obj_tex,default)
S3method(prep_obj_tex,gt_tbl)
export(apply_scale)
export(apply_to_gt_group)
export(as_docorator)
export(create_chunks_all)
export(doc_datetime)
export(doc_pagenum)
export(doc_path)
export(docorate)
export(fancyfoot)
export(fancyhead)
export(fancyrow)
export(geom_set)
export(hf_extract)
export(hf_height)
export(hf_process)
export(hf_to_gt)
export(png_path)
export(prep_obj_docx)
export(prep_obj_rtf)
export(prep_obj_tex)
export(render_docx)
export(render_pdf)
export(render_rtf)
export(scale_gt)
importFrom(rlang,`%||%`)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# docorator <a href='https://gsk-biostatistics.github.io/docorator/'><img src="man/figures/docorator_hex.png" align="right" alt = "docorator logo" style="height:139px;"/></a>

<!-- badges: start -->

[![R-CMD-check](https://github.com/GSK-Biostatistics/docorator/actions/workflows/R-CMD-check.yml/badge.svg)](https://github.com/GSK-Biostatistics/docorator/actions/workflows/R-CMD-check.yml)
[![status:
experimental](https://github.com/GIScience/badges/raw/master/status/experimental.svg)](https://github.com/GIScience/badges#experimental)
<!-- badges: end -->

{docorator} is an engine for creating production displays from R.
Specifically, {docorator} provides the following functionality:

- *Decorate*: Surround displays with document-level headers, footers,
  and page numbers
- *Output*: Export decorated displays to a file such as PDF or RTF

The main function, `as_docorator()` accepts an R object containing the
display as input. For best results with sizing, the table should be
prepared via {gt}, or packages that interface with {gt} such as {tfrmt},
and the figure should be prepared with {ggplot2}. Alternatively, the
path to a PNG can be passed to {docorator}. Specifically, the `x` should
be one of the following:

- `gt` table
- `gt_group` (list of `gt` tables)
- `ggplot2` figure
- list of `ggplot` figures
- path to PNG file (i.e. a previously created figure)
- list of paths to PNG files

The `docorator` object, which is saved by default, can then be output
using one or both of the following render functions which can be piped
together: `render_pdf()` and `render_rtf()`

# Installation

The {docorator} package can be installed from CRAN with:

``` r
install.packages("docorator")
```

The development version of {docorator} can be installed with:

``` r
devtools::install_github("GSK-Biostatistics/docorator")
```

# Use

``` r
# Load package
library(docorator) 

# Create table (or figure)
mytbl <- gt::exibble |> 
  gt::gt()

# Export display   
mytbl |> 
  as_docorator( 
    display_name = "mytbl",
    header = fancyhead(
      fancyrow(left = "My Study", center = NA, right = doc_pagenum()),
      fancyrow(left = "My Population", center = NA, right = NA),
      fancyrow(left = NA, center = "My Table", right = NA)
    ),
    footer = fancyfoot(
      fancyrow(left = "mypath/mytbl.R", center = NA, right = "Data as of 2025-01-01")
    ) 
  ) |> 
  render_pdf() |>
  render_rtf()
```

<img src="man/figures/doc_example.png" alt="Docorator example" width="700px" style="display: block; margin: auto;" />

# Note

{docorator} requires LaTeX libraries for PDF rendering. If using
{tinytex}, the “TinyTeX” bundle covers the core dependencies.

#' Render to pdf
#'
#' @param x `docorator` object
#' @param display_loc optional path to save the output pdf to
#' @param transform optional latex transformation function to apply to a gt latex string
#' @param header_latex optional .tex file of header latex
#' @param keep_tex Boolean indicating if to keep resulting .tex file from latex conversion. Defaults to FALSE.
#' @param escape_latex Boolean indicating if headers and footers of a gt table should be escaped with gt::escape_latex
#' @param quarto Boolean indicating whether to use Quarto as the rendering engine. Defaults to `FALSE`, which uses Rmarkdown to render. `r lifecycle::badge("experimental")`
#' @param version_check Boolean indicating whether to print a note if gt or ggplot versions dont match between the original docorator object and the one being used for rendering
#' @param fancywrap Boolean indicating if headers and footers should be split to fit the page. Defaults to `TRUE`. Note that only fancyrows with one `left`, `right` OR `center` element will be wrapped. `r lifecycle::badge("experimental")`
#'
#'
#' @returns This function saves a pdf to a specified location
#' @export
#'
#' @section Examples:
#'
#' ```r
#' gt::gtcars |>
#'   dplyr::slice_head(n = 10) |>
#'   dplyr::select(mfr, model, year, msrp) |>
#'   gt::gt(groupname_col = "mfr",
#'          row_group_as_column = TRUE) |>
#'   as_docorator(
#'    header = fancyhead(fancyrow("Header 1"), fancyrow("Header 2")),
#'    display_name = "mytbl") |>
#'  render_pdf()
#' ```
#'
render_pdf <- function(
  x,
  display_loc = NULL,
  transform = NULL,
  header_latex = NULL,
  keep_tex = FALSE,
  escape_latex = TRUE,
  quarto = FALSE,
  version_check = TRUE,
  fancywrap = TRUE
) {
  if (!inherits(x, "docorator")) {
    cli::cli_abort(
      "The {.arg {rlang::caller_arg(x)}} argument must be class docorator, not {.obj_type_friendly {x}}. See documentation for `as_docorator`.",
      call = rlang::caller_env()
    )
  }

  # check package versions
  if (isTRUE(version_check)) {
    check_pkg_version(x)
  }

  if (isTRUE(fancywrap)) {
    x <- fancywrap(x)
  }

  # check transform is a function if not convert to NULL
  if (!is.null(transform) & !inherits(transform, "function")) {
    cli::cli_warn(
      "The transform argument must be a function, not {.obj_type_friendly {transform}}. No transform applied.",
      call = rlang::caller_env()
    )
    transform <- NULL
  }

  # copy template rmd to a temp directory
  template <- system.file("template", package = "docorator")
  temp_dir <- tempdir()
  file.copy(template, temp_dir, overwrite = TRUE, recursive = TRUE)
  template_folder <- file.path(temp_dir, "template")

  # copy tex header to temp directory and rename if one exists
  if (!is.null(header_latex)) {
    if (file.exists(header_latex) && tools::file_ext(header_latex) == "tex") {
      file_name <- basename(header_latex)
      file.copy(
        header_latex,
        template_folder,
        overwrite = TRUE,
        recursive = TRUE
      )
      file.rename(
        file.path(template_folder, file_name),
        file.path(template_folder, "header.tex")
      )
    } else {
      cli::cli_warn(
        "The header_latex argument must point to a valid .tex file. No header options applied.",
        call = rlang::caller_env()
      )
    }
  }

  # if no path is given, use docorator path
  if (is.null(display_loc)) {
    display_loc <- x$display_loc %||% "."
  }

  # set filename
  filename <- paste0(x$display_name, ".pdf")

  if (quarto) {
    withr::with_envvar(
      new = c("DOCORATOR_RENDER_ENGINE" = "qmd"),
      render_pdf_qmd(x, display_loc, transform, header_latex, clean = !keep_tex)
    )
  } else {
    # render rmd -> pdf
    doc <- rmarkdown::render(
      file.path(temp_dir, "template", "template.Rmd"),
      output_file = filename,
      output_dir = display_loc,
      output_options = list(keep_tex = keep_tex),
      params = list(
        x = x,
        header = hf_process(x$header, escape_latex = escape_latex),
        footer = hf_process(x$footer, escape_latex = escape_latex),
        geometry = geom_process(
          x$header,
          x$footer,
          x$fontsize,
          x$geometry
        ),
        transform = transform
      ),
      quiet = TRUE
    )

    if (!is.null(doc)) {
      cli::cli_alert_success("Document created at: {doc}")
    }

    # clean up temp files
    unlink(file.path(temp_dir, "template"), recursive = TRUE)

    # return docorator object for further renders
    invisible(x)
  }
}


#' Render to rtf
#'
#' `r lifecycle::badge('experimental')`
#'
#' @param x `docorator` object
#' @param display_loc path to save the output rtf to
#' @param remove_unicode_ws Option to remove unicode white space from text.
#' @param use_page_header If `TRUE` then all table headings will be migrated to the page header. See https://gt.rstudio.com/reference/tab_options.html#arg-page-header-use-tbl-headings
#' @param version_check Boolean indicating whether to print a note if gt or ggplot versions dont match between the original docorator object and the one being used for rendering
#'
#' @details Option `remove_unicode_ws` serves as a workaround for this
#'   [issue](https://github.com/rstudio/gt/issues/1437) in gt
#'
#' @returns This function saves an rtf to a specified location
#' @export
#'
#' @section Examples:
#'
#' ```r
#' gt::gtcars |>
#'   dplyr::slice_head(n = 10) |>
#'   dplyr::select(mfr, model, year, msrp) |>
#'   gt::gt(groupname_col = "mfr",
#'          row_group_as_column = TRUE) |>
#'   as_docorator(
#'    header = fancyhead(fancyrow("Header 1"), fancyrow("Header 2")),
#'    display_name = "mytbl") |>
#'  render_rtf()
#' ```
#'
render_rtf <- function(
  x,
  display_loc = NULL,
  remove_unicode_ws = TRUE,
  use_page_header = FALSE,
  version_check = TRUE
) {
  if (!inherits(x, "docorator")) {
    cli::cli_abort(
      "The {.arg {rlang::caller_arg(x)}} argument must be class docorator, not {.obj_type_friendly {x}}. See documentation for `as_docorator`.",
      call = rlang::caller_env()
    )
  }

  # check package versions
  if (isTRUE(version_check)) {
    check_pkg_version(x)
  }

  # if no path is given, use docorator path
  if (is.null(display_loc)) {
    display_loc <- x$display_loc %||% "."
  }

  # set name
  filename <- paste0(x$display_name, ".rtf")

  # convert outputs to gt for rtf render
  gt <- prep_obj_rtf(x)

  # page headers
  gt <- apply_to_gt_group(
    gt,
    gt::tab_options,
    list(
      page.numbering = FALSE,
      page.header.use_tbl_headings = use_page_header
    )
  )

  # render rtf
  doc <- gt::gtsave(gt, filename = filename, path = display_loc)

  if (!is.null(doc)) {
    if (remove_unicode_ws) {
      doc_tmp <- readLines(doc)

      doc_tmp_new <- gsub("\u00A0", " ", doc_tmp, perl = TRUE)

      writeLines(
        doc_tmp_new,
        sep = "\n",
        file.path(display_loc, filename)
      )
    }
    cli::cli_alert_success(
      "Document created at: {normalizePath(doc, winslash = \"/\")}"
    )
  }

  # return docorator object for further renders
  invisible(x)
}


#' Render to pdf (quarto)
#'
#' @param x `docorator` object
#' @param display_loc path to save the output pdf to
#' @param transform optional latex transformation function to apply to a gt latex string
#' @param header_latex optional .tex file of header latex
#' @param clean whether to clean the temporary docorator files. Defaults to `TRUE`
#'
#' @returns This function saves a pdf to a specified location
#' @noRd
render_pdf_qmd <- function(
  x,
  display_loc = NULL,
  transform = NULL,
  header_latex = NULL,
  clean = TRUE
) {
  if (!is.null(transform)) {
    cli::cli_warn(
      "The {.arg {rlang::caller_arg(transform)}} argument is not currently available for quarto rendered documents. Try `quarto = FALSE`",
      call = rlang::caller_env()
    )
  }

  # create a full path
  if (!is.null(display_loc)) {
    display_loc <- normalizePath(display_loc, winslash = "/")
  }

  qmd_name <- paste0(x$display_name, ".qmd")
  pdf_name <- paste0(x$display_name, ".pdf")
  docorator_name <- paste0(x$display_name, "_docorator_obj.Rds")

  if (!is.null(display_loc)) {
    render_dir <- file.path(
      display_loc,
      paste0(x$display_name, "_docorator_files")
    )
  } else {
    render_dir <- file.path(paste0(x$display_name, "_docorator_files"))
  }
  if (!dir.exists(render_dir)) {
    dir.create(render_dir)
  }

  on.exit(
    {
      if (isTRUE(clean)) {
        unlink(render_dir, recursive = TRUE)
      }
    },
    add = TRUE
  )

  withr::with_dir(
    new = render_dir,
    code = {
      # copy template qmd to the render dir
      template <- system.file("template", "template.qmd", package = "docorator")
      file.copy(template, ".", overwrite = TRUE, recursive = TRUE)
      file.rename("template.qmd", qmd_name)

      # copy tex header to render dir and rename if one exists
      if (!is.null(header_latex)) {
        if (
          file.exists(header_latex) && tools::file_ext(header_latex) == "tex"
        ) {
          file_name <- basename(header_latex)
          file.copy(header_latex, ".", overwrite = TRUE, recursive = TRUE)
          file.rename(file_name, "header.tex")
        } else {
          cli::cli_warn(
            "The header_latex argument must point to a valid .tex file. No header options applied.",
            call = rlang::caller_env()
          )
        }
      }

      # save docorator obj to render dir
      saveRDS(x, docorator_name)

      # render pdf
      doc <-
        quarto::quarto_render(
          input = qmd_name,
          output_format = "pdf",
          output_file = pdf_name,
          execute_params = list(
            display_name = x$display_name,
            pkg_path = "", #set to cur_dir in development
            transform = NULL # disabled for quarto
          ),
          quiet = TRUE
        )
    }
  )

  if (file.exists(file.path(render_dir, pdf_name))) {
    if (!is.null(display_loc)) {
      out_path <- file.path(display_loc, pdf_name)
    } else {
      out_path <- pdf_name
    }
    file_ok <- file.copy(
      from = file.path(render_dir, pdf_name),
      to = out_path,
      overwrite = TRUE
    )

    if (file_ok) {
      cli::cli_alert_success("Document created at: {out_path}")
    }
  }

  # return docorator object for further renders
  invisible(x)
}

#' Apply scaling to gt objects
#'
#' @param x gt object
#' @param tbl_scale Boolean for whether or not to automatically scale table columns to fit display area. Defaults to TRUE. Note that this will overwrite scaling set in the table directly unless set to FALSE.
#' @param tbl_stub_pct percent of total width that should be dedicated to stub column(s). If more than 1 stub column then this is the total for both.
#' @param fontsize document font size
#' @return scaled gt object
#' @name apply_scale
#' @examples
#' gt <- gt::exibble |>
#' gt::gt()
#'
#' apply_scale(gt, fontsize = 10, tbl_scale = FALSE, tbl_stub_pct = "20%")
NULL

#' @name apply_scale
#' @export
#' @keywords internal
apply_scale <- function(x, fontsize, tbl_scale, tbl_stub_pct) {
  UseMethod("apply_scale", x)
}

#' @name apply_scale
#' @export
#' @keywords internal
apply_scale.default <- function(x, fontsize, tbl_scale, tbl_stub_pct) {
  x
}

#' @name apply_scale
#' @export
#' @keywords internal
apply_scale.gt_tbl <- function(x, fontsize, tbl_scale, tbl_stub_pct) {

  table_width <- "100%"
  if (isFALSE(tbl_scale)) {
    scaling <- check_gt_widths(x)
    tbl_scale <- scaling$tbl_scale
    table_width <- scaling$table_width
  }

  if (isTRUE(tbl_scale)) {
    x <- scale_gt(x, tbl_stub_pct)
  }

  x |>
    gt::tab_options(
      table.font.size = paste0(fontsize, "pt"),
      heading.subtitle.font.size = paste0(fontsize, "pt"),
      heading.title.font.size = paste0(fontsize, "pt"),
      table.width = table_width
    )
}

#' @name apply_scale
#' @export
#' @keywords internal
apply_scale.gt_group <- function(x, fontsize, tbl_scale, tbl_stub_pct) {
  arg_list <- list(
    fontsize = fontsize,
    tbl_scale = tbl_scale,
    tbl_stub_pct = tbl_stub_pct
  )

  x <- apply_to_gt_group(x, "apply_scale", arg_list)

  x
}

#' @name apply_scale
#' @export
#' @keywords internal
apply_scale.list <- function(x, fontsize, tbl_scale, tbl_stub_pct) {
  x <- lapply(
    seq_along(x),
    FUN = function(i) {
      x[[i]] <- apply_scale(
        x[[i]],
        fontsize = fontsize,
        tbl_scale = tbl_scale,
        tbl_stub_pct = tbl_stub_pct
      )
      x[[i]]
    }
  )
  x
}


#' Scale gt table contents for document
#'
#' @param x table of class `gt_tbl`
#' @param tbl_stub_pct percent of total width that should be dedicated to stub column(s). If more than 1 stub column then this is the total for both.
#'
#' @return Table with col_widths settings applied
#' @export
#'
#' @examples
#'
#' gt::gtcars |>
#'   dplyr::slice_head(n = 10) |>
#'   dplyr::select(mfr, model, year, msrp, ctry_origin) |>
#'   gt::gt(
#'       groupname_col = "ctry_origin",
#'       rowname_col = "mfr",
#'       row_group_as_column = TRUE) |>
#'   scale_gt(tbl_stub_pct = 0.4)
#'
scale_gt <- function(x, tbl_stub_pct = 0.3) {
  # will group add to stub?
  row_grp_as_column <- x$`_options` |>
    dplyr::filter(.data$parameter == "row_group_as_column") |>
    dplyr::pull(.data$value) |>
    unlist()

  # total # of columns in table
  boxhead <- x[["_boxhead"]] |>
    dplyr::filter(!.data$type == "hidden")
  n_cols <- nrow(boxhead)

  # # of stub columns in table
  n_stubs <- boxhead |>
    dplyr::filter(.data$type %in% c("stub", "row_group")) |>
    nrow()

  # account for row_grp NOT as column
  if (n_stubs > 0 && "row_group" %in% boxhead$type && !row_grp_as_column) {
    n_stubs <- n_stubs - 1
  }

  if (n_stubs == 0) {
    tbl_stub_pct <- 0
  }
  widths_cols <- (100 - 100 * tbl_stub_pct) / (n_cols - n_stubs)
  list_widths <- vector(mode = "list")

  # add stub %s
  if (n_stubs > 0) {
    widths_stub <- 100 * tbl_stub_pct / n_stubs

    if ("stub" %in% x[["_boxhead"]]$type) {
      list_widths <- c(
        list_widths,
        rlang::new_formula(quote(gt::stub()), paste0(widths_stub, "%"))
      )
    }
    if ("row_group" %in% x[["_boxhead"]]$type) {
      row_grp_var <- dplyr::filter(
        x[["_boxhead"]],
        .data$type == "row_group"
      )$var
      list_widths <- c(
        list_widths,
        rlang::new_formula(row_grp_var, paste0(widths_stub, "%"))
      )
    }
  }

  list_widths <- c(
    list_widths,
    rlang::new_formula(quote(dplyr::everything()), paste0(widths_cols, "%"))
  )

  # temp fix for order until gt #1922 resolved
  x$`_boxhead` <- x$`_boxhead` |>
    dplyr::arrange(match(
      .data$type,
      c("stub", "row_group", "default", "hidden")
    ))

  gt::cols_width(
    x,
    .list = list_widths
  )
}

#' function to check manually specified widths are in pct and are <100
#' @return boolean true if needs rescaling, false otherwise.
#' @noRd
check_gt_widths <- function(x) {
  # preset table_width
  table_width <- "100%"

  # preset to false
  tbl_scale <- FALSE

  # get rid of hidden columns
  no_hide <- x$`_boxhead` |>
    dplyr::filter(!.data$type == "hidden")

  # vector of widths frm gt
  width_vec <- unlist(no_hide$column_width)

  # remove empty widths
  width_vec_full <- width_vec[width_vec != ""]

  # extract numeric values and sum
  nums <- gregexpr("[0-9]+(?:\\.[0-9]+)?", width_vec_full)
  width_sum <- sum(as.numeric(unlist(regmatches(width_vec_full, nums))))

  if (grepl("px", paste0(width_vec_full, collapse = ""))) {
    cli::cli_warn(
      "Column widths must be specified in % not px. Applying auto table scaling.",
      call = rlang::caller_env()
    )
    tbl_scale <- TRUE
  } else if (
    length(width_vec_full) > 0 &
      !grepl("%", paste0(width_vec_full, collapse = ""))
  ) {
    cli::cli_warn(
      "Column widths must be specified in %. Applying auto table scaling.",
      call = rlang::caller_env()
    )
    tbl_scale <- TRUE
  } else if (
    nrow(no_hide) > length(width_vec_full) &
      length(width_vec_full) > 0 &
      width_sum < 100
  ) {
    cli::cli_inform(
      "NOTE: Column widths total <100%. Resulting table may not respect document margins.",
      call = rlang::caller_env()
    )

    # dont rescale
  } else if (width_sum > 100) {
    cli::cli_warn(
      "Column widths must be add to =<100%, not {width_sum}%. Applying auto table scaling.",
      call = rlang::caller_env()
    )

    tbl_scale <- TRUE
  } else if (length(width_vec_full) == nrow(no_hide)) {
    # if all cols are specified, don't set table width = 100, dont rescale
    table_width <- NULL
  }

  list(tbl_scale = tbl_scale, table_width = table_width)
}

test_that("scale_gt works", {

  # table with 1 stub column
  tbl_1stub <- gt::exibble |>
    gt::gt(
      rowname_col = "row"
    )

  gt_out <- scale_gt(tbl_1stub, tbl_stub_pct = 0.3)$`_boxhead`

  expect_equal(
    dplyr::filter(gt_out, type=="stub")$column_width |> unlist(),
    "30%"
  )
  expect_equal(
    dplyr::filter(gt_out, !type=="stub")$column_width |> unlist(),
    rep("8.75%", ncol(gt::exibble)-1)
  )


  # table with 1 stub column and hidden column
  tbl_1stub_1hidden <- tbl_1stub |>
    gt::cols_hide(
      group
    )


  gt_out <- scale_gt(tbl_1stub_1hidden, tbl_stub_pct = 0.3)$`_boxhead`

  expect_equal(
    dplyr::filter(gt_out, type=="stub")$column_width |> unlist(),
    "30%"
  )
  expect_equal(
    dplyr::filter(gt_out, !type=="stub")$column_width |> unlist(),
    rep("10%", ncol(gt::exibble)-1)
  )

  # table with 2 stub columns
  tbl_2stub <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    ) |>
    gt::tab_options(
      row_group.as_column = TRUE
    )

  gt_out <- scale_gt(tbl_2stub, tbl_stub_pct = 0.3)$`_boxhead`

  expect_equal(
    dplyr::filter(gt_out, type=="stub")$column_width |> unlist(),
    "15%"
  )
  expect_equal(
    dplyr::filter(gt_out, !type=="stub" & var=="group")$column_width |> unlist(),
    "15%"
  )
  expect_equal(
    dplyr::filter(gt_out, !type=="stub" & !var=="group")$column_width |> unlist(),
    rep("10%", ncol(gt::exibble)-2)
  )

  # table with 0 stub columns
  tbl_nostub <- gt::exibble |>
    gt::gt()

  gt_out <- scale_gt(tbl_nostub, tbl_stub_pct = 0.3)$`_boxhead`

  widths <- as.numeric(gsub("%","",gt_out$column_width |> unlist()))
  expect_equal(unique(widths),
               100/ncol(gt::exibble))

  # table with 1 stub column that is a groupname_col (not rowname_col)
  tbl_grp1stub <- gt::exibble |>
    gt::gt(groupname_col = "fctr",
           row_group_as_column = TRUE)

  gt_out <- scale_gt(tbl_grp1stub, tbl_stub_pct = 0.3)$`_boxhead`

  expect_equal(dplyr::filter(gt_out, type=="stub") |> nrow(),
               0)
  expect_equal(
    dplyr::filter(gt_out, type=="row_group")$column_width |> unlist(),
    "30%"
  )
  expect_equal(
    dplyr::filter(gt_out, !type=="row_group")$column_width |> unlist(),
    rep("8.75%", ncol(gt::exibble)-1)
  )
})

test_that("scale_gt works with gt >= 1.1.0", {

  # up version once gt is released - requires multi-rowname cols (1.1.0) and
  #  bug fix for stub() helper (1.1.0.9000 currently)
  skip_if_not_installed("gt", minimum_version = "1.1.0.9000")

  # table with 2 stub columns that are rowname_col
  tbl_row2stub <- gt::exibble |>
    gt::gt(rowname_col = c("fctr", "row"))

  gt_out <- scale_gt(tbl_row2stub, tbl_stub_pct = 0.3)$`_boxhead`

  expect_equal(
    dplyr::filter(gt_out, type=="stub")$column_width |> unlist(),
    c("15%","15%")
  )
  expect_equal(
    dplyr::filter(gt_out, type=="row_group") |> nrow(),
    0
  )
  expect_equal(
    dplyr::filter(gt_out, !type %in% c("stub", "row_group"))$column_width |> unlist(),
    rep("10%", ncol(gt::exibble)-2)
  )

  # table with 3 stub columns - 1 grp, 2 rowname
  tbl_grprow3stub <- gt::exibble |>
    gt::gt(groupname_col = "group",
           rowname_col = c("fctr", "row"),
           row_group_as_column = TRUE)

  gt_out <- scale_gt(tbl_grprow3stub, tbl_stub_pct = 0.3)$`_boxhead`

  expect_equal(
    dplyr::filter(gt_out, type=="stub")$column_width |> unlist(),
    c("10%","10%")
  )
  expect_equal(
    dplyr::filter(gt_out, type=="row_group")$column_width |> unlist(),
    "10%"
  )
  expect_equal(
    dplyr::filter(gt_out, !type %in% c("stub", "row_group"))$column_width |> unlist(),
    rep(paste0(70/(ncol(gt::exibble)-3), "%"), ncol(gt::exibble)-3)
  )
})

test_that("check_gt_widths works",{
  no_widths <- gt::exibble |>
    gt::gt()

  expect_equal(check_gt_widths(no_widths),list(tbl_scale = FALSE, table_width = "100%"))

  large_widths <- no_widths |>
    gt::cols_width(
      num ~ "400.5%",
      char ~ "3%"
    )
  expect_warning(check_gt_widths(large_widths),"Column widths must be add to =<100%, not 403.5%. Applying auto table scaling.")
  expect_equal(suppressWarnings(check_gt_widths(large_widths)),list(tbl_scale = TRUE, table_width = "100%"))

  px_widths <- no_widths |>
    gt::cols_width(
      num ~ "400px",
      char ~ "3%"
    )

  expect_warning(check_gt_widths(px_widths),"Column widths must be specified in % not px. Applying auto table scaling.")
  expect_equal(suppressWarnings(check_gt_widths(px_widths)), list(tbl_scale = TRUE, table_width = "100%"))

  # defaults to px
  px_widths2 <- no_widths |>
    gt::cols_width(
      num ~ 40,
      char ~ 3
    )

  expect_warning(check_gt_widths(px_widths2),"Column widths must be specified in % not px. Applying auto table scaling.")
  expect_equal(suppressWarnings(check_gt_widths(px_widths2)), list(tbl_scale = TRUE, table_width = "100%"))

  # no % specified
  no_pct_widths <- no_widths |>
    gt::cols_width(
      num ~ "40",
      char ~ "3"
    )

  expect_warning(check_gt_widths(no_pct_widths),"Column widths must be specified in %. Applying auto table scaling.")
  expect_equal(suppressWarnings(check_gt_widths(px_widths2)),list(tbl_scale = TRUE, table_width = "100%"))

  good_widths <- no_widths |>
    gt::cols_width(
      num ~ "10%",
      char ~ "10%",
      fctr ~ "10%",
      date ~ "10%",
      time ~ "10%",
      datetime ~ "10%",
      currency ~ "10%",
      row ~ "30%"
    )

  expect_equal(check_gt_widths(good_widths), list(tbl_scale = FALSE, table_width = "100%"))

  # all columns are specified, <100
  small_widths <- no_widths |>
    gt::cols_width(
      num ~ "1%",
      char ~ "1%",
      fctr ~ "1%",
      date ~ "1%",
      time ~ "1%",
      datetime ~ "1%",
      currency ~ "1%",
      row ~ "3%",
      group ~ "2%"
    )

  expect_equal(check_gt_widths(small_widths), list(tbl_scale = FALSE, table_width = NULL))
})

test_that("apply_scale works - gt_tbl",{
  # bad column widths, scale false
  gt_tbl1 <- gt::exibble |>
    gt::gt() |> gt::cols_width(num ~ "400%", char ~ "3%") |>
    gt:: tab_options(table.font.size = "10pt",
                     heading.subtitle.font.size = "10pt",
                     heading.title.font.size = "10pt",
                     table.width = "100%"
    )

  expect_warning(apply_scale(gt_tbl1, fontsize = 10, tbl_scale = FALSE, tbl_stub_pct = 0.3),"Column widths must be add to =<100%, not 403%. Applying auto table scaling.")
  expect_identical(suppressWarnings(apply_scale(gt_tbl1, fontsize = 10, tbl_scale = FALSE, tbl_stub_pct = 0.3)), scale_gt(gt_tbl1, tbl_stub_pct = 0.3))

  # col widths <100
  gt_tbl2 <- gt::exibble |>
    gt::gt() |> gt::cols_width(num ~ "10%", char ~ "3%")

  expect_message(apply_scale(gt_tbl2, fontsize = 10, tbl_scale = FALSE, tbl_stub_pct = 0.3),"NOTE: Column widths total <100%. Resulting table may not respect document margins.")
  expect_identical(suppressMessages(apply_scale(gt_tbl2, fontsize = 10,  tbl_scale = FALSE, tbl_stub_pct = 0.3)), gt_tbl2 |>
                     gt:: tab_options(table.font.size = "10pt",
                                      heading.subtitle.font.size = "10pt",
                                      heading.title.font.size = "10pt",
                                      table.width = "100%"
                     ))

  # col widths =100
  gt_tbl3 <- gt::exibble |>
    dplyr::select(num, char) |>
    gt::gt() |> gt::cols_width(everything() ~ "50%" )


  expect_identical(apply_scale(gt_tbl3, fontsize = 10,  tbl_scale = FALSE, tbl_stub_pct = 0.3), gt_tbl3 |>
                     gt:: tab_options(table.font.size = "10pt",
                                      heading.subtitle.font.size = "10pt",
                                      heading.title.font.size = "10pt"
                     ))

  # scale true
  gt_tbl4 <- gt::exibble |>
    gt::gt() |> gt::cols_width(num ~ "10%", char ~ "3%")

  expect_identical(apply_scale(gt_tbl4, fontsize = 10, tbl_scale = TRUE, tbl_stub_pct = 0.3), scale_gt(gt_tbl4, tbl_stub_pct = 0.3)|>
                     gt:: tab_options(table.font.size = "10pt",
                                      heading.subtitle.font.size ="10pt",
                                      heading.title.font.size = "10pt",
                                      table.width = "100%"
                     ))
})

test_that("check_gt_widths works - hidden columns",{

  hidden_cols <- gt::exibble |>
    gt::gt() |>
    gt::cols_hide(num) |>
    gt::cols_width(
      num ~ "400.5%",
      char ~ "3%"
    )

 # should ignore the 400.5% and total should be <100
  expect_message(check_gt_widths(hidden_cols),"NOTE: Column widths total <100%. Resulting table may not respect document margins.")
  expect_equal(suppressMessages(check_gt_widths(hidden_cols)),list(tbl_scale = FALSE, table_width = "100%"))
})

test_that("apply_scale works - gt_group",{
  gt_tbl <- gt::exibble |>
    gt::gt()|>
    gt:: tab_options(table.font.size = "10pt",
                     heading.subtitle.font.size = "10pt",
                     heading.title.font.size = "10pt",
                     table.width = "100%"
    )

  # one table with bad column widths one is fine
  gt_group1 <- gt::gt_group(gt_tbl |> gt::cols_width(num ~ "400%", char ~ "3%"), gt_tbl)

  # expect first table to be auto rescaled, second table unchanged
  expected_group <- gt::gt_group(scale_gt(gt_tbl, tbl_stub_pct = 0.3), gt_tbl)

  expect_warning(apply_scale(gt_group1, fontsize = 10, tbl_scale = FALSE, tbl_stub_pct = 0.3),"Column widths must be add to =<100%, not 403%. Applying auto table scaling.")
  expect_identical(suppressWarnings(apply_scale(gt_group1, fontsize = 10, tbl_scale = FALSE, tbl_stub_pct = 0.3)), expected_group)

  # tbl_scale = TRUE
  gt_group2 <- gt::gt_group(gt_tbl, gt_tbl)
  expected_group2 <- gt::gt_group(scale_gt(gt_tbl, tbl_stub_pct = 0.3), scale_gt(gt_tbl, tbl_stub_pct = 0.3))

  expect_identical(apply_scale(gt_group2, fontsize = 10, tbl_scale = TRUE, tbl_stub_pct = 0.3), expected_group2)
})

test_that("apply_scale works - list", {
  gt_tbl <- gt::exibble |>
    gt::gt() |>
    gt::tab_options(
      table.font.size = "10pt",
      heading.subtitle.font.size = "10pt",
      heading.title.font.size = "10pt",
      table.width = "100%"
    )
  list_gt <- list(
    # one bad column widths
    gt_tbl |> gt::cols_width(num ~ "400%", char ~ "3%"),
    # one good column widths
    gt_tbl
  )

  # expect first table to be auto rescaled, second table unchanged
  expected_list_gt <- list(scale_gt(gt_tbl, tbl_stub_pct = 0.3), gt_tbl)

  expect_warning(
    apply_scale(list_gt, fontsize = 10, tbl_scale = FALSE, tbl_stub_pct = 0.3),
    "Column widths must be add to =<100%, not 403%. Applying auto table scaling."
  )
  expect_identical(
    suppressWarnings(apply_scale(
      list_gt,
      fontsize = 10,
      tbl_scale = FALSE,
      tbl_stub_pct = 0.3
    )),
    expected_list_gt
  )
  
  ggplot <- ggplot2::ggplot(ggplot2::mpg, ggplot2::aes(x = displ, y = hwy)) +
      ggplot2::geom_point()
  
  list_gt_ggplot <- list(
    # one bad column widths
    gt_tbl |> gt::cols_width(num ~ "400%", char ~ "3%"),
    # one good column widths
    gt_tbl,
    # ggplot
    ggplot
  )

  # expect first table to be auto rescaled, second table unchanged, ggplot unchanged
  expected_list_gt_ggplot <- list(
    scale_gt(gt_tbl, tbl_stub_pct = 0.3),
    gt_tbl,
    ggplot
  )


  expect_warning(
    apply_scale(list_gt_ggplot, fontsize = 10, tbl_scale = FALSE, tbl_stub_pct = 0.3),
    "Column widths must be add to =<100%, not 403%. Applying auto table scaling."
  )
  expect_identical(
    suppressWarnings(apply_scale(
      list_gt_ggplot,
      fontsize = 10,
      tbl_scale = FALSE,
      tbl_stub_pct = 0.3
    )),
    expected_list_gt_ggplot
  )
})

#' Prepare table, listing, figure object for inclusion in the template Rmd
#'
#' @param x docorator object containing info about the table, listing or figure
#' @param transform optional latex transformation function to apply to a gt latex string
#' @param ... additional args
#'
#' @return object to be included as-is in render engine
#' @export
#' @keywords internal
#'
#' @examples
#' docorator <- gt::exibble |>
#' gt::gt() |>
#' as_docorator(
#' display_name = "mytbl", footer = NULL,
#' save_object = FALSE)
#'
#' prepared_obj <- prep_obj_tex(docorator)
prep_obj_tex <- function (x, transform = NULL, ...) {
  UseMethod("prep_obj_tex", x$display)
}

#' @rdname prep_obj_tex
#' @export
#' @keywords internal
prep_obj_tex.default <- function(x, transform = NULL, ...) {
  x$display
}

#' @rdname prep_obj_tex
#' @export
#' @keywords internal
prep_obj_tex.character <- function(x, transform = NULL, ...) {
  cat(x$display)
}

#' @rdname prep_obj_tex
#' @export
#' @keywords internal
prep_obj_tex.PNG <- function(x, transform = NULL, ... ) {

  if (Sys.getenv("DOCORATOR_RENDER_ENGINE")=="qmd"){
    tmpdir <- "."
  } else {
    tmpdir <- tempdir()
  }

  # temporarily store png
  temp <- tempfile(fileext = ".png", tmpdir = tmpdir)
  png::writePNG(x$display$png, temp)
  knitr::include_graphics(path = temp)
}

#' @rdname prep_obj_tex
#' @export
#' @keywords internal
prep_obj_tex.gt_tbl <- function(x, transform = NULL, ...) {
  gt_to_tex(x$display, transform) |>
    cat()
}

#' convert gt_tbl object to latex
#' @noRd
gt_to_tex <- function(x, transform = NULL){

  if (Sys.getenv("DOCORATOR_RENDER_ENGINE")=="qmd"){

    if ("latex_use_longtable" %in% x$`_options`$parameter){
      x <- x |>
        gt::tab_options(
          latex.use_longtable = FALSE,
          latex.tbl.pos = "H"
        )
    }

    tbl_tex <- x |>
      gt::as_latex() |> as.character()

  } else {

    if ("latex_use_longtable" %in% x$`_options`$parameter){
      x <- x |>
        gt::tab_options(
          latex.use_longtable = TRUE
        )
    }

    tbl_tex <- x |>
      gt::as_latex() |> as.character()

    # add line for repeated headers if table breaks on multiple pages
    #  longtable only
    tbl_tex <- sub("\\\\midrule","\\\\midrule\\\\endhead",tbl_tex)
  }

  # apply optional latex transform
  if(!is.null(transform)){
    tbl_tex |> transform()
  }else{
    tbl_tex
  }
}

test_that("gt to latex conversion preserves formatting",{
  ard <- tfrmt::data_efficacy
  tf <- tfrmt::json_to_tfrmt(system.file("json_examples/tfrmt_efficacy.json", package = "tfrmt"))
  tbl <- tfrmt::print_to_gt(tf, ard)

  # table header
  preamble <- gt_to_tex(tbl) |>
              strsplit("\n") |>
    unlist()

  expect_snapshot(
    preamble[1:6]
  )
})

test_that("gt to latex conversion works with additional transformation function",{
  ard <- tfrmt::data_efficacy
  tf <- tfrmt::json_to_tfrmt(system.file("json_examples/tfrmt_efficacy.json", package = "tfrmt"))
  tbl <- tfrmt::print_to_gt(tf, ard)
  transform <- function(x){stringr::str_replace_all(x,"Placebo", "PLACEBO")}

  # table header no transform
  preamble <- gt_to_tex(tbl) |>
    strsplit("\n") |>
    unlist()


  # table header transform
  preamble2 <- gt_to_tex(tbl, transform = transform) |>
    strsplit("\n") |>
    unlist()

  expect_true(stringr::str_detect(preamble[5], "Placebo"))
  expect_false(stringr::str_detect(preamble[5], "PLACEBO"))
  expect_true(stringr::str_detect(preamble2[5], "PLACEBO"))
  expect_false(stringr::str_detect(preamble2[5], "Placebo"))

})

test_that("png objects are prepared correctly ",{
  png <- png_path(path = system.file("extdata/test_image.png", package = "docorator"))
  docorator <- as_docorator(png, display_name = "myfig", save_object = FALSE)
  knit_png <- prep_obj_tex(docorator)
  expect_true(inherits(knit_png,c("knit_image_paths", "knit_asis")))
})

#' Process geometry
#'
#' @param header Document header
#' @param footer Document footer
#' @param fontsize Font size for headers and footers
#' @param geometry Named list of geometry options
#'
#' @return geometry for the specified render engine
#' @noRd
geom_process <- function(
  header = NULL,
  footer = NULL,
  fontsize = 10,
  geometry = geom_set(),
  engine = "pdf"
) {
  switch(
    engine,
    pdf = geom_process_pdf(
      header = header,
      footer = footer,
      fontsize = fontsize,
      geometry = geometry
    ),
    docx = geom_process_docx(
      geometry = geometry
    )
  )
}

#' Process geometry for latex
#'
#' @param header Document header
#' @param footer Document footer
#' @param fontsize Font size for headers and footers
#' @param geometry Named list of geometry options
#'
#' @return Character string
#' @noRd
geom_process_pdf <- function(header, footer, fontsize, geometry) {
  if (is.null(geometry$headheight)) {
    geometry$headheight <- paste0(hf_height(header, fontsize), "pt")
  }
  if (is.null(geometry$footskip)) {
    geometry$footskip <- paste0(hf_height(footer, fontsize), "pt")
  }

  paste(names(geometry), geometry, sep = "=", collapse = ", ")
}

#' Process geometry for docx
#'
#' @param geometry Named list of geometry options
#'
#' @return Named list of geometry options with numeric values in inches
#' @noRd
geom_process_docx <- function(geometry) {
  # if geometry doesnt contain all required margins, add from geom_set
  if (!all(c("left", "right", "top", "bottom") %in% names(geometry))) {
    missing <- setdiff(c("left", "right", "top", "bottom"), names(geometry))

    geometry <- utils::modifyList(
      geom_set()[c("left", "right", "top", "bottom")],
      geometry
    )
    # note that defaults are being used
    cli::cli_text(
      "Geometry values for {.val {missing}} are not specified. Using defaults from geom_set()"
    )
  }

  # only want left right top bottom for docx geometry - the rest are not applicable
  geometry <- geometry[c("left", "right", "top", "bottom")]

  # the attributes here contain a unit in a text string ie "8in" - need to convert to numeric and flag if not inches
  lapply(geometry, function(x) {
    if (grepl("in", x)) {
      as.numeric(gsub("in", "", x))
      # if no characters are present assume it's a numeric value in inches and return as is
    } else if (!grepl("[a-zA-Z]", x)) {
      as.numeric(x)
    } else {
      inches <- as.numeric(gsub("[a-zA-Z]", "", x))
      cli::cli_text(
        "Geometry value {.val {x}} changed to {.val {inches}} inches for docx output"
      )
      inches
    }
  })
}

#' Set document geometry defaults
#' @param ... Series of named value pairs for latex geometry options
#'
#' @details Type geom_set() in console to view package defaults. Use of the
#'   function will add to the defaults and/or override included defaults of the
#'   same name. For values that are `NULL`, such as for `headheight` and
#'   `footskip`, the values will be calculated automatically based on the number
#'   of header and/or footer lines. For all geometry settings, reference the
#'   documentation here: https://texdoc.org/serve/geometry.pdf/0
#'
#' @return Named list
#' @export
#'
#' @examples
#' # view defaults
#' geom_set()
#'
#' # Update the defaults
#' geom_set(left="0.5in", right="0.5in")
#'
#' # add new defaults
#' geom_set(paper = "legalpaper")
#'
geom_set <- function(...) {
  args <- list(...)

  # default list
  geom_list <- list(
    paperheight = "8.5in",
    paperwidth = "11in",
    left = "1in",
    right = "1in",
    top = "1.25in",
    bottom = "1.25in",
    headsep = "10pt",
    includehead = TRUE,
    includefoot = TRUE,
    headheight = NULL,
    footskip = NULL
  )

  # overwrite any user-specified defaults
  idx_drop <- which(names(geom_list) %in% names(args))

  if (length(idx_drop) > 0) {
    geom_list <- geom_list[-idx_drop]
  }
  c(geom_list, args)
}

#' Path of program
#'
#' @param filename Name of output file
#' @param path program path
#'
#' @return character string
#' @export
#' @examples
#' \dontrun{
#'  doc_path(filename = "my_tbl.pdf", path = NULL)
#'}
doc_path <- function(filename = NULL, path = NULL){
  path <- path %||% "."
  doc_path <- thisfile(filename, path)
  if(length(doc_path)==0){
    return(NA)
  }
  doc_path
}

#' Date and time of program run
#'
#' @return character string
#' @export
#' @examples
#' \donttest{
#'  doc_datetime()
#'}
doc_datetime <- function(){
  as.POSIXct(Sys.time(), tz = "UTC") |>
    format("%d%B%Y %H:%M")
}

#' Automatic page numbering
#'
#' @return character string containing latex code
#' @export
#' @examples
#' doc_pagenum()
#'
doc_pagenum <- function(){
  "_DOCORATOR_PAGE_PLACEHOLDER_"
}


#' Functions adapted from whereami package, formerly in rprojroot
#' https://github.com/yonicd/whereami/blob/master/R/thisfile.R
#'
#' @noRd
thisfile <- function(filename, path) {
  if (!is.null(res <- thisfile_source())) {
    res
  } else if (!is.null(res <- thisfile_r())) {
    res
  } else if (!is.null(res <- thisfile_rscript())) {
    res
  } else if (Sys.getenv("RSTUDIO") == "1" &&
             !is.null(rstudioapi::getSourceEditorContext()$path) &&
             !rstudioapi::getSourceEditorContext()$path==""){
    normalizePath(rstudioapi::getSourceEditorContext()$path, winslash = "/")
  } else if (!is.null(filename)){
    paste0(
      normalizePath(path, winslash = "/"), "/", tools::file_path_sans_ext(filename), ".R"
    )
  } else {
    NULL
  }
}

#' @noRd
thisfile_source <- function() {
  for (i in -(1:sys.nframe())) {
    if (identical(args(sys.function(i)), args(base::source))) {
      return(normalizePath(sys.frame(i)$ofile,winslash = '/'))
    }
  }

  NULL
}

#' @noRd
thisfile_r <- function() {
  cmd_args <- commandArgs(trailingOnly = FALSE)
  if (!grepl("^R(?:|term)(?:|[.]exe)$", basename(cmd_args[[1L]]), ignore.case = TRUE)) {
    return(NULL)
  }

  cmd_args_trailing <- commandArgs(trailingOnly = TRUE)
  leading_idx <-
    seq.int(from = 1, length.out = length(cmd_args) - length(cmd_args_trailing))
  cmd_args <- cmd_args[leading_idx]
  file_idx <- c(which(cmd_args == "-f") + 1, which(grepl("^--file=", cmd_args)))
  res <- gsub("^(?:|--file=)(.*)$", "\\1", cmd_args[file_idx])

  # If multiple --file arguments are given, R uses the last one
  res <- utils::tail(res[res != ""], 1)
  if (length(res) > 0) {
    return(res)
  }

  NULL
}

#' @noRd
thisfile_rscript <- function() {
  cmd_args <- commandArgs(trailingOnly = FALSE)
  if (!grepl("^R(?:term|script)(?:|[.]exe)$", basename(cmd_args[[1L]]), ignore.case = TRUE)) {
    return(NULL)
  }

  cmd_args_trailing <- commandArgs(trailingOnly = TRUE)
  leading_idx <-
    seq.int(from = 1, length.out = length(cmd_args) - length(cmd_args_trailing))
  cmd_args <- cmd_args[leading_idx]
  res <- gsub("^(?:--file=(.*)|.*)$", "\\1", cmd_args)

  # If multiple --file arguments are given, R uses the last one
  res <- utils::tail(res[res != ""], 1)
  if (length(res) > 0) {
    return(res)
  }

  NULL
}

#' Path of png file
#'
#' @param path path to png
#'
#' @return object with png attribute
#' @export
#' @examples
#' \dontrun{
#' png_path <- png_path(path = "path_to_my_png.png")
#' }
png_path <- function(path = NULL){
  # path provided, is valid, is png
  if(is.null(path) | !file.exists(path) | toupper(tools::file_ext(path)) != "PNG"){
    cli::cli_abort("Please provide a valid path to a PNG file.",
              call = rlang::caller_env())
  }

  png <- png::readPNG(path)

  structure(list(
    png = png),
    class = "PNG")

}

#' Create prep_obj_tex code chunks
#' Taken from https://gist.github.com/MichaelJW/b4a3dd999a47b137d12f42a8f7562b11
#'
#' @param x docorator object
#' @param id id number for the chunk
#' @param transform optional latex transformation function to apply to a gt latex string
#'
#' @return printed code chunk to be included as-is in the render engine
#' @noRd
#' @examples
#' \dontrun{
#' docorator <- gt::exibble |>
#'   gt::gt() |>
#'   as_docorator(save_object = FALSE)
#' create_chunk(docorator, transform = NULL)
#' }
create_chunk <- function(x, id = 1, transform) {
  deparsed <- paste0(deparse(
    function() {
      prep_obj_tex(x, transform)
    }
  ), collapse = '')

  # set figure output args
  if(isTRUE(x$convert_ggplot)){
    size_args <- paste0("out.height='", x$fig_dim[1], "in', out.width='", x$fig_dim[2], "in',")
  } else {
    size_args <- ""
  }

  new_chunk <- paste0("
  `","``{r new_chunk", id, ", fig.height=", x$fig_dim[1], ", fig.width=", x$fig_dim[2], ",", size_args, " echo=FALSE, results='asis', output='asis'}",
                       "\n(",
                       deparsed
                       , ")()",
                       "\n  `","``
   ")


  cat(knitr::knit(text = knitr::knit_expand(text = new_chunk), quiet = TRUE))
}

#' Create code chunks
#'
#' @param x docorator object
#' @param transform optional latex transformation function to apply to a gt latex string
#'
#' @return printed code chunk(s) to be included as-is in the render engine
#' @export
#' @keywords internal
#' @examples
#' \dontrun{
#' docorator <- gt::exibble |>
#'   gt::gt() |>
#'   as_docorator(save_object = FALSE)
#' create_chunks_all(docorator, transform = NULL)
#' }
create_chunks_all <- function(x, transform) {
  if (!inherits(x, "docorator")) {
    cli::cli_abort("The {.arg {rlang::caller_arg(x)}} argument must be class docorator, not {.obj_type_friendly {x}}. See documentation for `as_docorator`.",
                   call = rlang::caller_env())
  }

  display <- x$display

  if(identical(class(display), "list")) {
    for (i in 1:length(display)) {
      new_docorator <- x
      new_docorator$display <- display[[i]]
      create_chunk(new_docorator,i,transform)
      # no pagebreak for the last chunk, otherwise it will create an extra blank page at the end of the document
      if (i != length(display)) {
        cat("\\pagebreak\n")
      }
    }
  }  else {
    create_chunk(x, 1, transform)
  }
}

#' Check package versions for docorator object are the same as loaded
#'
#' @param x docorator object
#' @keywords internal
check_pkg_version <- function(x) {

  if (rlang::inherits_any(x$display, c("gt_tbl", "gt_group"))) {
    pkg <- "gt"
  }else if (inherits(x$display,"ggplot")) {
    pkg <- "ggplot2"
  }else{
    return()
  }

 package_list <- append(x$session_info$otherPkgs,x$session_info$loadedOnly)
 pkg_version <- package_list[[pkg]][["Version"]]

 # get current version if it exists
 current_version = as.character(utils::packageVersion(pkg))

 if(utils::compareVersion(current_version, pkg_version)!= 0){
   cli::cli_text("Note: docorator object was created with {.pkg {pkg} {pkg_version}}. You are now running {.pkg {pkg} {current_version}}. There may be issues rendering your document.")}
}

# convert any gt_groups to a list, flatten any lists containing gt_groups to a list of gt_tbls
#' @param x object containing the display.
#' @return object containing the display, converted to list if appropriate
#' @noRd
#' @keywords internal
convert_list_displays <- function(x) {
  if (inherits(x, "gt_group")) {
    x <- lapply(seq_len(nrow(x$gt_tbls)), function(i) {
      gt::grp_pull(x, i)
    })
  }

  # if a list contains gt_group, flatten to gt_tbls
  if (identical(class(x), "list")) {
    x <- lapply(x, function(j) {
      if (inherits(j, "gt_group")) {
        lapply(seq_len(nrow(j$gt_tbls)), function(i) {
          gt::grp_pull(j, i)
        })
        # lists of lists etc not supported
      } else if (identical(class(j), "list")) {
        cli::cli_abort(
          "Nested lists are not supported. See documentation for supported display types.",
          call = rlang::caller_env()
        )
      } else {
        list(j)
      }
    })
    x <- unlist(x, recursive = FALSE)
  }
  x
}


#' convert all the ggplots in a display or list of displays to png objects
#' @param x display object
#' @param fig_dim figure dimensions
#' @param convert_ggplot Boolean for whether or not to convert ggplot objects to PNG files to preserve scaling for all render engines. Defaults to TRUE.
#' @return display object with ggplots converted to png objects
#' @noRd
#' @keywords internal
gg_to_PNG <- function(x, fig_dim = c(5,8), convert_ggplot = TRUE) {
  # if not a list convert to one 
  if(!identical(class(x), "list")){
    x <- list(x)
  }

  x <- lapply(
    seq_along(x),
    FUN = function(i) {
      if (inherits(x[[i]], "ggplot") && convert_ggplot) {
        filepath <- gg_to_image(x[[i]], fig_dim, path = tempdir())
        x[[i]] <- png_path(filepath)
      }
      x[[i]]
    }
  )

  # if the list has only one element, return the element instead of a list
  if(length(x) == 1){
    x <- x[[1]]
  }

  x
}


#' prep display for rendering, convert display type where appropriate, apply scaling
#' @param x display object
#' @param fig_dim figure dimensions
#' @param convert_ggplot Boolean for whether or not to convert ggplot objects to PNG files to preserve scaling for all render engines. Defaults to TRUE.
#' @param fontsize font size for scaling
#' @param tbl_scale Boolean for whether or not to scale tables
#' @param tbl_stub_pct percentage of table stub to scale
#' @return display object ready for rendering
#' @noRd
#' @keywords internal
prep_display <- function(x, fig_dim = c(5,8), convert_ggplot = TRUE, fontsize = 10, tbl_scale = TRUE, tbl_stub_pct = 0.3) {
  
  # convert gt_groups if appropriate
  x <- convert_list_displays(x)
  # convert ggplot to png if appropriate
  x <- gg_to_PNG(x, fig_dim, convert_ggplot)

  # notify user about any possible scaling issues
  x <- apply_scale(
    x,
    fontsize = fontsize,
    tbl_scale = tbl_scale,
    tbl_stub_pct = tbl_stub_pct
  )
  
  x
}

test_that("create PNG object",{
  png_obj <- png_path(path = system.file("extdata/test_image.png", package = "docorator"))

  expect_equal(class(png_obj), "PNG")
})

test_that("markdown chunks are created correctly",{

  skip_if_not(interactive())

  png_obj <- png_path(path = system.file("extdata/test_image.png", package = "docorator"))
  docorator <- as_docorator(png_obj, display_name = "myfig", save_object = FALSE)
  chunk_png <- create_chunks_all(x = docorator, transform = NULL) |> capture.output()
  # avoiding snapshot because of temp directory + mardown header
  expect_true(any(grepl('<div class=\"figure\">', chunk_png)))

  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    ) |>
    apply_scale(fontsize = 10,
                tbl_scale = TRUE,
                tbl_stub_pct = 0.5)
  docorator <- as_docorator(my_gt, display_name = "mytbl", save_object = FALSE)
  chunk_gt <- create_chunks_all(x = docorator, transform = NULL) |> capture.output()
  expect_snapshot(chunk_gt)

  my_gt_group <- gt::gt_group(my_gt,my_gt)
  docorator <- as_docorator(my_gt_group, display_name = "mytbl", save_object = FALSE)
  chunk_gt_group <- create_chunks_all(x = docorator, transform = NULL) |> capture.output()
  expect_snapshot(chunk_gt_group)

  my_gt_group_caption <- gt::gt_group(my_gt |> gt::tab_caption("test"),my_gt |> gt::tab_caption("test"))
  docorator <- as_docorator(my_gt_group_caption, display_name = "mytbl", save_object = FALSE)
  chunk_gt_group_caption <- create_chunks_all(x = docorator, transform = NULL) |> capture.output()
  expect_snapshot(chunk_gt_group_caption)
  expect_true(any(grepl('new_chunk1', chunk_gt_group_caption)))
  expect_true(any(grepl('new_chunk2', chunk_gt_group_caption)))

  # list of ggplot2 - tempdir to stop creation of figure folder
  withr::with_tempdir({

  ggplot1 <- ggplot2::ggplot(data = mtcars, ggplot2::aes(y=cyl, x=mpg)) +
    ggplot2::geom_point() +
    ggplot2::labs(title = "title1", subtitle = "subtitle1", tag = "tag1", caption = "footnote1")
  ggplot2 <- ggplot2::ggplot(data = mtcars, ggplot2::aes(x=cyl, y=mpg)) +
    ggplot2::geom_point() +
    ggplot2::labs(title = "title2", subtitle = "subtitle2", tag = "tag2", caption = "footnote2")

  list_of_ggplots <- list(ggplot1, ggplot2)
  docorator <- as_docorator(list_of_ggplots, display_name = "my_plot_list", save_object = FALSE)
  chunk_list <- create_chunks_all(x = docorator, transform = NULL) |> capture.output()
  expect_true(any(grepl('new_chunk1', chunk_list)))
  expect_true(any(grepl('new_chunk2', chunk_list)))

})

})

test_that("package version messages are printed correctly",{

  skip_on_cran()
  skip_on_ci()

  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    )

  docorator <- as_docorator(my_gt,
                            display_name = "mytbl",
                            save_object = FALSE)

  current_version <- utils::packageVersion("gt")

  # amend the sessionInfo

  # non-existent gt version
  old_gt <- docorator
  old_gt$session_info$loadedOnly$gt$Version <- "0.0.1234"
  suppressMessages({
    expected_message <- cli::cli_text("Note: docorator object was created with {.pkg gt 0.0.1234}. You are now running {.pkg gt {current_version}}. There may be issues rendering your document.")
    expect_equal( check_pkg_version(old_gt), expected_message)
  })

  # empty other packages still renders message
  old_gt$session_info$otherPkgs <- NULL
  suppressMessages({
    expect_equal( check_pkg_version(old_gt), expected_message)
  })

})

test_that("convert_list_displays works", {
  gt_tbl <- gt::exibble |> gt::gt()
  gt_tbl2 <- gt::exibble |>
    dplyr::slice_head(n = 5) |>
    gt::gt() |>
    gt::tab_header(title = "title2")
  gt_group <- gt::gt_group(gt_tbl, gt_tbl2)

  gt_list <- convert_list_displays(gt_group)

  expect_equal(class(gt_list), "list")
  expect_equal(length(gt_list), 2)
  expect_true(inherits(gt_list[[1]], "gt_tbl"))
  expect_true(inherits(gt_list[[2]], "gt_tbl"))

  # display is not a gt_group stays the same

  no_list <- convert_list_displays(gt_tbl)
  expect_equal(no_list, gt_tbl)

  # list with gt_groups gets flattened to list with gt_tbls
  plot <- ggplot2::ggplot(mtcars) +
    ggplot2::aes(x = disp, y = mpg) +
    ggplot2::geom_point()
  list <- list(plot, gt_group, gt_tbl2)

  list_flattened <- convert_list_displays(list)

  expect_equal(class(list_flattened), "list")
  # should have 3 gt_tbls and 1 ggplot
  expect_equal(length(list_flattened), 4)
  expect_true(inherits(list_flattened[[1]], "ggplot"))
  expect_true(inherits(list_flattened[[2]], "gt_tbl"))
  expect_true(inherits(list_flattened[[3]], "gt_tbl"))
  expect_true(inherits(list_flattened[[4]], "gt_tbl"))

  # nested lists are not supported
  nested_list <- list(gt_tbl, list(gt_tbl2, plot))

  expect_error(
    convert_list_displays(nested_list),
    "Nested lists are not supported. See documentation for supported display types."
  )
})

test_that("gg_to_png works", {
  # single ggplot object
  gg <- ggplot2::ggplot(mtcars, ggplot2::aes(x = wt, y = mpg)) +
    ggplot2::geom_point()

  # list of ggplot objects
  list_gg <- list(
    ggplot2::ggplot(mtcars, ggplot2::aes(x = wt, y = mpg)) + ggplot2::geom_point(),
    ggplot2::ggplot(mtcars, ggplot2::aes(x = hp, y = mpg)) + ggplot2::geom_point()
  )

  # list of gt tables and ggplot objects
  list_mixed <- list(
    gt::gt(mtcars),
    ggplot2::ggplot(mtcars, ggplot2::aes(x = wt, y = mpg)) + ggplot2::geom_point()
  )

  list_no_gg <- list(
    gt::gt(mtcars),
    gt::gt(mtcars)
  )

  png <- gg_to_PNG(gg)
  expect_equal(class(png), "PNG")

  list_png <- gg_to_PNG(list_gg)
  expect_equal(class(list_png), "list")
  expect_equal(length(list_png), 2)
  expect_true(all(sapply(list_png, function(x) inherits(x, "PNG"))))

  list_mixed_png <- gg_to_PNG(list_mixed)
  expect_equal(class(list_mixed_png), "list")
  expect_equal(length(list_mixed_png), 2)
  expect_true(inherits(list_mixed_png[[1]], "gt_tbl"))
  expect_true(inherits(list_mixed_png[[2]], "PNG"))

  # if convert_ggplot is FALSE, ggplot objects should not be converted
  list_mixed_no_convert <- gg_to_PNG(list_mixed, convert_ggplot = FALSE)
  expect_equal(class(list_mixed_no_convert), "list")
  expect_equal(length(list_mixed_no_convert), 2)
  expect_true(inherits(list_mixed_no_convert[[1]], "gt_tbl"))
  expect_true(inherits(list_mixed_no_convert[[2]], "ggplot"))

  # no ggplots present should not change
  list_no_gg_png <- gg_to_PNG(list_no_gg)
  expect_identical(list_no_gg, list_no_gg_png)

})

test_that("prep_display works", {
  # single gt table - only scaling should happen
  gt_tbl <- gt::gt(mtcars)
  scaled_gt_tbl <- apply_scale(gt_tbl, fontsize = 10, tbl_scale = TRUE, tbl_stub_pct = 0.3)
  prep_gt <- prep_display(gt_tbl)
  expect_identical(prep_gt, scaled_gt_tbl)

  # single ggplot - only conversion should happen
  gg <- ggplot2::ggplot(mtcars, ggplot2::aes(x = wt, y = mpg)) +
    ggplot2::geom_point()
  prep_gg <- prep_display(gg)
  expect_true(inherits(prep_gg, "PNG"))
  
  list_mixed <- list(
    # should be scaled
    gt_tbl,
    # should be converted to PNG
    ggplot2::ggplot(mtcars, ggplot2::aes(x = wt, y = mpg)) + ggplot2::geom_point(),
    # should be converted to gt_tbls and scaled
    gt::gt_group(gt_tbl, gt_tbl)
  )
  prep_list_mixed <- prep_display(list_mixed)
  expect_equal(class(prep_list_mixed), "list")
  expect_equal(length(prep_list_mixed), 4)
  expect_identical(prep_list_mixed[[1]], scaled_gt_tbl)
  expect_true(inherits(prep_list_mixed[[2]], "PNG"))
  expect_identical(prep_list_mixed[[3]], scaled_gt_tbl)
  expect_identical(prep_list_mixed[[4]], scaled_gt_tbl)

})

#'  Construct document header
#'
#'  Define document header through a series of `fancyrow`s. Each row represents
#'  a new line in the header with options for positioning text at left, center,
#'  and/or right positions.
#'
#' @param ... Series of objects of class `fancyrow`. Each entry represents a new
#'   row in the document header.
#'
#' @return Character string containing latex code for the `fancyhead` entries as
#'   part of the `fancyhdr` latex framework
#'
#' @export
#' @examples
#' fancyhead(
#'  fancyrow(left = "Protocol: 12345", right = doc_pagenum()),
#'  fancyrow(center = "Demographic Summary")
#' )
#'
fancyhead <- function(...) {
  dots <- list(...)
  if (rlang::is_empty(dots)) {
    return(NULL)
  }
  check_fancyrows(dots)

  structure(
    dots,
    class = c("fancyhead", "fancyhdr")
  )
}

#'  Construct document footer
#'
#'  Define document footer through a series of `fancyrow`s. Each row represents
#'  a new line in the footer with options for positioning text at left, center,
#'  and/or right positions.
#'
#' @param ... Series of objects of class `fancyrow`. Each entry represents a new
#'   row in the document footer.
#'
#' @return Character string containing latex code for the `fancyfoot` entries as
#'   part of the `fancyhdr` latex framework
#'
#' @export
#' @examples
#' fancyfoot(
#'  fancyrow(left = "My first footnote", right = doc_datetime())
#' )
#'
fancyfoot <- function(...) {
  dots <- list(...)
  if (rlang::is_empty(dots)) {
    return(NULL)
  }
  check_fancyrows(dots)

  structure(
    dots,
    class = c("fancyfoot", "fancyhdr")
  )
}

#'  Construct document header row
#'
#'  Define a single row in the document header/footer. Each row represents
#'  a single line of text, with options for positioning text at left, center,
#'  and/or right.
#'
#' @param left Character string to be aligned to the left side of the row.
#' @param center Character string to be aligned to the center of the row.
#' @param right Character string to be aligned to the right side of the row.
#'
#' @return Object of class `fancyrow`
#' @export
#' @examples
#' fancyrow(left = "Left most text", right = "Right most text")
#'
#' fancyrow(center = "Just text in the center")
#'
#' fancyrow(left = "All", center = "Three", right = "Positions filled")
fancyrow <- function(left = NA, center = NA, right = NA) {
  # Check that all arguments are single strings
  check_fancyrow_string(left = left, center = center, right = right)

  structure(
    list(left = left, center = center, right = right),
    class = "fancyrow"
  )
}

#' Check that all arguments in fancyrow() are single strings or NA
#' @param left Character string to be aligned to the left side of the row.
#' @param center Character string to be aligned to the center of the row.
#' @param right Character string to be aligned to the right side of the row.
#' @noRd
#' @keywords internal
#'
check_fancyrow_string <- function(left = NA, center = NA, right = NA) {
  # Create list of arguments for iteration
  args <- list(left = left, center = center, right = right)
  errors <- character(0)

  for (name in names(args)) {
    value <- args[[name]]
    # Check if arguments are a vector of size greater than 1
    if (length(value) > 1) {
      errors <- c(
        errors,
        cli::format_message(
          "{.arg {name}} must be a single value, but has a length of {length(value)}."
        )
      )
      next
    }

    # Check if arguments contain any non-string values if not NA
    if (!is.na(value) && !is.character(value)) {
      errors <- c(
        errors,
        cli::format_message(
          "{.arg {name}} must be a character string or NA, but is {class(value)}."
        )
      )
    }
  }

  if (length(errors) > 0) {
    cli::cli_abort(c("Invalid input in fancyrow():", errors))
  }

  invisible(TRUE)
}

#' Check that header and footer args are of class `fancyhdr`
#' @param x object passed to `as_docorator`
#' @param chr_ok Whether to accept character vectors as input
#' @noRd
#'
check_fancyhdr <- function(x, chr_ok = FALSE) {
  if (rlang::is_empty(x)) {
    return(invisible(x))
  }

  class_ok <- "fancyhdr"
  if (chr_ok) {
    class_ok <- c("fancyhdr", "character")
  }

  if (!inherits(x, class_ok)) {
    cli::cli_abort(
      "The {.arg {rlang::caller_arg(x)}} argument must be class {.cls {class_ok}} or empty, not {.obj_type_friendly {x}}.",
      call = rlang::caller_env()
    )
  }
  invisible(x)
}

#' Check that all entries are class `fancyrow`
#'
#' @param x List of entries passed to `fancyheader` or `fancyfooter`
#'
#' @noRd
check_fancyrows <- function(x) {
  for (idx in seq_along(x)) {
    if (!inherits(x[[idx]], "fancyrow")) {
      cli::cli_abort(
        "Entry number {idx} is not an object of class `fancyrow`.",
        call = rlang::caller_env()
      )
    }
  }
}

#' Convert `fancyrow` to tibble
#'
#' @param x Object of class `fancyrow`
#'
#' @return tibble
#' @noRd
as_tibble_fancyrow <- function(x, ...) {
  dplyr::tibble(left = x$left, center = x$center, right = x$right)
}

#' Process headers/footers
#'
#' @param x header or footer
#' @param escape_latex Boolean to escape latex in header/footer
#' @param engine render engine to dictate how the headers and footers should be constructed
#'
#' @return character string containing headers and footers latex code
#' @export
#' @keywords internal
#'
#' @examples
#' header <- fancyhead(
#' fancyrow(left = "Protocol: 12345", right = doc_pagenum()),
#' fancyrow(center = "Demographic Summary"))
#'
#' hf_process(header)
#'
hf_process <- function(x, escape_latex = TRUE, fontsize = 10, engine = "pdf") {
  engine <- match.arg(engine, choices = c("pdf", "docx"))
  UseMethod("hf_process", x)
}

#' @export
#' @rdname hf_process
#' @keywords internal
hf_process.default <- function(
  x,
  escape_latex = TRUE,
  fontsize = 10,
  engine = "pdf"
) {
  if (is.null(x)) {
    return(NULL)
  }
  x
}

#' @export
#' @rdname hf_process
#' @keywords internal
hf_process.character <- function(
  x,
  escape_latex = TRUE,
  fontsize = 10,
  engine = "pdf"
) {
  cli::cli_alert_info(
    "Coercing `header` from {.cls {'character'}} to {.cls {'fancyhead'}} with {length(x)} row{?s}"
  )
  lapply(x, fancyrow) |>
    process_rows(
      type = "head",
      escape_latex = escape_latex,
      fontsize = fontsize,
      engine = engine
    )
}

#' @export
#' @rdname hf_process
#' @keywords internal
hf_process.fancyhead <- function(
  x,
  escape_latex = TRUE,
  fontsize = 10,
  engine = "pdf"
) {
  process_rows(
    x,
    type = "head",
    escape_latex = escape_latex,
    fontsize = fontsize,
    engine = engine
  )
}

#' @export
#' @rdname hf_process
#' @keywords internal
hf_process.fancyfoot <- function(
  x,
  escape_latex = TRUE,
  fontsize = fontsize,
  engine = "pdf"
) {
  process_rows(
    x,
    type = "foot",
    escape_latex = escape_latex,
    fontsize = fontsize,
    engine = engine
  )
}

#' Process list of `fancyrow` objects for different render engines
#'
#' @param x list of `fancyrow` objects
#' @param type Text positioning in the header (`head`) or footer (`foot`) of
#'   document. Defaults to `head`.
#' @inheritParams hf_process
#'
#' @return processed headers and footers for the specified render engine
#' @noRd
process_rows <- function(
  x,
  type = c("head", "foot"),
  escape_latex = TRUE,
  fontsize = 10,
  engine = "pdf"
) {
  type <- match.arg(type)

  x_df <- purrr::map_dfr(x, as_tibble_fancyrow)

  switch(
    engine,
    pdf = process_rows_pdf(x_df, type = type, escape_latex = escape_latex),
    docx = process_rows_docx(x, fontsize = fontsize)
  )
}


#' Process list of `fancyrow` objects into character string containing latex code
#'
#' @param x list of `fancyrow` objects
#' @param type Text positioning in the header (`head`) or footer (`foot`) of
#'   document. Defaults to `head`.
#' @inheritParams hf_process
#'
#' @return Character string
#' @noRd
process_rows_pdf <- function(x, type = c("head", "foot"), escape_latex = TRUE) {
  if (isTRUE(escape_latex)) {
    x <- x |>
      dplyr::mutate(
        # escape latex characters
        dplyr::across(dplyr::everything(), hf_escape)
      )
  }

  x_df <- x |>
    dplyr::mutate(
      dplyr::across(dplyr::everything(), \(x) {
        # replace the page number placeholder with the latex code for page numbering
        stringr::str_replace_all(
          x,
          stringr::fixed("\\_DOCORATOR\\_PAGE\\_PLACEHOLDER\\_"),
          "Page \\thepage\\ of \\pageref*{LastPage}"
        ) |>
        as.character() |>
        tidyr::replace_na("\\phantom{}")
      })
    ) |>
    dplyr::summarise(
      dplyr::across(
        dplyr::everything(),
        \(x) {
          x_braces <- paste0("{", x, "}")
          paste(x_braces, collapse = "\\\\")
        }
      )
    )

  paste0(
    "\\fancy",
    type,
    "[L]{\\begin{tabular}[b]{@{}l@{}}",
    x_df$left,
    "\\end{tabular}}",
    "\\fancy",
    type,
    "[C]{\\begin{tabular}[b]{@{}c@{}}",
    x_df$center,
    "\\end{tabular}}",
    "\\fancy",
    type,
    "[R]{\\begin{tabular}[b]{@{}r@{}}",
    x_df$right,
    "\\end{tabular}}"
  )
}

#' Process list of `fancyrow` objects into `fpar` objects for Word document headers and footers
#'
#' @param x list of `fancyrow` objects
#' @param fontsize Document font size (pt)
#' @inheritParams hf_process
#'
#' @return list of `fpar` objects for Word document headers and footers
#' @noRd
process_rows_docx <- function(x, fontsize = 10) {
  # set header/footer positions
  header_footer_tabs <- officer::fp_tabs(
    officer::fp_tab(pos = 4.5, style = "center"),
    officer::fp_tab(pos = 10, style = "right")
  )

  # font property
  font <- officer::fp_text(font.size = fontsize)

  # create headers from docorator
  fpar_list <- lapply(x, function(row) {
    headers <- c(
      make_hf_run(row$left, font),
      list(officer::run_tab()),
      make_hf_run(row$center, font),
      list(officer::run_tab()),
      make_hf_run(row$right, font)
    )

    do.call(
      officer::fpar,
      c(headers, list(fp_p = officer::fp_par(tabs = header_footer_tabs)))
    )
  })

  fpar_list
}

#' Calculate desired header or footer height for the document

#' @param x header or footer
#' @param fontsize Document font size (pt)
#'
#' @return Numeric value
#' @keywords internal
#' @export
#' @examples
#' header <- fancyhead(
#' fancyrow(left = "Protocol: 12345", right = doc_pagenum()),
#' fancyrow(center = "Demographic Summary"))
#'
#' hf_height(header, 10)
#'
hf_height <- function(x, fontsize) {
  UseMethod("hf_height", x)
}

#' @export
#' @rdname hf_height
#' @keywords internal
hf_height.default <- function(x, fontsize) {
  ceiling(fontsize * 1.2)
}

#' @export
#' @rdname hf_height
#' @keywords internal
hf_height.fancyhdr <- function(x, fontsize) {
  if (is.null(x)) {
    return(0)
  }

  # find # of rows
  num_rows <- purrr::map_dfr(x, as_tibble_fancyrow) |> nrow()

  # rough estimate but should be sufficient to avoid the fancyhdr warning
  ceiling(num_rows * fontsize * 1.2)
}

#' escape the latex characters, but keep the output of doc_pagenum unescaped
#' @param x text string to be escaped
#' @noRd
hf_escape <- function(x) {
  latex_text <- gt::escape_latex(x, unicode_conversion = TRUE)

  # keep the display page number unescaped
  page_num_pattern <- c(
    "\\\\textbackslash\\{\\}thepage\\\\textbackslash\\{\\}" = "\\\\thepage\\\\",
    "\\\\textbackslash\\{\\}pageref\\*\\\\\\{LastPage\\\\\\}" = "\\\\pageref*{LastPage}"
  )
  escaped_full <- latex_text |>
    stringr::str_replace_all(page_num_pattern)
}


#' Wrap `fancyrow`s
#'
#' @param x object with `fancyrow`s to wrap
#' @param chars number of characters to wrap on
#'
#' @return `docorator` object with wrapped headers and footers
#' @noRd
#' @keywords internal
#'
#' @examples
#' header <- fancyhead(
#' fancyrow(left = "Protocol: 12345"),
#' fancyrow(center = "Table 1.1 Demographic Summary"))
#'
#' fancywrap(header, 5)
#'
fancywrap <- function(x, chars) {
  UseMethod("fancywrap", x)
}

#' @rdname fancywrap
#' @export
#' @noRd
fancywrap.default <- function(x, chars) {
  if (is.null(x)) {
    return(NULL)
  }
  x
}

#' @rdname fancywrap
#' @export
#' @noRd
fancywrap.fancyrow <- function(x, chars){

  # check chars is numeric length 1 
  if(!is.numeric(chars) || length(chars) != 1){
    cli::cli_abort("The {.arg chars} argument must be a single numeric value, but is {.obj_type_friendly {chars}} length {length(chars)}.",
              call = rlang::caller_env())
  }

  # which elements have strings in them
  str_to_wrap <- which(!is.na(x))
  if (length(str_to_wrap) != 1) {
    # check if the total of the elements are too long for the page
    total_length <- sum(nchar(x[str_to_wrap]))
    total_string <- paste(x[str_to_wrap], collapse = ", ")
    if (total_length > chars) {
      cli::cli_text(
        "Note: Text is only wrapped for `fancyrows` with one element. Your fancyrow text {.arg {total_string}} is over the character limit of {.arg {chars}} and could overrun the page margins."
      )
    }
    wrapped_rows <- list(x)
  } else {
    row <- x[[str_to_wrap]]
    # let stringi do the hard part of the string wrapping
    rows <- stringi::stri_wrap(row, chars, whitespace_only = TRUE)

    wrapped_rows <- lapply(rows, function(x) {
      args <- list(x)
      names(args) <- names(str_to_wrap)
      do.call(fancyrow, args)
    })
  }

  wrapped_rows
}

#' @rdname fancywrap
#' @export
#' @noRd
fancywrap.fancyhead <- function(x, chars) {
  wrapped_headers <- lapply(x, function(row) {
    fancywrap(row, chars)
  })

  do.call(fancyhead, unlist(wrapped_headers, recursive = FALSE))
}

#' @rdname fancywrap
#' @export
#' @noRd
fancywrap.fancyfoot <- function(x, chars) {
  wrapped_footers <- lapply(x, function(row) {
    fancywrap(row, chars)
  })

  do.call(fancyfoot, unlist(wrapped_footers, recursive = FALSE))
}


#' @rdname fancywrap
#' @export
#' @noRd
fancywrap.docorator <- function(x, chars = NULL){

  # calculate if not provided
  if(is.null(chars)){
    # width in pts is roughly 50% font size
    # 1 inch 72 points
    # 11 inch page without margins, 9 inches
    # per page roughly 630 points
    # character width
    ch_width <- x$fontsize * 0.5
    chars <- floor(630 / ch_width)
  }


  if(!is.null(x$header)){
    x$header <- fancywrap(x$header, chars = chars)
  }

  if(!is.null(x$footer)){
    x$footer <- fancywrap(x$footer, chars = chars)
  }

  x
}

#' Build a header/footer run, substituting the page number placeholder if needed
#' @param x text value (may be empty or the page placeholder string)
#' @param font `fp_text` object for styling
#' @noRd
#' @keywords internal
make_hf_run <- function(x, font) {
  if (is.na(x) || is.null(x)) {
    return(list(officer::ftext("", font)))
  }
  if (x == "_DOCORATOR_PAGE_PLACEHOLDER_") {
    return(list(
      officer::ftext("Page ", prop = font),
      officer::run_word_field("PAGE"),
      officer::ftext(" of ", prop = font),
      officer::run_word_field("NUMPAGES")
    ))
  }
  list(officer::ftext(x, font))
}

test_that("Header and footer processes into latex",{

  # character (headers)
  expect_snapshot(
    hf_process("My first line")
  )
  expect_snapshot(
    hf_process(c("Two","Lines"))
  )

  # fancyhead
  expect_snapshot(
    fancyhead(
      fancyrow(right = doc_pagenum()),
      fancyrow(left = "a", center = "b", right = "c"),
      fancyrow(left = "something very longgggggggggggggggggggg")
    ) |>
      hf_process()
  )

  # fancyfoot
  expect_snapshot(
    fancyfoot(
      fancyrow(left = "something very longgggggggggggggggggggg"),
      fancyrow(right = "something else"),
      fancyrow(center = "middle")
    ) |>
      hf_process()
  )
})

test_that("Header and footer heights are calculated correctly",{

  fontsize <- 12
  my_header <- fancyhead(fancyrow("one line"))

  expect_equal(
    hf_height(my_header, fontsize),
    ceiling(fontsize*1.2)
  )

  fontsize <- 14
  my_header <- fancyhead(fancyrow("one"), fancyrow("two"))

  expect_equal(
    hf_height(my_header, fontsize),
    ceiling(fontsize*1.2*2)
  )


  fontsize <- 9
  my_header <- fancyfoot(fancyrow("one"),
                         fancyrow("two"),
                         fancyrow("three"),
                         fancyrow("four"),
                         fancyrow("five"))

  expect_equal(
    hf_height(my_header, fontsize),
    ceiling(fontsize*1.2*5)
  )
})

test_that("Footnote characters are escaped correctly",{

  footnote_string <- hf_process(fancyfoot(fancyrow(file.path("path","to", "my_file.R"))))
  title_string <- hf_process(fancyhead(fancyrow("hello_world")))
  pagenum <- hf_process(fancyhead(fancyrow(doc_pagenum())))

  expect_true(stringr::str_detect(footnote_string, "\\\\_file"))
  expect_true(stringr::str_detect(title_string, "\\\\_world"))
  expect_snapshot(pagenum)

  # escape_latex false
  footnote_string <- hf_process(fancyfoot(fancyrow(file.path("path","to", "my_file.R"))), escape_latex = FALSE)
  title_string <- hf_process(fancyhead(fancyrow("hello_world")), escape_latex = FALSE)


  expect_true(stringr::str_detect(footnote_string, "path/to/my_file.R"))
  expect_true(stringr::str_detect(title_string, "hello_world"))
})

test_that("fancyrow() accepts valid arguments",{

  expect_silent(fancyrow(left = "Left text", center = "Center text"))
  expect_silent(fancyrow(left = NA, center = "Center text", right = NA))
  expect_invisible(check_fancyrow_string(left = NA, center = "Center text", right = NA), TRUE)
})

test_that("fancyrow() flags non-character arguments",{

  expect_error(fancyrow(left = 123, center = "Center text", right = "Right text"), "`left` must be a character string or NA, but is numeric.")
  expect_error(fancyrow(left = "Left text", center = list("Center"), right = "Right text"), "`center` must be a character string or NA, but is list.")
})

test_that("fancyrow() flags arguments with length > 1",{

  expect_error(fancyrow(left = c("Left1", "Left2"), center = "Center text", right = "Right text"), "`left` must be a single value, but has a length of 2.")
  expect_error(fancyrow(left = 123, center = c("Center1", "Center2"), right = "`left` must be a character string or NA, but is numeric.
`center` must be a single value, but has a length of 2."))
  expect_error(fancyrow(left = 123, center = c("Center1", "Center2", "Center3")),"`left` must be a character string or NA, but is numeric.
`center` must be a single value, but has a length of 3.")
})

test_that("fancyrows are split when required", {
  # 185 char fancyrow with only 1 element, should be split into 3
  fancyrow1 <- fancyrow(left = "The quick brown fox jumps over the lazy dog, showcasing a vibrant array of colors and swift movements across the tranquil, sun-drenched landscape. This idyllic scene unfolds gracefully.")
  fancywrap1 <- fancywrap(fancyrow1, chars = 70)
  expect_equal(fancywrap1,
               list(fancyrow(left = "The quick brown fox jumps over the lazy dog, showcasing a vibrant"),
                    fancyrow(left = "array of colors and swift movements across the tranquil, sun-drenched"),
                    fancyrow(left = "landscape. This idyllic scene unfolds gracefully.")))
  # right
  fancyrow2 <- fancyrow(right = "The quick brown fox jumps over the lazy dog, showcasing a vibrant array of colors and swift movements across the tranquil, sun-drenched landscape. This idyllic scene unfolds gracefully.")
  fancywrap2 <- fancywrap(fancyrow2, chars = 70)
  expect_equal(fancywrap2,
               list(fancyrow(right = "The quick brown fox jumps over the lazy dog, showcasing a vibrant"),
                    fancyrow(right = "array of colors and swift movements across the tranquil, sun-drenched"),
                    fancyrow(right = "landscape. This idyllic scene unfolds gracefully.")))

  # center
  fancyrow3 <- fancyrow(center = "The quick brown fox jumps over the lazy dog, showcasing a vibrant array of colors and swift movements across the tranquil, sun-drenched landscape. This idyllic scene unfolds gracefully.")
  fancywrap3 <- fancywrap(fancyrow3, chars = 70)
  expect_equal(fancywrap3,
               list(fancyrow(center = "The quick brown fox jumps over the lazy dog, showcasing a vibrant"),
                    fancyrow(center = "array of colors and swift movements across the tranquil, sun-drenched"),
                    fancyrow(center = "landscape. This idyllic scene unfolds gracefully.")))

  # 2 elements - no wrapping just adds to list, too long to fit
  fancyrow4 <- fancyrow(left = "The quick brown fox jumps over the lazy dog, showcasing a vibrant array of colors and swift movements across the tranquil, sun-drenched landscape. This idyllic scene unfolds gracefully.",
                        right = "second element which prevents wrapping")
  fancywrap4 <- suppressMessages(fancywrap(fancyrow4, chars = 70))
  # tells you it's too long and won't fit
  expect_message(fancywrap(fancyrow4, chars = 70))
  expect_equal(fancywrap4, list(fancyrow4))

  # 2 elements - no wrapping just adds to list, everything fits
  fancyrow5 <- fancyrow(left = "The quick brown fox",
                        right = "second element which prevents wrapping")
  fancywrap5 <- fancywrap(fancyrow5, chars = 70)
  expect_equal(list(fancyrow5), fancywrap5)


  # empty fancyrow - no change
  fancyrow6 <- fancyrow()
  fancywrap6 <- fancywrap(fancyrow6, chars = 10)
  expect_equal(list(fancyrow6), fancywrap6)

  # error if chars is not numeric length 1
  expect_error(fancywrap(fancyrow1, chars = "not a number")) 

})

test_that("splitting of fancyhead and fancyfoot elements are handled correctly",{

  fancyhead1 <- fancyhead(
    # one element should be wrapped into 2
    fancyrow(left = "The quick brown fox jumps over the lazy dog, showcasing a vibrant array of colors and swift movements across the tranquil, sun-drenched landscape. This idyllic scene unfolds gracefully."),
    # two elements should be left as is
    fancyrow(center = "The quick brown fox jumps over the lazy dog, showcasing a vibrant array of colors and swift movements across the tranquil, sun-drenched landscape. This idyllic scene unfolds gracefully.",
             right = "second element that will prevent wrapping")
  )
  fancywrap1 <- suppressMessages(fancywrap(fancyhead1, chars = 70))

  # should still be a fancyheader
  expect_true(inherits(fancywrap1, "fancyhead"))
  expect_equal(
    # 4 fancyrows
    fancyhead(
      fancyrow(left = "The quick brown fox jumps over the lazy dog, showcasing a vibrant"),
      fancyrow(left = "array of colors and swift movements across the tranquil, sun-drenched"),
      fancyrow(left = "landscape. This idyllic scene unfolds gracefully."),
      fancyrow(center = "The quick brown fox jumps over the lazy dog, showcasing a vibrant array of colors and swift movements across the tranquil, sun-drenched landscape. This idyllic scene unfolds gracefully.", right = "second element that will prevent wrapping")
    ),
    fancywrap1

  )


  fancyfoot1 <- fancyfoot(
    # two elements should be left as is
    fancyrow(center = "The quick brown fox jumps over the lazy dog, showcasing a vibrant array of colors and swift movements across the tranquil, sun-drenched landscape. This idyllic scene unfolds gracefully.",
             right = "second element that will prevent wrapping"),
    # one element should be wrapped into 2
    fancyrow(left = "The quick brown fox jumps over the lazy dog, showcasing a vibrant array of colors and swift movements across the tranquil, sun-drenched landscape. This idyllic scene unfolds gracefully.")
  )
  fancywrap2 <- suppressMessages(fancywrap(fancyfoot1, chars = 70))

  # should still be a fancyheader
  expect_true(inherits(fancywrap2, "fancyfoot"))
  expect_equal(
    # 4 fancyrows
    fancyfoot(
      fancyrow(center = "The quick brown fox jumps over the lazy dog, showcasing a vibrant array of colors and swift movements across the tranquil, sun-drenched landscape. This idyllic scene unfolds gracefully.", right = "second element that will prevent wrapping"),
      fancyrow(left = "The quick brown fox jumps over the lazy dog, showcasing a vibrant"),
      fancyrow(left = "array of colors and swift movements across the tranquil, sun-drenched"),
      fancyrow(left = "landscape. This idyllic scene unfolds gracefully.")
    ),
    fancywrap2
  )

})

test_that("splitting of fancyhead and fancyfoot elements in a docorator object is handled correctly",{
  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    )

  # no headers and footers
  docorator1 <- as_docorator(
    x = my_gt,
    header = NULL,
    footer=NULL,
    display_name = "no_headers_or_footers",
    display_loc = NULL,
    save_object = FALSE
  )

  fancywrap1 <- fancywrap(docorator1)

  # no change
  expect_equal(fancywrap1, docorator1)

  docorator2 <- as_docorator(
    x = my_gt,
    header = fancyhead(
      fancyrow(left = "The quick brown fox jumps over the lazy dog, showcasing a vibrant array of colors and swift movements across the tranquil, sun-drenched landscape. This idyllic scene unfolds gracefully.")
    ),
    footer = fancyfoot(
      fancyrow(left = "The quick brown fox jumps over the lazy dog, showcasing a vibrant array of colors and swift movements across the tranquil, sun-drenched landscape. This idyllic scene unfolds gracefully.")
    ),
    display_name = "headers_and_footers",
    display_loc = NULL,
    save_object = FALSE
  )

  fancywrap2 <- fancywrap(docorator2)

  # for font size 10 the characters should be 126
  expect_equal(
    as_docorator(
      x = my_gt,
      header = fancywrap(fancyhead(
        fancyrow(left = "The quick brown fox jumps over the lazy dog, showcasing a vibrant array of colors and swift movements across the tranquil, sun-drenched landscape. This idyllic scene unfolds gracefully.")
      ), 126),
      footer = fancywrap(fancyfoot(
        fancyrow(left = "The quick brown fox jumps over the lazy dog, showcasing a vibrant array of colors and swift movements across the tranquil, sun-drenched landscape. This idyllic scene unfolds gracefully.")
      ), 126),
      display_name = "headers_and_footers",
      display_loc = NULL,
      save_object = FALSE
    ),
    fancywrap2
  )

  # pass number of characters 
  fancywrap3 <- fancywrap(docorator2, chars = 100)

  # for font size 10 the characters should be 100
  expect_equal(
    as_docorator(
      x = my_gt,
      header = fancywrap(fancyhead(
        fancyrow(left = "The quick brown fox jumps over the lazy dog, showcasing a vibrant array of colors and swift movements across the tranquil, sun-drenched landscape. This idyllic scene unfolds gracefully.")
      ), 100),
      footer = fancywrap(fancyfoot(
        fancyrow(left = "The quick brown fox jumps over the lazy dog, showcasing a vibrant array of colors and swift movements across the tranquil, sun-drenched landscape. This idyllic scene unfolds gracefully.")
      ), 100),
      display_name = "headers_and_footers",
      display_loc = NULL,
      save_object = FALSE
    ),
    fancywrap3
  )

})

test_that("doc_pagenum renders correctly in different render engines", {
  fancyrows <- fancyhead(fancyrow(
    left = "Left text",
    center = "Center text",
    right = doc_pagenum()
  ))
  rows_df <- purrr::map_dfr(fancyrows, as_tibble_fancyrow)
  latex <- process_rows_pdf(rows_df, type = "head")
  expect_true(grep("Page \\\\thepage", latex) > 0)

  docx_fpar <- process_rows_docx(fancyrows)
  expect_true((grepl("PAGE", docx_fpar[[1]]$chunks[[6]]$field)))
  expect_true((grepl("NUMPAGES", docx_fpar[[1]]$chunks[[8]]$field)))
})

test_that("prep_obj_rtf - ggplot", {

  ggplot1 <- ggplot2::ggplot(data = mtcars, ggplot2::aes(y=cyl, x=mpg)) +
    ggplot2::geom_point() +
    ggplot2::labs(title = "title1", subtitle = "subtitle1", tag = "tag1", caption = "footnote1")
  ggplot2 <- ggplot2::ggplot(data = mtcars, ggplot2::aes(x=cyl, y=mpg)) +
    ggplot2::geom_point() +
    ggplot2::labs(title = "title2", subtitle = "subtitle2", tag = "tag2", caption = "footnote2")

  # list of ggplots
  docorator <- as_docorator(
    x = list(ggplot1, ggplot2),
    header = fancyhead(fancyrow(center = "this is the title"),
                       fancyrow(center = "this is the subtitle1"),
                       fancyrow(center = "this is the subtitle2")),
    footer = fancyfoot(fancyrow(left = "this is the docorator footnote")),
    display_name = "mytbl",
    save_object = FALSE,
    convert_ggplot = FALSE
  )

  gt_display <- prep_obj_rtf(docorator)

  # should be a gt_group object
  expect_equal(class(gt_display), "gt_group")

  gt_1 <- gt::grp_pull(gt_display, 1)
  gt_2 <- gt::grp_pull(gt_display, 2)

  # check headers and footers are added correctly for each

  # ggplot footnote first
  expect_equal(unlist(gt_1$`_footnotes`$footnotes), c("footnote1", "this is the docorator footnote"))
  expect_equal(unlist(gt_2$`_footnotes`$footnotes), c("footnote2", "this is the docorator footnote"))

  # title should be docorator title
  # subtitle should be docorator subtitles then ggplot title, subtitle then tag
  expect_equal(gt_1$`_heading`$title, "this is the title")
  expect_equal(as.character(gt_1$`_heading`$subtitle), "this is the subtitle1<br>this is the subtitle2<br>title1<br>subtitle1<br>tag1")
  expect_equal(gt_2$`_heading`$title, "this is the title")
  expect_equal(as.character(gt_2$`_heading`$subtitle), "this is the subtitle1<br>this is the subtitle2<br>title2<br>subtitle2<br>tag2")


  # converted to png 

  # list of ggplots
  docorator <- as_docorator(
    x = list(ggplot1, ggplot2),
    header = fancyhead(fancyrow(center = "this is the title"),
                       fancyrow(center = "this is the subtitle1"),
                       fancyrow(center = "this is the subtitle2")),
    footer = fancyfoot(fancyrow(left = "this is the docorator footnote")),
    display_name = "mytbl",
    save_object = FALSE
  )

  gt_display <- prep_obj_rtf(docorator)

  # should be a gt_group object
  expect_equal(class(gt_display), "gt_group")

  gt_1 <- gt::grp_pull(gt_display, 1)
  gt_2 <- gt::grp_pull(gt_display, 2)

  # check headers and footers are added correctly for each

  # only docorator footnotes
  expect_equal(unlist(gt_1$`_footnotes`$footnotes), c("this is the docorator footnote"))
  expect_equal(unlist(gt_2$`_footnotes`$footnotes), c("this is the docorator footnote"))

  # title should be docorator titles only 
  expect_equal(gt_1$`_heading`$title, "this is the title")
  expect_equal(as.character(gt_1$`_heading`$subtitle), "this is the subtitle1<br>this is the subtitle2")
  expect_equal(gt_2$`_heading`$title, "this is the title")
  expect_equal(as.character(gt_2$`_heading`$subtitle), "this is the subtitle1<br>this is the subtitle2")

})

test_that("prep_obj_rtf - png", {

  png_obj1 <- png_path(path = system.file("extdata/test_image.png", package = "docorator"))
  png_obj2 <- png_path(path = system.file("extdata/test_image.png", package = "docorator"))

  # list of pngs
  docorator <- as_docorator(
    x = list(png_obj1, png_obj2),
    header = fancyhead(fancyrow(center = "this is the title"),
                       fancyrow(center = "this is the subtitle1"),
                       fancyrow(center = "this is the subtitle2")),
    footer = fancyfoot(fancyrow(left = "this is the docorator footnote")),
    display_name = "mytbl",
    save_object = FALSE)

  gt_display <- prep_obj_rtf(docorator)

  # should be a gt_group object
  expect_equal(class(gt_display), "gt_group")

  gt_1 <- gt::grp_pull(gt_display, 1)
  gt_2 <- gt::grp_pull(gt_display, 2)

  # check headers and footers are added correctly for each
  expect_equal(unlist(gt_1$`_footnotes`$footnotes), "this is the docorator footnote")
  expect_equal(unlist(gt_2$`_footnotes`$footnotes), "this is the docorator footnote")

  # title should be docorator title
  # subtitle should be docorator subtitles
  expect_equal(gt_1$`_heading`$title, "this is the title")
  expect_equal(as.character(gt_1$`_heading`$subtitle), "this is the subtitle1<br>this is the subtitle2")
  expect_equal(gt_2$`_heading`$title, "this is the title")
  expect_equal(as.character(gt_2$`_heading`$subtitle), "this is the subtitle1<br>this is the subtitle2")

})

test_that("prep_obj_rtf - character", {
  character <- "No Data to Display"

  docorator <- as_docorator(
    x = character,
    header = fancyhead(fancyrow(center = "this is the title"),
                       fancyrow(center = "this is the subtitle1"),
                       fancyrow(center = "this is the subtitle2")),
    footer = fancyfoot(fancyrow(left = "this is the docorator footnote")),
    display_name = "mytbl",
    save_object = FALSE)

  gt_display <- prep_obj_rtf(docorator)

  # should be a gt_table object
  expect_equal(class(gt_display), c("gt_tbl","list"))


  # check headers and footers are added correctly

  expect_equal(unlist(gt_display$`_footnotes`$footnotes), "this is the docorator footnote")

  # title should be docorator title
  # subtitle should be docorator subtitle
  expect_equal(gt_display$`_heading`$title, "this is the title")
  expect_equal(as.character(gt_display$`_heading`$subtitle), "this is the subtitle1<br>this is the subtitle2")
 })

test_that("prep_obj_rtf - invalid", {
  numeric <- 1

  docorator <- as_docorator(
    x = numeric,
    header = fancyhead(fancyrow(center = "this is the title"),
                       fancyrow(center = "this is the subtitle1"),
                       fancyrow(center = "this is the subtitle2")),
    footer = fancyfoot(fancyrow(left = "this is the docorator footnote")),
    display_name = "mytbl",
    save_object = FALSE)

  expect_error(prep_obj_rtf(docorator), "For RTF render the display must be class character, gt_tbl, gt_group, PNG, or ggplot, not a number.")
})

test_that("save docorator object works", {

  skip_on_cran()
  skip_on_ci()

  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    )

  withr::with_tempdir({
    res <- suppressMessages(as_docorator(
      x = my_gt,
      header = fancyhead(
        fancyrow("first line header"),
        fancyrow("second line header")
      ),
      footer = NULL,
      display_name = "my_first_gt",
      display_loc = NULL
    )
    )

    expect_true(file.exists("my_first_gt.RDS"))

    # read object back in and check it matches
    docorator_obj <- readRDS("my_first_gt.RDS")

    expect_equal(docorator_obj, res)

  })
})

test_that("deprecated - docorate works", {

  skip_on_cran()
  skip_on_ci()

  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    )

  withr::with_tempdir({

    expect_error(
      docorate(
        x = my_gt,
        header = fancyhead(
          fancyrow("first line header"),
          fancyrow("second line header")
        ),
        footer = NULL,
        filename = "my_first_gt.pdf",
        path = NULL
      ))

    expect_false(file.exists("my_first_gt.pdf"))

  })
})

test_that("Create docorator object without display_name errors",{

  skip_on_cran()
  skip_on_ci()

  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    )

  expect_error(
    suppressMessages(
      as_docorator(
      x = my_gt,
      header = fancyhead(
        fancyrow("first line header")
      ),
      footer = NULL,
      save_object = FALSE
    )
    ),
    "The `display_name` argument must be specified"
  )



})

test_that("Set geometry defaults", {
  defaults <- list(
    paperheight = "8.5in",
    paperwidth = "11in",
    left = "1in",
    right = "1in",
    top = "1.25in",
    bottom = "1.25in",
    headsep = "10pt",
    includehead = TRUE,
    includefoot = TRUE,
    headheight = NULL,
    footskip = NULL
  )

  expect_equal(defaults, geom_set())

  expect_equal(
    geom_set(headheight = "20pt", footskip = "10pt"),
    list(
      paperheight = "8.5in",
      paperwidth = "11in",
      left = "1in",
      right = "1in",
      top = "1.25in",
      bottom = "1.25in",
      headsep = "10pt",
      includehead = TRUE,
      includefoot = TRUE,
      headheight = "20pt",
      footskip = "10pt"
    )
  )

  expect_equal(
    geom_set(left = "2in", paper = "legalpaper"),
    list(
      paperheight = "8.5in",
      paperwidth = "11in",
      right = "1in",
      top = "1.25in",
      bottom = "1.25in",
      headsep = "10pt",
      includehead = TRUE,
      includefoot = TRUE,
      headheight = NULL,
      footskip = NULL,
      left = "2in",
      paper = "legalpaper"
    )
  )
})

test_that("geometry processing with headers, footers, and fontsize - pdf", {
  fontsize <- 12
  my_header <- fancyhead(fancyrow("one"), fancyrow("two"))
  my_footer <- fancyfoot(fancyrow("one"), fancyrow("two"), fancyrow("three"))

  # auto heights
  expect_equal(
    geom_process(
      my_header,
      my_footer,
      fontsize,
      geom_set(headheight = NULL, footskip = NULL)
    ),
    paste0(
      "paperheight=8.5in, paperwidth=11in, left=1in, right=1in, top=1.25in, ",
      "bottom=1.25in, headsep=10pt, includehead=TRUE, includefoot=TRUE, headheight=29pt, footskip=44pt"
    )
  )

  # fixed heights
  expect_equal(
    geom_process(
      my_header,
      my_footer,
      fontsize,
      geom_set(headheight = "20pt", footskip = "30pt")
    ),
    paste0(
      "paperheight=8.5in, paperwidth=11in, left=1in, right=1in, top=1.25in, ",
      "bottom=1.25in, headsep=10pt, includehead=TRUE, includefoot=TRUE, headheight=20pt, footskip=30pt"
    )
  )
})

test_that("geometry processing - docx", {
  expect_equal(
    geom_process(
      geometry = geom_set(left = "0.5in", right = "0.5in"),
      engine = "docx"
    ),
    list(
      left = 0.5,
      right = 0.5,
      top = 1.25,
      bottom = 1.25
    )
  )

  # units in cm
  expect_equal(
    suppressMessages(geom_process(
      geometry = geom_set(left = "1.27cm", right = "1.27cm"),
      engine = "docx"
    )),
    list(
      left = 1.27,
      right = 1.27,
      top = 1.25,
      bottom = 1.25
    )
  )

  expect_snapshot(
    geom <- geom_process(
      geometry = geom_set(left = "1.27cm"),
      engine = "docx"
    )
  )

  # no units
  expect_equal(
    geom_process(
      geometry = geom_set(left = "1.27", right = "1.27"),
      engine = "docx"
    ),
    list(
      left = 1.27,
      right = 1.27,
      top = 1.25,
      bottom = 1.25
    )
  )

  # some values are missing
  expect_equal(
    suppressMessages(geom_process(
      geometry = list(left = "1.27in"),
      engine = "docx"
    )),
    list(
      left = 1.27,
      right = 1,
      top = 1.25,
      bottom = 1.25
    )
  )
  # test cli_text message for missing values
  expect_snapshot(
    geom <- geom_process(
      geometry = list(left = "1.27in"),
      engine = "docx"
    )
  )
})

test_that("render docx works - string", {
  withr::with_tempdir({
    suppressMessages(
      # single string with headers and footers
      as_docorator(
        "This is a string",
        display_name = "string",
        header = fancyhead(
          fancyrow(
            left = "Left header"
          ),
          fancyrow(center = "Center header", right = "Right header")
        ),
        footer = fancyfoot(
          fancyrow(
            left = "Left footer",
            center = "Center footer"
          ),
          fancyrow(right = "Right footer")
        )
      ) |>
        render_docx()
    )

    # file exists
    expect_true(file.exists("string.docx"))

    # headers and footers are as expected
    doc <- officer::read_docx("string.docx")

    # header xml
    header_xml <- xml2::read_xml(file.path(
      doc$package_dir,
      "word",
      "header1.xml"
    )) |>
      as.character()
    # check for header content
    expect_equal(
      stringr::str_detect(header_xml, "Left header"),
      TRUE
    )
    expect_equal(
      stringr::str_detect(header_xml, "Center header"),
      TRUE
    )
    expect_equal(
      stringr::str_detect(header_xml, "Right header"),
      TRUE
    )

    # footer xml
    footer_xml <- xml2::read_xml(file.path(
      doc$package_dir,
      "word",
      "footer1.xml"
    ))
    # check for footer content
    expect_equal(
      stringr::str_detect(as.character(footer_xml), "Left footer"),
      TRUE
    )
    expect_equal(
      stringr::str_detect(as.character(footer_xml), "Center footer"),
      TRUE
    )
    expect_equal(
      stringr::str_detect(as.character(footer_xml), "Right footer"),
      TRUE
    )

    # file contains expected content
    officer::docx_summary(doc) |>
      dplyr::pull(text) |>
      stringr::str_detect("This is a string") |>
      expect_true()

    # vector of strings
    suppressMessages(
      as_docorator(
        c("This is a string", "with multiple lines"),
        display_name = "string_vec"
      ) |>
        render_docx()
    )

    # file exists
    expect_true(file.exists("string_vec.docx"))

    # file contains expected content
    doc_vec <- officer::read_docx("string_vec.docx")
    expect_equal(
      officer::docx_summary(doc_vec) |>
        dplyr::pull(text) |>
        stringr::str_detect("This is a string") |>
        any(),
      TRUE
    )
    expect_equal(
      officer::docx_summary(doc_vec) |>
        dplyr::pull(text) |>
        stringr::str_detect("with multiple lines") |>
        any(),
      TRUE
    )
  })
})

test_that("render docx works - gt table", {
  withr::with_tempdir({
    gt_tbl <- gt::gt(mtcars[1:5, 1:5]) |>
      gt::tab_header(title = "", subtitle = "A gt table")

    suppressMessages(
      gt_tbl |>
        as_docorator(
          display_name = "gt_tbl"
        ) |>
        render_docx()
    )

    # file exists
    expect_true(file.exists("gt_tbl.docx"))

    doc <- officer::read_docx("gt_tbl.docx")

    # file contains expected content - subtitle
    expect_equal(
      officer::docx_summary(doc) |>
        dplyr::pull(text) |>
        stringr::str_detect("A gt table") |>
        any(),
      TRUE
    )

    # check table content is present
    expect_equal(
      officer::docx_summary(doc) |>
        dplyr::pull(text) |>
        stringr::str_detect("mpg") |>
        any(),
      TRUE
    )
  })
})

test_that("render docx works - gt_group", {
  withr::with_tempdir({
    gt_tbl <- gt::gt(mtcars[1:5, 1:5]) |>
      gt::tab_header(title = "", subtitle = "A gt table")
    gt_tbl2 <- gt::gt(mtcars[6:10, 6:10]) |>
      gt::tab_header(title = "", subtitle = "A second gt table")
    gt_group <- gt::gt_group(gt_tbl, gt_tbl2)
    suppressMessages(
      gt_group |>
        as_docorator(
          display_name = "gt_group"
        ) |>
        render_docx()
    )
    # file exists
    expect_true(file.exists("gt_group.docx"))
    # file contains expected content - subtitle from first table
    doc <- officer::read_docx("gt_group.docx")
    expect_equal(
      officer::docx_summary(doc) |>
        dplyr::pull(text) |>
        stringr::str_detect("A gt table") |>
        any(),
      TRUE
    )
    # file contains expected content - subtitle from second table
    expect_equal(
      officer::docx_summary(doc) |>
        dplyr::pull(text) |>
        stringr::str_detect("A second gt table") |>
        any(),
      TRUE
    )
  })

  # check both table contents are present - mpg is in first table, qsec is in second table
  expect_equal(
    officer::docx_summary(doc) |>
      dplyr::pull(text) |>
      stringr::str_detect("mpg") |>
      any(),
    TRUE
  )
  expect_equal(
    officer::docx_summary(doc) |>
      dplyr::pull(text) |>
      stringr::str_detect("qsec") |>
      any(),
    TRUE
  )
})

test_that("render docx works - ggplot2", {
  withr::with_tempdir({
    p <- ggplot2::ggplot(mtcars, ggplot2::aes(x = wt, y = mpg)) +
      ggplot2::geom_point() +
      ggplot2::labs(title = "A ggplot2 plot")

    suppressMessages(
      p |>
        as_docorator(display_name = "ggplot2") |>
        render_docx()
    )

    expect_true(file.exists("ggplot2.docx"))

    doc <- officer::read_docx("ggplot2.docx")
    doc_summary <- officer::docx_summary(doc, detailed = TRUE)

    expect_true(stringr::str_detect(doc_summary$image_path, ".png"))
  })
})

test_that("render docx works - png", {
  withr::with_tempdir({
    p <- ggplot2::ggplot(mtcars, ggplot2::aes(x = wt, y = mpg)) +
      ggplot2::geom_point() +
      ggplot2::labs(title = "A ggplot2 plot")

    ggplot2::ggsave(
      filename = "test.png",
      plot = p,
      width = 5,
      height = 5
    )

    suppressMessages(
      as_docorator(
        png_path("test.png"),
        display_name = "png"
      ) |>
        render_docx()
    )

    expect_true(file.exists("png.docx"))

    doc <- officer::read_docx("png.docx")
    doc_summary <- officer::docx_summary(doc, detailed = TRUE)

    expect_true(stringr::str_detect(doc_summary$image_path, ".png"))
  })
})

test_that("render docx works - list", {
  withr::with_tempdir({
    gt_tbl <- gt::gt(mtcars[1:5, 1:5]) |>
      gt::tab_header(title = "", subtitle = "A gt table")
    gt_tbl2 <- gt::gt(mtcars[6:10, 6:10]) |>
      gt::tab_header(title = "", subtitle = "A second gt table")
    gt_group <- gt::gt_group(gt_tbl2, gt_tbl2)
    p <- ggplot2::ggplot(mtcars, ggplot2::aes(x = wt, y = mpg)) +
      ggplot2::geom_point() +
      ggplot2::labs(title = "A ggplot2 plot")

    suppressMessages(
      list(gt_tbl, p, gt_group) |>
        as_docorator(
          display_name = "gt_group"
        ) |>
        render_docx()
    )
    # file exists
    expect_true(file.exists("gt_group.docx"))

    # file contains expected content - subtitle from first table
    doc <- officer::read_docx("gt_group.docx")
    summary <- officer::docx_summary(doc, detailed = TRUE)
    expect_equal(
      summary |>
        dplyr::pull(run_content_text) |>
        stringr::str_detect("A gt table") |>
        any(),
      TRUE
    )
    # file contains expected content - plot
    expect_true(any(stringr::str_detect(summary$image_path, ".png")))

    # file contains expected content - subtitle from gt_group appears twice
    expect_equal(
      summary |>
        dplyr::pull(run_content_text) |>
        stringr::str_detect("A second gt table") |>
        sum(na.rm = TRUE),
      2
    )
  })
})

test_that("render to pdf works", {

  skip_on_cran()
  skip_on_ci()

  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    )

  docorator <- as_docorator(
    x = my_gt,
    header = fancyhead(fancyrow("first line header"), fancyrow("second line header")),
    footer = NULL,
    display_name = "my_first_gt",
    display_loc = NULL,
    save_object = FALSE
  )

  # path set to NULL
  withr::with_tempdir({
    res <- suppressMessages( docorator |> render_pdf()
    )

    expect_true(file.exists("my_first_gt.pdf"))
  })

  # with path supplied
  withr::with_tempdir({
    dir.create("tempdir2")
    res <- suppressMessages(docorator |> render_pdf(
      display_loc = "tempdir2"
    )
    )

    expect_true(file.exists(file.path("tempdir2", "my_first_gt.pdf")))
  })

  # quarto render with path supplied
  withr::with_tempdir({

    dir.create("tempdir3")

    res <- suppressMessages( docorator |> render_pdf(
      quarto = TRUE,
      display_loc = "tempdir3"
    )
    )
    expect_true(file.exists(file.path("tempdir3", "my_first_gt.pdf")))
  })

  # quarto render with no path supplied
  withr::with_tempdir({
    docorator <- as_docorator(
      x = my_gt,
      header = fancyhead(fancyrow("first line header"), fancyrow("second line header")),
      footer = NULL,
      display_name = "my_first_gt",
      save_object = FALSE
    )
    res <- suppressMessages( docorator |> render_pdf(
      quarto = TRUE
    )
    )

    expect_true(file.exists("my_first_gt.pdf"))

  })
})

test_that("render to pdf, lists", {

  skip_on_cran()
  skip_on_ci()

  png_obj1 <- png_path(path = system.file("extdata/test_image.png", package = "docorator"))
  png_obj2 <- png_path(path = system.file("extdata/test_image.png", package = "docorator"))

  ggplot1 <- ggplot2::ggplot(data = mtcars, ggplot2::aes(y=cyl, x=mpg)) +
    ggplot2::geom_point()
  ggplot2 <- ggplot2::ggplot(data = mtcars, ggplot2::aes(x=cyl, y=mpg)) +
    ggplot2::geom_point()


  withr::with_tempdir({

    # list of pngs
    docorator <- as_docorator(
      x = list(png_obj1, png_obj2),
      header = fancyhead(fancyrow("first line header"), fancyrow("second line header")),
      footer = NULL,
      display_name = "my_first_list",
      display_loc = NULL,
      save_object = FALSE
    )

    # list of ggplots
    docorator2 <- as_docorator(
      x = list(ggplot1, ggplot2),
      header = fancyhead(fancyrow("first line header"), fancyrow("second line header")),
      footer = NULL,
      display_name = "my_first_ggplot_list",
      display_loc = NULL,
      save_object = FALSE
    )


    res <- suppressMessages( docorator |> render_pdf()
    )
    res2 <- suppressMessages( docorator2 |> render_pdf()
    )

    expect_true(file.exists("my_first_list.pdf"))
    expect_true(file.exists("my_first_ggplot_list.pdf"))

    # 2 pages
    expect_equal(pdftools::pdf_info("my_first_list.pdf")$pages,2)
    expect_equal(pdftools::pdf_info("my_first_ggplot_list.pdf")$pages,2)
  })

})

test_that("render to pdf, lists - quarto", {

  skip_on_cran()
  skip_on_ci()

  png_obj1 <- png_path(path = system.file("extdata/test_image.png", package = "docorator"))
  png_obj2 <- png_path(path = system.file("extdata/test_image.png", package = "docorator"))

  ggplot1 <- ggplot2::ggplot(data = mtcars, ggplot2::aes(y=cyl, x=mpg)) +
    ggplot2::geom_point()
  ggplot2 <- ggplot2::ggplot(data = mtcars, ggplot2::aes(x=cyl, y=mpg)) +
    ggplot2::geom_point()

  withr::with_tempdir({

    docorator <- as_docorator(
      x = list(png_obj1, png_obj2),
      header = fancyhead(fancyrow("first line header"), fancyrow("second line header")),
      footer = NULL,
      display_name = "my_first_list",
      display_loc = NULL,
      save_object = FALSE
    )

    # list of ggplots
    docorator2 <- as_docorator(
      x = list(ggplot1, ggplot2),
      header = fancyhead(fancyrow("first line header"), fancyrow("second line header")),
      footer = NULL,
      display_name = "my_first_ggplot_list",
      display_loc = NULL,
      save_object = FALSE
    )


    res <- suppressMessages( docorator |> render_pdf(quarto = TRUE))

    res2 <- suppressMessages( docorator2 |> render_pdf(quarto = TRUE))


    expect_true(file.exists("my_first_list.pdf"))
    expect_true(file.exists("my_first_ggplot_list.pdf"))

    # 2 pages
    expect_equal(pdftools::pdf_info("my_first_list.pdf")$pages,2)
    expect_equal(pdftools::pdf_info("my_first_ggplot_list.pdf")$pages,2)
  }
  )

})

test_that("render to pdf - transform", {

  skip_on_cran()
  skip_on_ci()

  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    )

  transform <- function(x){stringr::str_replace_all(x,"num", "NUM")}

  withr::with_tempdir({

    docorator <- as_docorator(
      x = my_gt,
      header = fancyhead(fancyrow("first line header"), fancyrow("second line header")),
      footer = NULL,
      display_name = "my_first_gt",
      display_loc = NULL,
      save_object = FALSE
    )

    res <- suppressMessages( docorator |> render_pdf(transform = transform)
    )

    expect_true(file.exists("my_first_gt.pdf"))
    expect_true(stringr::str_detect(pdftools::pdf_text("my_first_gt.pdf"),"NUM"))
    expect_false(stringr::str_detect(pdftools::pdf_text("my_first_gt.pdf"),"num"))

  })

  # quarto render
  withr::with_tempdir({

    docorator <- as_docorator(
      x = my_gt,
      header = fancyhead(fancyrow("first line header"), fancyrow("second line header")),
      footer = NULL,
      display_name = "my_first_gt",
      save_object = FALSE
    )
    expect_warning(suppressMessages( docorator |> render_pdf(
      quarto = TRUE, transform = transform
    )))

    expect_true(file.exists("my_first_gt.pdf"))

   })
})

test_that("render to rtf works", {

  skip_on_cran()
  skip_on_ci()

  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    )

  withr::with_tempdir({

    docorator <- as_docorator(
      x = my_gt,
      header = fancyhead(fancyrow(left = "first line header"), fancyrow(center = "second line header")),
      footer = NULL,
      display_name = "my_first_gt",
      display_loc = NULL,
      save_object = FALSE
    )

    res <- suppressMessages( docorator |> render_rtf()
    )

    expect_true(file.exists("my_first_gt.rtf"))
  })

})

test_that("rtf unicode characters",{


  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    ) |>
    gt::text_transform(
      locations = gt::cells_body(columns = everything()),
      fn = function(x) {
        paste0("\U00A0", x ,"\U00A0")
      }
    )

  withr::with_tempdir({

    docorator <- as_docorator(
      x = my_gt,
      display_name = "my_gt_with_spaces",
      display_loc = NULL,
      save_object = FALSE
    )

    # unicode spaces have been replaced with actual spaces
    res <- suppressMessages( docorator |> render_rtf())
    doc <- readLines("my_gt_with_spaces.rtf")|> paste0(collapse = "")
    expect_false(grepl("\u00A0", doc, perl = TRUE))

    # unicode spaces are still present
    res <- suppressMessages( docorator |> render_rtf(remove_unicode_ws = FALSE))
    doc <- readLines("my_gt_with_spaces.rtf") |> paste0(collapse = "")
    expect_true(grepl("\u00A0", doc, perl = TRUE))
  })

})

test_that("rtf headers",{

  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    )|>
    gt::tab_options(
      page.header.use_tbl_headings = TRUE
    )

  withr::with_tempdir({
    docorator <- as_docorator(
      x = my_gt,
      display_name = "my_gt_header",
      display_loc = NULL,
      save_object = FALSE,
      header = fancyhead(fancyrow(center = "Header 1")),
      footer = fancyfoot(fancyrow("test footnote"))
    )

    res <- suppressMessages( docorator |> render_rtf())
    doc <- readLines("my_gt_header.rtf") |> paste0(collapse="")
    expect_false(grepl("\\{\\\\header",doc))
    res2 <- suppressMessages( docorator |> render_rtf(use_page_header = TRUE))
    doc2 <- readLines("my_gt_header.rtf")|> paste0(collapse="")
    expect_true(grepl("\\{\\\\header",doc2))

  })
})

test_that("render to rtf, lists of figures", {

  skip_on_cran()
  skip_on_ci()

  png_obj1 <- png_path(path = system.file("extdata/test_image.png", package = "docorator"))
  png_obj2 <- png_path(path = system.file("extdata/test_image.png", package = "docorator"))

  ggplot1 <- ggplot2::ggplot(data = mtcars, ggplot2::aes(y=cyl, x=mpg)) +
    ggplot2::geom_point() +
    ggplot2::labs(title = "title1", subtitle = "subtitle1", tag = "tag1", caption = "footnote1")
  ggplot2 <- ggplot2::ggplot(data = mtcars, ggplot2::aes(x=cyl, y=mpg)) +
    ggplot2::geom_point() +
    ggplot2::labs(title = "title2", subtitle = "subtitle2", tag = "tag2", caption = "footnote2")


  withr::with_tempdir({

    # list of pngs
    docorator <- as_docorator(
      x = list(png_obj1, png_obj2),
      header = fancyhead(fancyrow(center = "first line header"), fancyrow(center = "second line header")),
      footer = NULL,
      display_name = "my_first_list",
      display_loc = NULL,
      save_object = FALSE
    )

    # list of ggplots
    docorator2 <- as_docorator(
      x = list(ggplot1, ggplot2),
      header = fancyhead(fancyrow(center = "first line header"), fancyrow(center = "second line header")),
      footer = NULL,
      display_name = "my_first_ggplot_list",
      display_loc = NULL,
      save_object = FALSE
    )

    # warnings as gt cannot handle rtf figures yet
    res <- suppressWarnings(suppressMessages( docorator |> render_rtf()
    ))
    res2 <- suppressWarnings(suppressMessages( docorator2 |> render_rtf()
    ))

    expect_true(file.exists("my_first_list.rtf"))
    expect_true(file.exists("my_first_ggplot_list.rtf"))

  })

})

test_that("pipe together renders",{

  skip_on_cran()
  skip_on_ci()

  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    )

  withr::with_tempdir({

    docorator <- as_docorator(
      x = my_gt,
      header = fancyhead(fancyrow(left = "first line header"), fancyrow(center = "second line header")),
      footer = NULL,
      display_name = "my_first_gt",
      display_loc = NULL,
      save_object = FALSE
    )

    res <- suppressMessages( docorator |> render_rtf() |> render_pdf()
    )

    expect_true(file.exists("my_first_gt.rtf"))
    expect_true(file.exists("my_first_gt.pdf"))
  })
})

test_that("render non docorator object fails", {

  skip_on_cran()
  skip_on_ci()

  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    )

  expect_error(render_pdf(my_gt), "The `my_gt` argument must be class docorator, not a <gt_tbl> object. See documentation for `as_docorator`.")
})

test_that("render invalid transform", {
  skip_on_cran()
  skip_on_ci()

  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    )

  withr::with_tempdir({
    docorator <- as_docorator(
      x = my_gt,
      header = fancyhead(fancyrow("first line header"), fancyrow("second line header")),
      footer = NULL,
      display_name = "my_first_gt",
      display_loc = NULL,
      save_object = FALSE
    )

    expect_warning(suppressMessages( docorator |> render_pdf(transform = "INVALID_TRANSFORM")), "The transform argument must be a function, not a string. No transform applied.")
    expect_true(file.exists("my_first_gt.pdf"))

  })
})

test_that("render invalid header_latex", {
  skip_on_cran()
  skip_on_ci()

  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    )

  withr::with_tempdir({
    docorator <- as_docorator(
      x = my_gt,
      header = fancyhead(fancyrow("first line header"), fancyrow("second line header")),
      footer = NULL,
      display_name = "my_first_gt",
      display_loc = NULL,
      save_object = FALSE
    )

    expect_warning(suppressMessages( docorator |> render_pdf(header_latex = "INVALID_TRANSFORM.R")), "The header_latex argument must point to a valid .tex file. No header options applied.")
    expect_true(file.exists("my_first_gt.pdf"))

  })
})

test_that("render header_latex", {
  skip_on_cran()
  skip_on_ci()

  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    )

  withr::with_tempdir({

    # write .tex header file
    latex_commands <- c(
      "\\begin{document}",
      "This will be the only content in the document",
      "\\end{document}"
    )
    writeLines(latex_commands, "latex_header_file.tex")

    docorator <- as_docorator(
      x = my_gt,
      header = fancyhead(fancyrow("first line header"), fancyrow("second line header")),
      footer = NULL,
      display_name = "my_first_gt",
      display_loc = NULL,
      save_object = FALSE
    )

    res <- suppressMessages( docorator |> render_pdf(header_latex = "latex_header_file.tex")
    )

    expect_true(file.exists("my_first_gt.pdf"))

    pdf_text <- pdftools::pdf_text("my_first_gt.pdf")
    expect_true(stringr::str_detect(pdf_text,"This will be the only content in the document"))

  })
})


test_that("render keep tex file", {
  skip_on_cran()
  skip_on_ci()

  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    )

  withr::with_tempdir({
    docorator <- as_docorator(
      x = my_gt,
      header = fancyhead(fancyrow("first line header"), fancyrow("second line header")),
      footer = NULL,
      display_name = "my_first_gt",
      display_loc = NULL,
      save_object = FALSE
    )

    res <- suppressMessages( docorator |> render_pdf()
    )

    expect_false(file.exists("my_first_gt.tex"))

    res <- suppressMessages( docorator |> render_pdf(keep_tex = TRUE)
    )

    expect_true(file.exists("my_first_gt.tex"))
  })

})


test_that("render to pdf works with brackets in headers/footers", {

  skip_on_cran()
  skip_on_ci()

  my_gt <- gt::exibble |>
    gt::gt(
    )

  docorator <- as_docorator(
    x = my_gt,
    header = fancyhead(fancyrow("[a] first line header"), fancyrow("[b] second line header")),
    footer = fancyfoot(fancyrow("[1] first line footer"), fancyrow("[2] second line footer")),
    display_name = "my_first_gt",
    display_loc = NULL,
    save_object = FALSE
  )

  # rmd render pdf
  withr::with_tempdir({
    res <- suppressMessages( docorator |> render_pdf()
    )

    expect_true(file.exists("my_first_gt.pdf"))
  })


  # # quarto render pdf - doesn't work
  # withr::with_tempdir({

  #   dir.create("tempdir2")

  #   res <- suppressMessages( docorator |> render_pdf(
  #     quarto = TRUE,
  #     keep_tex = TRUE,
  #     display_loc = "tempdir2"
  #   )
  #   )
  #   expect_true(file.exists(file.path("tempdir2", "my_first_gt.pdf")))
  # })

})

#' Apply headers and footnotes to gt object
#'
#' Pulls headers and footnotes from the docorator object and applies them to the gt object
#' Note: this function is only for use by downstream tools and is not intended for general users
#'
#' @param x a docorator object
#'
#' @return gt object (either tbl or group)
#' @keywords internal
#'
#' @export
#' @keywords internal
#'
#' @examples
#' docorator <- gt::exibble |>
#'   gt::gt()|>
#'   as_docorator(
#'   display_name = "my_tbl",
#'   header = fancyhead(
#'   fancyrow(left = "Protocol: 12345", right = doc_pagenum()),
#'   fancyrow(center = "Demographic Summary")
#'   ),
#'   footer = NULL,
#'   save_object = FALSE)
#'
#' hf_to_gt(docorator)

hf_to_gt <- function(x) {

  if (!inherits(x, "docorator")) {
    cli::cli_abort("The {.arg {rlang::caller_arg(x)}} argument must be class docorator, not {.obj_type_friendly {x}}. See documentation for `as_docorator`.",
              call = rlang::caller_env())
  }

  gt <- x$display

  if (!rlang::inherits_any(gt, c("gt_tbl", "gt_group"))) {
    cli::cli_abort("The {.arg {rlang::caller_arg(x)}} argument must be class gt_tbl or gt_group, not {.obj_type_friendly {x}}.",
              call = rlang::caller_env())
  }

  head_foot <- hf_extract(x)

  if(inherits(gt, "gt_tbl")){
    gt <- hf_to_gt_tbl(gt, head_foot$head_data, head_foot$subhead_data, head_foot$foot_data)
  }

  return(gt)
}

#' headers and footers for gt_tbl
#' @param gt a gt object
#' @param header a character vector containing header information
#' @param subheader a character vector containing subheader information
#' @param footer a character vector containing footnote information
#'
#' @noRd
hf_to_gt_tbl <- function(gt, header, subheader, footer){
  if(inherits(gt, "gt_tbl")){

    # order of titles should be: all docorator headers (header and subheader) then gt title, then gt subtitles.
    subtitle <- NULL # will create a vector of new subtitles and split with <br> characters
    gt_headers <- NULL # vector of existing gt headers to be added at the end of docorator headers

    # check title and subtitle arent empty strings and add to gt_headers.
    if(!is.null(gt$`_heading`$title) && !identical(gt$`_heading`$title, "")){
      gt_headers <- gt$`_heading`$title
    }

    # check if existing subtitle exists, if so add to gt_headers
    if(!is.null(gt$`_heading`$subtitle) && !identical(gt$`_heading`$subtitle, "")){
      gt_headers <- c(gt_headers,gt$`_heading`$subtitle)
    }

    # add docorator subtitle text before old gt headers.
    subtitle <- c(subheader, gt_headers)

    # if not null or empty, add breaks and md()
    if(!is.null(subtitle) && !identical(subtitle,"")){
      subtitle <- gt::md(paste0(subtitle, collapse = "<br>"))
    }
    # header
    if(length(header)>0 | length(subtitle)>0){
      gt <- gt |>
        gt::tab_header(title = paste0(header, collapse = " "), subtitle = subtitle)
    }
    # footer
  if(length(footer)>0){
    for(footnote in footer){
      gt <- gt |>
        gt::tab_footnote(footnote = footnote)
    }
  }
  }
  return(gt)
}


#' extract header footer info from docorator object
#' @param x a docorator object
#'
#' @return list of header footer info
#' @keywords internal
#'
#' @export
#' @keywords internal
#' @examples
#' docorator <- gt::exibble |>
#'   gt::gt()|>
#'   as_docorator(
#'   display_name = "mytbl",
#'   header = fancyhead(
#'   fancyrow(left = "Protocol: 12345", right = doc_pagenum()),
#'   fancyrow(center = "Demographic Summary")
#'   ),
#'   footer = NULL,
#'   save_object = FALSE)
#'
#' hf_extract(docorator)
hf_extract <- function(x){
  # get header and footer information
  header <- x$header
  footer <- x$footer

  # Take titles that are alignment center, remove any missing
  # First value is title, any remaining are subtitles
  all_headers <- unlist(lapply(header, function(x){
    if(!is.na(x$center)){
      x$center
    }}))

  head_data <- all_headers[1]

  # subheaders
  if(length(all_headers[-1])>0){
    subhead_data <- all_headers[-1]
  }else{
    subhead_data <- NULL
  }

  # Take footers that are alignment left, remove any missing
  foot_data <- unlist(lapply(footer, function(x){
    if(!is.na(x$left)){
      x$left
    }}))

  list(head_data = head_data,
       subhead_data = subhead_data,
       foot_data = foot_data)

}

#' apply a gt function to a gt_group
#' @param x gt group object
#' @param func string with function name
#' @param args named list of function arguments with gt_tbl or gt_group as first arg
#' @param call caller env
#' @export
#' @examples
#' gt_tbl <- gt::exibble|> gt::gt()
#' gt_group <- gt::gt_group(gt_tbl, gt_tbl)
#'
#' func <- gt::tab_options
#' arg_list_group <- list(page.header.use_tbl_headings = c(TRUE))
#'
#' apply_to_gt_group(gt_group, func,arg_list_group)
#'
apply_to_gt_group <- function(x, func, args, call = rlang::caller_env()){

  if(!(inherits(x, c("gt_tbl", "gt_group")))){
    cli::cli_abort("First arg must be a gt_tbl or gt_group object, not {.obj_type_friendly {x}}")
  }
  # add gt to args list as first element
  full_args <- append(args, list(x), after = 0)

  if(inherits(x, "gt_tbl")){
    x <- do.call(func, full_args, envir = call)
  }else if(inherits(x, "gt_group")){
    for (i in seq_len(nrow(x$gt_tbls))) {
      # pull out gt_tbl, apply function, reinsert into group
      gt_tbl <- gt::grp_pull(x, i)
      # replace data arg with current gt_tbl
      full_args[[1]] <- gt_tbl
      #make it clear which table if an error occurs
      gt_tbl <- tryCatch({
        do.call(func, full_args, envir = call)
      },
      error = function(e) {
        cli::cli_abort("Failure in Table {i}", parent = e)
      })

      x <- gt::grp_replace(x, gt_tbl, .which = i)
    }
  }

  x
}

#' Convert png object to gt from docorator object
#' @param x docorator object
#' @noRd
#' @keywords internal
png_to_gt <- function(x){

  # save png to temp file
 temp_png <- tempfile(
    pattern = "temp_png_",
    tmpdir = tempdir(),
    fileext = ".png")

 png::writePNG(x$display$png, temp_png)

  # convert to gt
  gt <- dplyr::tibble(ggplot =  temp_png) |>
    gt::gt() |>
    gt::fmt_image(columns = dplyr::everything(), sep = ",", height = paste0(x$fig_dim[1], "in"), width = paste0(x$fig_dim[2], "in")) |>
    # remove column headers and borders
    gt::tab_options(
      column_labels.hidden = TRUE,
      table.border.top.style = "hidden",
      table.border.bottom.style = "hidden"
    )

  gt

}

#' Convert ggplot object to gt from docorator object
#' @param x docorator object
#' @noRd
#' @keywords internal
gg_to_gt <- function(x){

  if (!inherits(x$display, "ggplot")) {
    cli::cli_abort("The display must be class `ggplot`, not {.obj_type_friendly {x$display}}.",
                   call = rlang::caller_env())
  }

  # remove header footer information
  display_info <- hf_strip(x$display)

  # convert to gt
  gt <- dplyr::tibble(ggplot =  display_info$display |>
                 gg_to_image(fig_dim = x$fig_dim, path = tempdir())) |>
    gt::gt() |>
    gt::fmt_image(columns = dplyr::everything(), sep = ",", width = "6in") |>
    # add header and footer information
    gt::tab_header(title = display_info$head_data, subtitle = display_info$subhead_data) |>
    # remove column headers and borders
    gt::tab_options(
      column_labels.hidden = TRUE,
      table.border.top.style = "hidden",
      table.border.bottom.style = "hidden"
    )

  # add footnotes if not NULL - avoids null row being added
  if(!is.null(display_info$foot_data)){
    gt <- gt |>
      gt::tab_footnote(footnote = display_info$foot_data)
  }

  gt

}

#' ggplot to image filepath
#' Taken from code here: https://github.com/rstudio/gt/blob/6312cf5be92fae41633b5df7d41aa948b850aaf8/R/image.R#L351C1-L392C2
#' @param plot_object ggplot object
#' @param fig_dim vector containing figure height and width in inches. Defaults to c(5,8) - docorator defaults
#' @param path file path including the png location to save the output to
#' @return filepath to snap of ggplot
#' @keywords internal
#' @noRd
gg_to_image <- function(plot_object, fig_dim = c(5,8), path = NULL) {


  # Upgrade x to a list if only a single ggplot object is provided
  if (inherits(plot_object, "gg")) {
    plot_object <- list(plot_object)
  }

  vapply(
    seq_along(plot_object),
    FUN.VALUE = character(1L),
    USE.NAMES = FALSE,
    FUN = function(x) {

      filename <- tempfile(
        pattern = "temp_gt_ggplot_",
        tmpdir = path,
        fileext = ".png")

      # Save PNG file to disk
      ggplot2::ggsave(
        filename = filename,
        create.dir = TRUE,
        plot = plot_object[[x]],
        device = "png",
        dpi = 300,
        width = fig_dim[2],
        height = fig_dim[1],
        units = "in"
      )

      filename

    }
  )
}

#' strip header footer info from ggplot object
#' @param x a ggplot object
#'
#' @return list of header footer info
#'
#' @noRd
#' @keywords internal
hf_strip <- function(x){


  if (!inherits(x, "ggplot")) {
    cli::cli_abort("The display must be class `ggplot`, not {.obj_type_friendly {x}}.",
                   call = rlang::caller_env())
  }

  head_data <- x$labels$title
  # move tag to subtitle
  subhead_data <- c(x$labels$subtitle,x$labels$tag)
  foot_data <- x$labels$caption

  # set to null in ggplot object
  x$labels[c("title","subtitle", "caption", "tag")] <- NULL

  list(display = x,
       head_data = head_data,
       subhead_data = subhead_data,
       foot_data = foot_data)

}


#' Replace empty markdown labels in gt_tbl or gt_group with empty string
#' @param x a gt_tbl or gt_group
#' @keywords internal
#' @noRd
replace_empty_md <- function(x) {
  #TODO: remove this and set gt dep when this is released: https://github.com/rstudio/gt/pull/2120/changes

  if (inherits(x, "gt_tbl")) {
    # boxhead
    gt_tbl_labs <- x$`_boxhead`

    gt_tbl_labs <- gt_tbl_labs |>
      dplyr::mutate(
        column_label = lapply(.data$column_label, \(x) {
          if (inherits(x, "from_markdown")) {
            trimmed_lab <- unclass(trimws(x))
            if (identical(trimmed_lab, "")) {
              x <- ""
            }
          }
          x
        })
      )

    x$`_boxhead` <- gt_tbl_labs

     #stub
    gt_stub_labs <- x$`_stubhead`

    gt_stub_labs <- lapply(gt_stub_labs, \(x) {
      if (inherits(x, "from_markdown")) {
        trimmed_lab <- unclass(trimws(x))
        if (identical(trimmed_lab, "")) {
          x <- ""
        }
      }
      x
    })

    x$`_stubhead` <- gt_stub_labs

    # data - replace empty with invisible whitespace
    x$`_data`[x$`_data` == ""] <- "\u200B"
  }

  x
}

test_that("add footnotes and headers",{
  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    )


  docorator <- as_docorator(
    x = my_gt,
    header = fancyhead(
      fancyrow(center = "first line header"),
      fancyrow(left = "left header"),
      fancyrow(center = "second line header"),
      fancyrow(center = "third line header")
    ),
    footer = fancyfoot(
      fancyrow(left = "footnote 1"),
      fancyrow(left = "footnote 2"),
      fancyrow(right = "timestamp")),
    display_name = "my_first_gt",
    save_object = FALSE

  )

  head_foot_gt <- hf_to_gt(docorator)

  # footnote
  expect_equal(unlist(head_foot_gt$`_footnotes`$footnotes),c("footnote 1", "footnote 2"))
  # title
  expect_equal(head_foot_gt$`_heading`$title, "first line header")
  # subtitle
  expect_equal(head_foot_gt$`_heading`$subtitle, gt::md("second line header<br>third line header"))

})

test_that("add footnotes and headers - gt_group",{
  # create docorator object
  gt_group  <- gt::gt_group(gt::gt(head(mtcars)), gt::gt(tail(mtcars)))


  docorator <- as_docorator(
    x = gt_group,
    header = fancyhead(
      fancyrow(center = "first line header"),
      fancyrow(left = "left header"),
      fancyrow(center = "second line header"),
      fancyrow(center = "third line header")
    ),
    footer = fancyfoot(
      fancyrow(left = "footnote 1"),
      fancyrow(left = "footnote 2"),
      fancyrow(right = "timestamp")),
    display_name = "my_first_gt",
    save_object = FALSE

  )

# hf_to_gt is never called directly on a docorator with a gt_group, it is broken down into gt_tbls first
# run prep_obj_rtf to see if hfs are applied
  head_foot_gt_group <- prep_obj_rtf(docorator)

  # separate out the two tables
  tab1 <- gt::grp_pull(head_foot_gt_group,1)
  tab2 <- gt::grp_pull(head_foot_gt_group,2)

  # footnote
  expect_equal(unlist(tab1$`_footnotes`$footnotes),c("footnote 1", "footnote 2"))
  # title
  expect_equal(tab1$`_heading`$title, "first line header")
  # subtitle
  expect_equal(tab1$`_heading`$subtitle, gt::md("second line header<br>third line header"))

  # footnote
  expect_equal(unlist(tab2$`_footnotes`$footnotes),c("footnote 1", "footnote 2"))
  # title
  expect_equal(tab2$`_heading`$title, "first line header")
  # subtitle
  expect_equal(tab2$`_heading`$subtitle, gt::md("second line header<br>third line header"))


})

test_that("add footnotes and headers - existing subtitle info",{
  # create docorator object
  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    ) |>
    gt::tab_header(title = "",subtitle = "TRT = Placebo, <br>X = Y")



  docorator <- as_docorator(
    x = my_gt,
    header = fancyhead(
      fancyrow(center = "first line header"),
      fancyrow(left = "left header"),
      fancyrow(center = "second line header"),
      fancyrow(center = "third line header")
    ),
    footer = fancyfoot(
      fancyrow(left = "footnote 1"),
      fancyrow(left = "footnote 2"),
      fancyrow(right = "timestamp")),
    display_name = "my_first_gt",
    save_object = FALSE

  )

  head_foot_gt <- hf_to_gt(docorator)

  # subtitle
  expect_equal(head_foot_gt$`_heading`$subtitle, gt::md("second line header<br>third line header<br>TRT = Placebo, <br>X = Y"))

  # no additional subtitles
  docorator2 <- as_docorator(
    x = my_gt,
    display_name = "my_first_gt",
    save_object = FALSE
  )

  head_foot_gt2 <- hf_to_gt(docorator2)

  # subtitle
  expect_equal(head_foot_gt2$`_heading`$subtitle, gt::md("TRT = Placebo, <br>X = Y"))

})

test_that("add footnotes and headers - existing title info, no subtitle",{
  # create docorator object
  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    ) |>
    gt::tab_header(title = "existing title")



  docorator <- as_docorator(
    x = my_gt,
    header = fancyhead(
      fancyrow(center = "first line header"),
      fancyrow(left = "left header"),
      fancyrow(center = "second line header"),
      fancyrow(center = "third line header")
    ),
    footer = fancyfoot(
      fancyrow(left = "footnote 1"),
      fancyrow(left = "footnote 2"),
      fancyrow(right = "timestamp")),
    display_name = "my_first_gt",
    save_object = FALSE

  )

  head_foot_gt <- hf_to_gt(docorator)

  # subtitle
  expect_equal(head_foot_gt$`_heading`$subtitle, gt::md("second line header<br>third line header<br>existing title"))

  # no additional subtitles
  docorator2 <- as_docorator(
    x = my_gt,
    display_name = "my_first_gt",
    save_object = FALSE
  )

  head_foot_gt2 <- hf_to_gt(docorator2)

  # subtitle
  expect_equal(head_foot_gt2$`_heading`$subtitle, gt::md("existing title"))

})

test_that("add footnotes and headers - existing title + subtitle info",{
  # create docorator object
  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    ) |>
    gt::tab_header(title = "existing title",subtitle = "TRT = Placebo, <br>X = Y")



  docorator <- as_docorator(
    x = my_gt,
    header = fancyhead(
      fancyrow(center = "first line header"),
      fancyrow(left = "left header"),
      fancyrow(center = "second line header"),
      fancyrow(center = "third line header")
    ),
    footer = fancyfoot(
      fancyrow(left = "footnote 1"),
      fancyrow(left = "footnote 2"),
      fancyrow(right = "timestamp")),
    display_name = "my_first_gt",
    save_object = FALSE

  )

  head_foot_gt <- hf_to_gt(docorator)

  # subtitle
  expect_equal(head_foot_gt$`_heading`$subtitle, gt::md("second line header<br>third line header<br>existing title<br>TRT = Placebo, <br>X = Y"))

  # no additional subtitles
  docorator2 <- as_docorator(
    x = my_gt,
    display_name = "my_first_gt",
    save_object = FALSE
  )

  head_foot_gt2 <- hf_to_gt(docorator2)

  # subtitle
  expect_equal(head_foot_gt2$`_heading`$subtitle, gt::md("existing title<br>TRT = Placebo, <br>X = Y"))

})


test_that("no footnotes or headers",{
  my_gt <- gt::exibble |>
    gt::gt(
      rowname_col = "row",
      groupname_col = "group"
    )


  docorator <- as_docorator(
    x = my_gt,
    header = NULL,
    footer = NULL,
    display_loc ="test",
    display_name = "my_first_gt",
    save_object = FALSE
  )

  head_foot_gt <- hf_to_gt(docorator)

  # footnote
  expect_equal(unlist(head_foot_gt$`_footnotes`$footnotes),NULL)
  # title
  expect_equal(head_foot_gt$`_heading`$title, NULL)
  # subtitle
  expect_equal(head_foot_gt$`_heading`$subtitle, NULL)


  # no subtitle but title
  docorator2 <- as_docorator(
    x = my_gt,
    header = fancyhead(
      fancyrow(center = "first line header")
    ),
    footer = NULL,
    display_loc ="test",
    display_name = "my_first_gt",
    save_object = FALSE

  )

  head_foot_gt2 <- hf_to_gt(docorator2)

  # footnote
  expect_equal(unlist(head_foot_gt2$`_footnotes`$footnotes),NULL)
  # title
  expect_equal(head_foot_gt2$`_heading`$title, "first line header")
  # subtitle
  expect_equal(head_foot_gt2$`_heading`$subtitle, NULL)
})

test_that("apply_to_gt_group works",{
  # create gt group example
  gt_tbl <- gt::exibble |> gt::gt()
  gt_group <- gt::gt_group(gt_tbl, gt_tbl)

  # bad object
  arg_list <- list(data = mtcars)
  expect_error(apply_to_gt_group(mtcars, gt::tab_options, arg_list), 'First arg must be a gt_tbl or gt_group object, not a data frame')

  # create arguments - cols_align function
  gt_group <- gt::gt_group(gt_tbl, gt_tbl)
  func <- gt::tab_options
  arg_list <- list(page.header.use_tbl_headings = c(TRUE))

  # aligned gt_tbl
  options_tbl <- gt_tbl |>
    gt::tab_options(
      page.header.use_tbl_headings = TRUE
    )

  # aligned group 2 ways: one via apply_to_group one via individual aligned tables
  options_group <- apply_to_gt_group(gt_group,func,arg_list)
  expect_identical(options_group, gt::gt_group(options_tbl, options_tbl))

  # check apply_to_gt_group works for gt_tbl
  expect_identical(options_tbl, apply_to_gt_group(gt_tbl, func, arg_list))
})

test_that("Create png from ggplot", {

  withr::with_tempdir({
    gg <- ggplot2::ggplot(mtcars) +
      ggplot2::aes(x = disp, y = mpg) +
      ggplot2::geom_point()


    image_paths <- gg_to_image(gg, path = getwd())

    expect_length(image_paths, 1)

      expect_equal(
        file.exists(image_paths),
        c(TRUE)
      )
  })

})

test_that("Create set of png from list of ggplots", {

  withr::with_tempdir({
    gg1 <- ggplot2::ggplot(mtcars) +
      ggplot2::aes(x = disp, y = mpg) +
      ggplot2::geom_point()


    gg2 <- ggplot2::ggplot(mtcars) +
      ggplot2::aes(x = hp, y = mpg) +
      ggplot2::geom_point()

    image_paths <- gg_to_image(list(gg1, gg2), path = getwd())

    expect_length(image_paths, 2)

    expect_equal(
      file.exists(image_paths),
      c(TRUE, TRUE)
    )
  })
})

test_that("Extract header footer information from ggplot", {

  ggplot1 <- ggplot2::ggplot(data = mtcars, ggplot2::aes(y=cyl, x=mpg)) +
    ggplot2::geom_point() +
    ggplot2::labs(title = "title1", subtitle = "subtitle1", alt = "alt", tag = "tag1", caption = "footnote1")

  stripped_ggplot <- ggplot2::ggplot(data = mtcars, ggplot2::aes(y=cyl, x=mpg)) +
    ggplot2::geom_point() +
    ggplot2::labs(alt = "alt")

  hf_ggplot <- hf_strip(ggplot1)

  expect_equal(stripped_ggplot$labels, hf_ggplot$display$labels)

  expect_equal(hf_ggplot$head_data, "title1")
  expect_equal(hf_ggplot$subhead_data, c("subtitle1", "tag1"))
  expect_equal(hf_ggplot$foot_data, "footnote1")

})

test_that("replace empty md labels works", {
  gt_tbl <- gt::exibble |>
    gt::gt() |>
    gt::cols_label(
      num = gt::md(" "),
      char = gt::md("char")
    )

  # stubheader blank md()
  gt_with_stub <- gt::exibble |>
    gt::gt(rowname_col = "row") |>
    gt::tab_stubhead(gt::md(""))

  # stub md() not empty
  gt_with_char_stub <- gt::exibble |>
    gt::gt(rowname_col = "row") |>
    gt::tab_stubhead(gt::md("test"))

  # md in the body of the table
  gt_with_md_body <- gt::exibble |>
    dplyr::mutate(char = "") |>
    gt::gt() |>
    gt::fmt_markdown()

  expect_equal(
    lapply(gt_tbl$`_boxhead`$column_label, class),
    list(
      "from_markdown",
      "from_markdown",
      "character",
      "character",
      "character",
      "character",
      "character",
      "character",
      "character"
    )
  )

  cleaned_gt <- replace_empty_md(gt_tbl)

  expect_equal(
    lapply(cleaned_gt$`_boxhead`$column_label, class),
    list(
      "character",
      "from_markdown",
      "character",
      "character",
      "character",
      "character",
      "character",
      "character",
      "character"
    )
  )

  # no stub so cleaning shouldnt impact the stubhead
  expect_identical(
    gt_tbl$`_stubhead`,
    cleaned_gt$`_stubhead`
  )

  # with stub

  expect_equal(
    class(gt_with_stub$`_stubhead`$label),
    "from_markdown"
  )

  # stub isnt markdown in the boxhead
  expect_equal(
    lapply(gt_with_stub$`_boxhead`$column_label, class),
    list(
      "character",
      "character",
      "character",
      "character",
      "character",
      "character",
      "character",
      "character",
      "character"
    )
  )

  cleaned_gt_stub <- replace_empty_md(gt_with_stub)

  expect_equal(
    class(cleaned_gt_stub$`_stubhead`$label),
    "character"
  )

  expect_equal(
    lapply(cleaned_gt_stub$`_boxhead`$column_label, class),
    list(
      "character",
      "character",
      "character",
      "character",
      "character",
      "character",
      "character",
      "character",
      "character"
    )
  )

  # non empty stub md()

  cleaned_with_char_stub <- replace_empty_md(gt_with_char_stub)

  # shouldnt change the stubhead
  expect_equal(
    cleaned_with_char_stub$`_stubhead`,
    gt_with_char_stub$`_stubhead`
  )

  # empty md in the body of the table
  expect_equal(gt_with_md_body$`_data`$char, c("", "", "", "", "", "", "", ""))
  cleaned_md_body <- replace_empty_md(gt_with_md_body)
  expect_equal(
    cleaned_md_body$`_data`$char,
    c(
      "\u200B",
      "\u200B",
      "\u200B",
      "\u200B",
      "\u200B",
      "\u200B",
      "\u200B",
      "\u200B"
    )
  )
})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, eval = FALSE------------------------------------------------------
# library(docorator)
# library(ggplot2)
# library(gt)
# 
# # Create a plot
# plot <- ggplot(mtcars, aes(wt, mpg)) +
#   geom_point() +
#   labs(title = "Weight vs MPG")
# 
# # Create a table
# tbl <- head(mtcars) |>
#   gt::gt()
# 
# # Combine into a list
# output_list <- list(
#   plot,
#   tbl
# )
# 
# # Run `as_docorator` and set headers and footers
# as_docorator(
#   output_list,
#   display_name = "multi_page_example",
#   header = fancyhead(fancyrow(right = doc_pagenum()), # `doc_pagenum` will set pagenumbers in the right header
#                      fancyrow(center = "Multipage Display")),
#   footer = fancyfoot(fancyrow(left = "Footnote1"))
# ) |>  # render
# render_pdf()

#' Prepare table, listing, figure object for inclusion in the template Rmd
#'
#' @param x docorator object containing info about the table, listing or figure
#'
#' @return xml to be inserted into word document
#' @export
#' @keywords internal
#'
#' @examples
#' docorator <- gt::exibble |>
#' gt::gt() |>
#' as_docorator(
#' display_name = "mytbl", footer = NULL,
#' save_object = FALSE)
#'
#' prepared_obj <- prep_obj_docx(docorator)
prep_obj_docx <- function(x, ...) {
  UseMethod("prep_obj_docx", x$display)
}

#' @rdname prep_obj_docx
#' @export
#' @keywords internal
prep_obj_docx.default <- function(x, ...) {
  polish::polish_content_word(x$display)
}

#' @rdname prep_obj_docx
#' @export
#' @keywords internal
prep_obj_docx.PNG <- function(x, ...) {
  # save the png to a temp location and then read in with as_file from polish to get the xml
  # temporarily store png
  temp <- tempfile(fileext = ".png", tmpdir = tempdir())
  png::writePNG(x$display$png, temp)
  polish::polish_content_word(polish::as_file(temp), width = x$fig_dim[2], height = x$fig_dim[1])
}

#' @rdname prep_obj_docx
#' @export
#' @keywords internal
prep_obj_docx.gt_tbl <- function(x, ...) {
  gt_to_word(x$display)
}

#' @rdname prep_obj_docx
#' @export
#' @keywords internal
prep_obj_docx.list <- function(x, ...) {
  # list of ooxml - one for each element in the list
  lapply(x$display, function(i) {
    new_docorator <- x
    new_docorator$display <- i
    prep_obj_docx(new_docorator)
  })
}

#' convert gt_tbl object to ooxml
#' @noRd
gt_to_word <- function(x) {
  # replace empty mds with a string
  x <- replace_empty_md(x)

  # apply options to the gt
  # Arial font
  font <- "Arial"
  x <- x |>
    gt::tab_style(
      style = gt::cell_text(font = c(font)), ## size is in half points
      locations = list(
        gt::cells_title(),
        gt::cells_column_labels(),
        gt::cells_column_spanners(),
        gt::cells_stubhead(),
        gt::cells_stub(),
        gt::cells_body(),
        gt::cells_row_groups(),
        gt::cells_footnotes()
      )
    ) |>
    gt::tab_options(
      table.font.names = c(font)
    )

  # get ooxml from gt
  ooxml <- polish::polish_content_word(
    x,
    autonum = FALSE,
    keep_with_next = FALSE
  )

  # as_word from gt returns multiple ooxml elements
  # wrap these elements together so that they can be read in as xml for officer
  paste0(
    "<tablecontainer>",
    paste0(ooxml, collapse = ""),
    "</tablecontainer>"
  ) |>
    xml2::read_xml() |>
    xml2::xml_children() |>
    suppressWarnings()
}

test_that("gt_to_word outputs the correct ooxml", {
  gt_tbl <- gt::exibble |>
    gt::gt() |>
    gt::tab_header("header") |>
    gt::tab_footnote("footnote")

  ooxml <- gt_to_word(gt_tbl)
  expect_equal(class(ooxml), "xml_nodeset")

  # should be length 2 - one for the caption and one for the main body
  expect_equal(length(ooxml), 2)

  # should be Arial for both the caption and the main body
  caption <- as.character(ooxml[[1]])
  main_body <- as.character(ooxml[[2]])
  expect_true(grepl("Arial", caption))
  expect_true(grepl("Arial", main_body))

  # should contain header body and footer text
  expect_true(grepl("header", caption))
  expect_true(grepl("honeydew", main_body))
  expect_true(grepl("footnote", main_body))
})

test_that("prep_obj_docx works correctly for gt_group", {
  gt_tbl <- gt::exibble |>
    gt::gt() |>
    gt::tab_header("header") |>
    gt::tab_footnote("footnote")

  gt_tbl2 <- gt::exibble |>
    gt::gt() |>
    gt::tab_header("heading2") |>
    gt::tab_footnote("footer2")

  gt_group <- gt::gt_group(gt_tbl, gt_tbl2)

  docorator <- as_docorator(
    gt_group,
    display_name = "mytbl",
    footer = NULL,
    save_object = FALSE
  )

  ooxml <- prep_obj_docx(docorator)

  # should be a list length 2 - one for each gt_tbl
  expect_equal(length(ooxml), 2)
  # should be class list
  expect_equal(class(ooxml), "list")

  # table 1
  ooxml_1 <- ooxml[[1]]
  # should be length 2 - one for the caption and one for the main body
  expect_equal(length(ooxml_1), 2)

  # should be Arial for both the caption and the main body
  caption <- as.character(ooxml_1[[1]])
  main_body <- as.character(ooxml_1[[2]])
  expect_true(grepl("Arial", caption))
  expect_true(grepl("Arial", main_body))

  # should contain header body and footer text
  expect_true(grepl("header", caption))
  expect_true(grepl("honeydew", main_body))
  expect_true(grepl("footnote", main_body))

  # table 2
  ooxml_2 <- ooxml[[2]]
  # should be length 2 - one for the caption and one for the main body
  expect_equal(length(ooxml_2), 2)
  # should contain header body and footer text
  caption <- as.character(ooxml_2[[1]])
  main_body <- as.character(ooxml_2[[2]])
  expect_true(grepl("heading2", caption))
  expect_true(grepl("honeydew", main_body))
  expect_true(grepl("footer2", main_body))
})

#' Render to docx
#'
#' `r lifecycle::badge('experimental')`
#'
#' @param x `docorator` object
#' @param display_loc optional path to save the output docx to
#' @param version_check Boolean indicating whether to print a note if gt or ggplot versions dont match between the original docorator object and the one being used for rendering
#'
#' @returns This function saves a docx to a specified location
#' @export
#'
#' @section Examples:

#' ```r
#' gt::gtcars |>
#'   dplyr::slice_head(n = 10) |>
#'   dplyr::select(mfr, model, year, msrp) |>
#'   gt::gt(groupname_col = "mfr",
#'          row_group_as_column = TRUE) |>
#'   as_docorator(
#'    header = fancyhead(fancyrow("Header 1"), fancyrow("Header 2")),
#'    display_name = "mytbl") |>
#'  render_docx()
#' ```
#'
render_docx <- function(x, display_loc = NULL, version_check = TRUE) {
  if (!inherits(x, "docorator")) {
    cli::cli_abort(
      "The {.arg {rlang::caller_arg(x)}} argument must be class docorator, not {.obj_type_friendly {x}}. See documentation for `as_docorator`.",
      call = rlang::caller_env()
    )
  }

  # check package versions
  if (isTRUE(version_check)) {
    check_pkg_version(x)
  }

  engine = "docx"

  # if no path is given, use docorator path
  if (is.null(display_loc)) {
    display_loc <- x$display_loc %||% "."
  }

  # initialize empty docx file
  doc <- officer::read_docx(system.file(
    "template/template.docx",
    package = "docorator"
  ))

  # convert geometry values to numeric for officer
  geometry <- geom_process(geometry = x$geometry, engine = engine)

  # page properties
  # these are all in inches
  page_margins <- officer::page_mar(
    header = geometry$top,
    footer = geometry$bottom,
    left = geometry$left,
    right = geometry$right,
    gutter = 0
  )

  page_size <- officer::page_size(
    orient = "landscape",
    width = 11,
    height = 8.5
  )

  section_args <- list(
    page_margins = page_margins,
    page_size = page_size
  )

  # process headers and footers
  fpar_head_list <- hf_process(x$header, fontsize = x$fontsize, engine = engine)
  fpar_foot_list <- hf_process(x$footer, fontsize = x$fontsize, engine = engine)

  if (length(fpar_head_list) > 0) {
    section_args$header_default <- do.call(officer::block_list, fpar_head_list)
  }
  if (length(fpar_foot_list) > 0) {
    section_args$footer_default <- do.call(officer::block_list, fpar_foot_list)
  }

  ps <- do.call(officer::prop_section, section_args)

  # insert the body content
  xml <- prep_obj_docx(x)

  # # ensure that xml is a list
  if (!identical(class(xml), "list")) {
    xml <- list(xml)
  }

  # for every element in the list add to the body of the document, separate by page breaks
  for (i in seq_along(xml)) {
    if (i > 1) {
      doc <- officer::body_add_break(doc, pos = "after")
    }
    xml_element <- xml[[i]]
    for (j in seq_along(xml_element)) {
      doc <- officer::body_add_xml(doc, str = xml_element[[j]])
    }
  }

  doc <- officer::body_set_default_section(doc, value = ps)

  path <- print(doc, target = file.path(display_loc, paste0(x$display_name, ".docx")))

  if (!is.null(doc)) {
    cli::cli_alert_success("Document created at:  {normalizePath(path, winslash = \"/\")}")
  }

  invisible(x)
}

#' Prepare table, listing, figure object for output to rtf
#'
#' @param x docorator object containing display information
#' @param ... additional args
#'
#' @return gt object to be included as-is to render engine
#' @export
#' @keywords internal
#'
#' @examples
#' docorator <- gt::exibble |>
#' gt::gt() |>
#' as_docorator(
#' display_name = "my_tbl",
#' footer=NULL,
#' save_object = FALSE)
#'
#' prepared_obj <- prep_obj_rtf(docorator)
prep_obj_rtf <- function (x, ...) {
  UseMethod("prep_obj_rtf", x$display)
}

#' @rdname prep_obj_rtf
#' @export
#' @keywords internal
prep_obj_rtf.default <- function(x,  ...) {
  display<- x$display
  cli::cli_abort("For RTF render the display must be class character, gt_tbl, gt_group, PNG, or ggplot, not {.obj_type_friendly {display}}.",
                   call = rlang::caller_env())
}

#' @rdname prep_obj_rtf
#' @export
#' @keywords internal
prep_obj_rtf.character <- function(x,  ...) {
  x$display <- dplyr::tibble(character = x$display) |>
    gt::gt() |>
    gt::cols_label(character = "")
  hf_to_gt(x)
}

#' @rdname prep_obj_rtf
#' @export
#' @keywords internal
prep_obj_rtf.PNG <- function(x,  ... ) {

  x$display <- png_to_gt(x)
  hf_to_gt(x)

}

#' @rdname prep_obj_rtf
#' @export
#' @keywords internal
prep_obj_rtf.ggplot <- function(x,  ... ) {
  x$display <- gg_to_gt(x)
  hf_to_gt(x)

}

#' @rdname prep_obj_rtf
#' @export
#' @keywords internal
prep_obj_rtf.gt_tbl <- function(x,  ...) {

  hf_to_gt(x)

}

#' @rdname prep_obj_rtf
#' @export
#' @keywords internal
prep_obj_rtf.list <- function(x,  ...) {
  prepped_list <-lapply(x$display, function(j){
    new_docorator <- x
    new_docorator$display <- j
    prep_obj_rtf(new_docorator)
  })

  gt::gt_group(.list = prepped_list)

}

#' Decorate and output a table, listing, or figure to a file
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#'
#' This function was deprecated and replaced with `as_docorator` and a corresponding render function
#' i.e `render_pdf`
#'
#' @param x object containing the display. See @details for more information.
#' @param filename required name of file including extension (note: only PDF supported currently)
#' @param path optional path to save the output pdf to
#' @param header Document header. Accepts a `fancyhead` object. If `NULL`, no header will be displayed.
#' @param footer Document footer Accepts a `fancyfoot` object. If `NULL`, no footer will be displayed.
#' @param ... These dots are for future extensions and must be empty.
#' @param fontsize Font size (pt). Defaults to `10`. Accepted values: 10, 11, 12.
#' @param geometry Document sizing options based on the `geometry` latex
#'   package. Accepts a named list. Default is `geom_set()`.
#' @param fig_dim vector containing figure height and width in inches. Defaults
#'   to `c(5,8)`
#' @param tbl_scale Boolean for whether or not to automatically scale table columns to fit display area. Defaults to TRUE. Note that this will overwrite scaling set in the table directly unless set to FALSE.
#' @param tbl_stub_pct percent of total width that should be dedicated to stub column(s). If more than 1 stub column then this is the total for both.
#'
#' @details
#' While the `x` argument flexibly accepts many different R objects, the
#' following classes/types are recommended:
#'
#'  - `gt`
#'  - `gt_group` (list of `gt` objects)
#'  - `ggplot`
#'  - list of `ggplot`s
#'  - path to PNG file created via `png_path()`
#'  - list of paths to PNG files created via `png_path()`
#'
#' @return This function is called for its side effects
#' @export
#'
#' @examples
#' \dontrun{
#' gt::gtcars |>
#'   dplyr::slice_head(n = 10) |>
#'   dplyr::select(mfr, model, year, msrp) |>
#'   gt::gt(groupname_col = "mfr",
#'          row_group_as_column = TRUE) |>
#'   docorate(
#'    header = fancyhead(fancyrow("Header 1"), fancyrow("Header 2")),
#'    filename = "mytbl.pdf")
#'  }
#'
docorate <- function(
  x,
  filename,
  path = NULL,
  header = fancyhead(fancyrow(right = doc_pagenum())),
  footer = fancyfoot(fancyrow(
    left = doc_path(filename, path),
    right = doc_datetime()
  )),
  ...,
  fontsize = 10,
  geometry = geom_set(),
  fig_dim = c(5, 8),
  tbl_scale = TRUE,
  tbl_stub_pct = 0.3
) {
  lifecycle::deprecate_stop(
    when = "0.3.0",
    what = "docorator::docorate()",
    details = "Please use `as_docorator()` and the required render function, i.e `render_pdf()`",
    env = rlang::caller_env()
  )

  # check that name has been passed
  if (missing(filename)) {
    cli::cli_abort(
      "The {.arg {rlang::caller_arg(filename)}} argument must be specified",
      call = rlang::caller_env(),
      .envir = parent.frame()
    )
  }

  as_docorator(
    x,
    display_name = tools::file_path_sans_ext(filename),
    display_loc = path,
    header = header,
    footer = footer,
    save_object = FALSE,
    object_loc = path,
    ...,
    fontsize = fontsize,
    geometry = geometry,
    fig_dim = fig_dim,
    tbl_scale = tbl_scale,
    tbl_stub_pct = tbl_stub_pct
  ) |>
    render_pdf()
}

#' Create docorator object
#'
#' @param x object containing the display. See @details for more information.
#' @param display_name required name of file (excluding extension)
#' @param display_loc optional path to save the output file to
#' @param header Document header. Accepts a `fancyhead` object. If `NULL`, no header will be displayed.
#' @param footer Document footer Accepts a `fancyfoot` object. If `NULL`, no footer will be displayed.
#' @param save_object Boolean indicating if a docorator object should be saved.
#' @param object_loc path for the docorator object -  defaults to display_loc argument.
#' @param ... These dots are for future extensions and must be empty.
#' @param fontsize Font size (pt). Defaults to `10`. Accepted values: 10, 11, 12.
#' @param geometry Document sizing options based on the `geometry` latex
#'   package. Accepts a named list. Default is `geom_set()`.
#' @param fig_dim vector containing figure height and width in inches. Defaults
#'   to `c(5,8)`
#' @param convert_ggplot Boolean for whether or not to convert ggplot objects to PNG files to preserve scaling for all render engines. Defaults to TRUE.
#' @param tbl_scale Boolean for whether or not to automatically scale table columns to fit display area. Defaults to TRUE. Note that this will overwrite scaling set in the table directly unless set to FALSE.
#' @param tbl_stub_pct percent of total width that should be dedicated to stub column(s). If more than 1 stub column then this is the total for both.
#'
#' @details
#' While the `x` argument flexibly accepts many different R objects, the
#' following classes/types are recommended:
#'
#'  - `gt`
#'  - `gt_group` (list of `gt` objects)
#'  - `ggplot`
#'  - list of `ggplot`s
#'  - path to PNG file created via `png_path()`
#'  - list of paths to PNG files created via `png_path()`
#'
#' @return docorator object
#' @export
#'
#' @examples
#'
#' \dontrun{
#' gt::gtcars |>
#'   dplyr::slice_head(n = 10) |>
#'   dplyr::select(mfr, model, year, msrp) |>
#'   gt::gt(groupname_col = "mfr",
#'          row_group_as_column = TRUE) |>
#'   as_docorator(
#'    header = fancyhead(fancyrow("Header 1"), fancyrow("Header 2")),
#'    display_name = "mytbl",
#'    footer = NULL)
#' }
#'
as_docorator <- function(
  x,
  display_name,
  display_loc = NULL,
  header = fancyhead(fancyrow(right = doc_pagenum())),
  footer = NULL,
  save_object = TRUE,
  object_loc = display_loc,
  ...,
  fontsize = 10,
  geometry = geom_set(),
  fig_dim = c(5, 8),
  convert_ggplot = TRUE,
  tbl_scale = TRUE,
  tbl_stub_pct = 0.3
) {
  if (inherits(header, "character")) {
    lifecycle::deprecate_stop(
      "0.1.0",
      what = I(
        "Support of character vectors as input to the `header` argument of `as_docorator()`"
      ),
      details = "Please provide a `fancyhdr` object instead.",
      env = rlang::caller_env()
    )
  }

  # check that name has been passed
  if (missing(display_name)) {
    cli::cli_abort(
      "The {.arg {rlang::caller_arg(display_name)}} argument must be specified",
      call = rlang::caller_env(),
      .envir = parent.frame()
    )
  }

  # check inputs
  check_fancyhdr(header, chr_ok = TRUE)
  check_fancyhdr(footer)

  # prep displays for render
  x <- prep_display(
    x,
    fig_dim = fig_dim,
    convert_ggplot = convert_ggplot,
    fontsize = fontsize,
    tbl_scale = tbl_scale,
    tbl_stub_pct = tbl_stub_pct
  )

  docorator_obj <- structure(
    list(
      display = x,
      display_name = display_name,
      display_loc = display_loc,
      header = header,
      footer = footer,
      fontsize = fontsize,
      geometry = geometry,
      fig_dim = fig_dim,
      convert_ggplot = convert_ggplot,
      session_info = utils::sessionInfo()
    ),
    class = "docorator"
  )

  # save docorator object
  if (save_object) {
    object_filename <- paste0(display_name, ".RDS")
    object_loc <- object_loc %||% "."
    object_loc <- file.path(object_loc, object_filename)
    saveRDS(object = docorator_obj, file = object_loc)
  }

  docorator_obj
}

---
title: "Headers and Footers"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Headers and Footers}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(docorator)
```

A core feature of {docorator} is flexible header and footer options via the `header` and `footer` arguments in the `as_docorator()` function. These are made possible via the underlying [`fancyhdr`](https://www.overleaf.com/learn/latex/Headers_and_footers#Structure_of_fancyhdr_headers_and_footers) latex library. This makes it possible to have multiple lines of headers and/or footers, as well as multiple positions of text within each.

# Specifying headers 

The `header` argument accepts a `fancyhead` object. The `fancyhead` object accepts a series of `fancyrow` objects. These correspond to each line of text in the header. Within a `fancyrow`, you can optionally specify the left, center, and right text (`NA` indicates a blank position). Some examples are below.

## One line header 
```{r}
#| eval: false
fancyhead(
  fancyrow(left = "My Study", center = NA, right = NA)
)
```

## Multi-line header 
```{r}
#| eval: false
fancyhead(
  fancyrow(left = "My Study", center = NA, right = "Date as of today"),
  fancyrow(left = "My Population", center = NA, right = NA),
  fancyrow(left = NA, center = "My Table", right = NA)
)
```

# Specifying footers

The `footer` argument of `as_docorator()` accepts a `fancyfoot` object. This behaves identically to `fancyhead` above by accepting a series of `fancyrow` objects for each line of the footer. 

## One line footer 
```{r}
#| eval: false
fancyfoot(
  fancyrow(left = "Date as of today", center = NA, right = NA)
)
```

## Multi-line footer 
```{r}
#| eval: false
fancyfoot(
  fancyrow(left = "General footer for the table", center = NA, right = NA),
  fancyrow(left = "Date as of today", center = NA, right = "path/to/docname.pdf")
)
```

# Available utilities for standard or automated information

There are several functions available for features like page numbering and time stamps. These are prefixed with `doc_*()`. 

## Add automatic page numbering in upper left 
```{r}
#| eval: false
fancyhead(
  fancyrow(left = "My study", center = NA, right = doc_pagenum())
)
```

## Add file path in bottom left and datetime in bottom right
```{r}
#| eval: false
fancyfoot(
  fancyrow(left = doc_path(), center = NA, right = doc_datetime())
)
```
# Putting it all together

```{r}
#| eval: false
gt::gtcars |>
  dplyr::slice_head(n = 10) |>
  dplyr::select(mfr, model, year, msrp) |>
  gt::gt(
    groupname_col = "mfr",
    row_group_as_column = TRUE
  ) |> 
  as_docorator( 
    display_name = "mytbl",
    header = fancyhead(
      fancyrow(left = "My Study", center = NA, right = doc_pagenum()),
      fancyrow(left = "My Population", center = NA, right = NA),
      fancyrow(left = NA, center = "My Table", right = NA)
    ),
    footer = fancyfoot(
      fancyrow(
        left = "path/mytbl.pdf", center = NA, right = "Data as of 2025-01-01"
      )
    ) 
  ) |> 
  render_pdf()
```

```{r}
#| echo: false
#| fig-align: center
#| out-width: 700px
#| fig-alt: Docorator example
knitr::include_graphics("img/doc_example.png")
```

# Note about sizing

{docorator} will automatically adjust the allowed height for headers and footers according to the lines of text required. If many lines of headers and/or footers are specified, this cuts down on the available height for the table/figure. 

*NEW as of v0.6.0* {docorator} can auto-wrap long lines of text in PDF documents via the `fancywrap` argument of `render_pdf()`. See `?render_pdf` for details. 

Please file an [issue](https://github.com/GSK-Biostatistics/docorator/issues) if you discover unexpected results or problems related to headers and footers. 

---
title: "Document Sizing"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Document Sizing}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(docorator)
```

Document-level sizing can be customized via the `geometry` argument. This argument accepts a named list of options that are used by the underlying [`geometry`](https://texdoc.org/serve/geometry.pdf/0) latex package.

## Default sizing options

The default for the `geometry` package is `geom_set()`, which includes the following settings:

```{r}
geom_set()
```

Users can optionally override any defaults by passing new values as such:

```{r}
geom_set(top = "0.5in", bottom = "0.5in")
```


For values set to `NULL`, {docorator} will perform automatic size calculations. These can be overridden by hardcoded size values.

```{r}
geom_set(headheight = "20pt")
```


## Advanced use

Advanced users can pass additional geometry settings/values that are not covered by the defaults. These additional settings will be appended to the default list. Reference the `geometry` documentation for more information: https://texdoc.org/serve/geometry.pdf/0.

---
output:
  pdf_document:
    latex_engine: xelatex
    keep_tex:  false  
classoption: landscape
params:
  x: NULL 
  header: NULL
  footer: NULL
  geometry: NULL
  transform: NULL
fontsize: "`r paste0(params$x$fontsize, 'pt')`"
geometry: "`r params$geometry`" 
header-includes:  
  - \renewcommand*\familydefault{\ttdefault}
  - \usepackage{fancyhdr, lastpage}
  - \usepackage{booktabs, longtable, colortbl, array, multirow, anyfontsize}
  - \usepackage[labelformat=empty]{caption} 
  - \IfFileExists{header.tex}{\input{header.tex}}{}
fig_crop: FALSE
---

<!--- Define Headers and Footers --->
```{=tex}
\fancypagestyle{plain}{
  \renewcommand{\headrulewidth}{0pt}
  \fancyhf{}%
  `r params$header`
  `r params$footer`
}
\pagestyle{plain} 
```
 
\centering


```{r echo = FALSE, results = "asis"}
create_chunks_all(params$x, params$transform)
```

---
title: "Getting Started"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

After preparing a table or figure in R, users can easily prepare their display for rendering with a call to `as_docorator`:

```
library(docorator)

as_docorator(
  mytbl,
  display_name = "mytbl"
)
```

This will by default save a docorator object in the form of a .RDS file in the current working directory. Users can optionally pass a different directory to save the file and subsequent renders of the object via the `display_loc` argument.

To render this display, the `as_docorator` call is paired with a render function, for example `render_pdf`. 

```
library(docorator)

as_docorator(
  mytbl,
  display_name = "mytbl"
) |> 
render_pdf()
```

Additional customization options can be found in the "Display Sizing" and "Headers and Footers" vignettes.

More customization options will be offered in future releases. If you have a need to customize something that is not currently offered, please file an [issue](https://github.com/GSK-Biostatistics/docorator/issues).

---
format: 
  pdf:
    keep-tex: true
    latex-auto-install: false
params:
  display_name: ""
  pkg_path: ""
  transform: NULL

---

```{r echo = FALSE}

if(dir.exists(params$pkg_path)){
  devtools::load_all(params$pkg_path, export_all = FALSE, quiet = TRUE)
}else{
  suppressPackageStartupMessages(library(docorator))
}

```
  
```{r echo = FALSE}
dd <- readRDS(paste0(params$display_name, "_docorator_obj.Rds"))
geom <- docorator:::geom_process(dd$header,
                     dd$footer,
                     dd$fontsize,
                     dd$geometry)
```

---
geometry: "`r paste0(geom)`"
fontsize: "`r paste0(dd$fontsize, 'pt')`"
include-in-header:  
  - text: | 
      \renewcommand*\familydefault{\ttdefault}
      \usepackage{fancyhdr, lastpage}
      \usepackage{booktabs, longtable, colortbl, array, multirow, anyfontsize}
      \usepackage[labelformat=empty]{caption} 
      \usepackage{float}
      \floatplacement{table}{H}
      \IfFileExists{header.tex}{\input{header.tex}}{}
---
 
<!--- Define Headers and Footers --->
```{=tex}
\fancypagestyle{plain}{
  \renewcommand{\headrulewidth}{0pt}
  \fancyhf{}%
  `r docorator:::hf_process(dd$header)`
  `r docorator:::hf_process(dd$footer)`
}
\pagestyle{plain} 
```
 
\centering
 
```{r echo = FALSE, results = "asis"}
if (inherits(dd$display, c("gt_tbl","gt_group"))){
  prep_obj_tex(dd, params$transform)
} else {
  create_chunks_all(dd, params$transform)
}
```

---
title: "Multi Page Displays"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Multi Page Displays}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

Below we demonstrate how a list of objects can be passed to `as_docorator` to output a multi-page PDF document, with page numbers generated using `doc_pagenum()`. Note: `doc_pagenum()` is only supported for PDF output.

Your list can consist of any combination of docorator supported objects e.g gt tables, ggplots, PNGs etc. In the example below we create a list with a ggplot and a gt table.
```{r setup, eval = FALSE}
library(docorator)
library(ggplot2)
library(gt)

# Create a plot
plot <- ggplot(mtcars, aes(wt, mpg)) +
  geom_point() +
  labs(title = "Weight vs MPG")

# Create a table
tbl <- head(mtcars) |>
  gt::gt()

# Combine into a list
output_list <- list(
  plot,
  tbl
)

# Run `as_docorator` and set headers and footers
as_docorator(
  output_list,
  display_name = "multi_page_example",
  header = fancyhead(fancyrow(right = doc_pagenum()), # `doc_pagenum` will set pagenumbers in the right header
                     fancyrow(center = "Multipage Display")),
  footer = fancyfoot(fancyrow(left = "Footnote1"))
) |>  # render
render_pdf()
```

This will produce a PDF with a plot on the first page, a table on the second, and page numbers in the top right corner.

---
title: "Render"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Render}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(docorator)
```

Running `as_docorator()` does not render your display - it creates a `docorator` object to store all your display information. For rendering, an explicit call to a render function is needed. A display can be output to PDF, RTF, or both via a combination of the `render_pdf()` and `render_rtf()` functions.

Further, with `render_pdf()`, users have a choice of R Markdown or Quarto engines via the `quarto` option. Currently, the result is nearly identical, except for {gt} tables, where `quarto = FALSE` renders the `gt` via the `longtable` LaTeX library and `quarto = TRUE` via the `tabular` LaTeX library. Please note the Quarto engine is currently experimental and can take longer to render. 

## Render to PDF

```
as_docorator(
  mytbl,
  display_name = "mytbl"
) |> 
render_pdf()
```

If you understand the underlying LaTeX involved in going from a display object to PDF, `render_pdf` has a few arguments which could be of use. 

- The `transform` argument allows you to pass a function to manipulate the LaTeX string of the object before it goes to PDF. 
- The `header_latex` argument allows additional LaTeX to be added to the header-includes section of the template.Rmd used for PDF generation. For example, you can pass additional LaTeX libraries here. This should be passed as a .tex file. 
- The `escape_latex` argument controls escaping for special characters in headers and footers. This can be set to `FALSE` to disable the escaping if manual escaping is preferred, i.e. if results are not as expected.

## Render to RTF

```
as_docorator(
  mytbl,
  display_name = "mytbl"
) |> 
render_rtf()
```

Due to the limitations of the RTF file format, some arguments specified in your `as_docorator()` call are not preserved, such as certain formatting or table styling. Document decoration headers are also not preserved, only titles specified as 'center' will be applied. 

## Multi-render

```
as_docorator(
  mytbl,
  display_name = "mytbl"
) |>
render_pdf() |>
render_rtf()
```

Both `render_pdf()` and `render_rtf()` return the original object created with `as_docorator()`, and as such, can be piped one after another as shown in the example above.

# markdown chunks are created correctly

    Code
      chunk_gt
    Output
       [1] ""                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
       [2] "  \\begingroup"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
       [3] "  \\setlength\\LTleft{0\\linewidth}"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     
       [4] "  \\setlength\\LTright{0\\linewidth}\\fontsize{10.0pt}{12.0pt}\\selectfont"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              
       [5] "  \\begin{longtable}{@{\\extracolsep{\\fill}}>{\\raggedright\\arraybackslash}p{\\dimexpr 0.30\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}|>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedright\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\centering\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}}"
       [6] "  \\toprule"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             
       [7] "   & num & char & fctr & date & time & datetime & currency \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
       [8] "  \\midrule\\endhead\\addlinespace[2.5pt]"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               
       [9] "  \\multicolumn{8}{>{\\raggedright\\arraybackslash}m{1\\linewidth}}{\\parbox{\\linewidth}{grp\\_a}} \\\\[2.5pt] "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [10] "  \\midrule\\addlinespace[2.5pt]"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [11] "  row\\_1 & 1.111e-01 & apricot & one & 2015-01-15 & 13:35 & 2018-01-01 02:22 & 49.950 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            
      [12] "  row\\_2 & 2.222e+00 & banana & two & 2015-02-15 & 14:40 & 2018-02-02 14:33 & 17.950 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             
      [13] "  row\\_3 & 3.333e+01 & coconut & three & 2015-03-15 & 15:45 & 2018-03-03 03:44 & 1.390 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           
      [14] "  row\\_4 & 4.444e+02 & durian & four & 2015-04-15 & 16:50 & 2018-04-04 15:55 & 65100.000 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         
      [15] "  \\midrule\\addlinespace[2.5pt]"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [16] "  \\multicolumn{8}{>{\\raggedright\\arraybackslash}m{1\\linewidth}}{\\parbox{\\linewidth}{grp\\_b}} \\\\[2.5pt] "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [17] "  \\midrule\\addlinespace[2.5pt]"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [18] "  row\\_5 & 5.550e+03 & NA & five & 2015-05-15 & 17:55 & 2018-05-05 04:00 & 1325.810 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              
      [19] "  row\\_6 & NA & fig & six & 2015-06-15 & NA & 2018-06-06 16:11 & 13.255 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
      [20] "  row\\_7 & 7.770e+05 & grapefruit & seven & NA & 19:10 & 2018-07-07 05:22 & NA \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   
      [21] "  row\\_8 & 8.880e+06 & honeydew & eight & 2015-08-15 & 20:20 & NA & 0.440 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [22] "  \\bottomrule"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
      [23] "  \\end{longtable}"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      
      [24] "  \\endgroup"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            
      [25] "  "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      

---

    Code
      chunk_gt_group
    Output
       [1] ""                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
       [2] "  \\begingroup"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
       [3] "  \\setlength\\LTleft{0\\linewidth}"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     
       [4] "  \\setlength\\LTright{0\\linewidth}\\fontsize{10.0pt}{12.0pt}\\selectfont"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              
       [5] "  \\begin{longtable}{@{\\extracolsep{\\fill}}>{\\raggedright\\arraybackslash}p{\\dimexpr 0.30\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}|>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedright\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\centering\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}}"
       [6] "  \\toprule"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             
       [7] "   & num & char & fctr & date & time & datetime & currency \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
       [8] "  \\midrule\\endhead\\addlinespace[2.5pt]"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               
       [9] "  \\multicolumn{8}{>{\\raggedright\\arraybackslash}m{1\\linewidth}}{\\parbox{\\linewidth}{grp\\_a}} \\\\[2.5pt] "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [10] "  \\midrule\\addlinespace[2.5pt]"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [11] "  row\\_1 & 1.111e-01 & apricot & one & 2015-01-15 & 13:35 & 2018-01-01 02:22 & 49.950 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            
      [12] "  row\\_2 & 2.222e+00 & banana & two & 2015-02-15 & 14:40 & 2018-02-02 14:33 & 17.950 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             
      [13] "  row\\_3 & 3.333e+01 & coconut & three & 2015-03-15 & 15:45 & 2018-03-03 03:44 & 1.390 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           
      [14] "  row\\_4 & 4.444e+02 & durian & four & 2015-04-15 & 16:50 & 2018-04-04 15:55 & 65100.000 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         
      [15] "  \\midrule\\addlinespace[2.5pt]"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [16] "  \\multicolumn{8}{>{\\raggedright\\arraybackslash}m{1\\linewidth}}{\\parbox{\\linewidth}{grp\\_b}} \\\\[2.5pt] "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [17] "  \\midrule\\addlinespace[2.5pt]"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [18] "  row\\_5 & 5.550e+03 & NA & five & 2015-05-15 & 17:55 & 2018-05-05 04:00 & 1325.810 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              
      [19] "  row\\_6 & NA & fig & six & 2015-06-15 & NA & 2018-06-06 16:11 & 13.255 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
      [20] "  row\\_7 & 7.770e+05 & grapefruit & seven & NA & 19:10 & 2018-07-07 05:22 & NA \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   
      [21] "  row\\_8 & 8.880e+06 & honeydew & eight & 2015-08-15 & 20:20 & NA & 0.440 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [22] "  \\bottomrule"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
      [23] "  \\end{longtable}"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      
      [24] "  \\endgroup"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            
      [25] "  \\pagebreak"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           
      [26] ""                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [27] "  \\begingroup"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
      [28] "  \\setlength\\LTleft{0\\linewidth}"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     
      [29] "  \\setlength\\LTright{0\\linewidth}\\fontsize{10.0pt}{12.0pt}\\selectfont"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              
      [30] "  \\begin{longtable}{@{\\extracolsep{\\fill}}>{\\raggedright\\arraybackslash}p{\\dimexpr 0.30\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}|>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedright\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\centering\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}}"
      [31] "  \\toprule"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             
      [32] "   & num & char & fctr & date & time & datetime & currency \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [33] "  \\midrule\\endhead\\addlinespace[2.5pt]"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               
      [34] "  \\multicolumn{8}{>{\\raggedright\\arraybackslash}m{1\\linewidth}}{\\parbox{\\linewidth}{grp\\_a}} \\\\[2.5pt] "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [35] "  \\midrule\\addlinespace[2.5pt]"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [36] "  row\\_1 & 1.111e-01 & apricot & one & 2015-01-15 & 13:35 & 2018-01-01 02:22 & 49.950 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            
      [37] "  row\\_2 & 2.222e+00 & banana & two & 2015-02-15 & 14:40 & 2018-02-02 14:33 & 17.950 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             
      [38] "  row\\_3 & 3.333e+01 & coconut & three & 2015-03-15 & 15:45 & 2018-03-03 03:44 & 1.390 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           
      [39] "  row\\_4 & 4.444e+02 & durian & four & 2015-04-15 & 16:50 & 2018-04-04 15:55 & 65100.000 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         
      [40] "  \\midrule\\addlinespace[2.5pt]"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [41] "  \\multicolumn{8}{>{\\raggedright\\arraybackslash}m{1\\linewidth}}{\\parbox{\\linewidth}{grp\\_b}} \\\\[2.5pt] "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [42] "  \\midrule\\addlinespace[2.5pt]"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [43] "  row\\_5 & 5.550e+03 & NA & five & 2015-05-15 & 17:55 & 2018-05-05 04:00 & 1325.810 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              
      [44] "  row\\_6 & NA & fig & six & 2015-06-15 & NA & 2018-06-06 16:11 & 13.255 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
      [45] "  row\\_7 & 7.770e+05 & grapefruit & seven & NA & 19:10 & 2018-07-07 05:22 & NA \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   
      [46] "  row\\_8 & 8.880e+06 & honeydew & eight & 2015-08-15 & 20:20 & NA & 0.440 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [47] "  \\bottomrule"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
      [48] "  \\end{longtable}"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      
      [49] "  \\endgroup"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            
      [50] "  "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      

---

    Code
      chunk_gt_group_caption
    Output
       [1] ""                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
       [2] "  \\begingroup"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
       [3] "  \\setlength\\LTleft{0\\linewidth}"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     
       [4] "  \\setlength\\LTright{0\\linewidth}\\fontsize{10.0pt}{12.0pt}\\selectfont"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              
       [5] "  \\begin{longtable}{@{\\extracolsep{\\fill}}>{\\raggedright\\arraybackslash}p{\\dimexpr 0.30\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}|>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedright\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\centering\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}}"
       [6] "  \\caption{\\label{tab:new_chunk1}test} \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
       [7] "  \\toprule"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             
       [8] "   & num & char & fctr & date & time & datetime & currency \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
       [9] "  \\midrule\\endhead\\addlinespace[2.5pt]"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               
      [10] "  \\multicolumn{8}{>{\\raggedright\\arraybackslash}m{1\\linewidth}}{\\parbox{\\linewidth}{grp\\_a}} \\\\[2.5pt] "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [11] "  \\midrule\\addlinespace[2.5pt]"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [12] "  row\\_1 & 1.111e-01 & apricot & one & 2015-01-15 & 13:35 & 2018-01-01 02:22 & 49.950 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            
      [13] "  row\\_2 & 2.222e+00 & banana & two & 2015-02-15 & 14:40 & 2018-02-02 14:33 & 17.950 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             
      [14] "  row\\_3 & 3.333e+01 & coconut & three & 2015-03-15 & 15:45 & 2018-03-03 03:44 & 1.390 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           
      [15] "  row\\_4 & 4.444e+02 & durian & four & 2015-04-15 & 16:50 & 2018-04-04 15:55 & 65100.000 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         
      [16] "  \\midrule\\addlinespace[2.5pt]"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [17] "  \\multicolumn{8}{>{\\raggedright\\arraybackslash}m{1\\linewidth}}{\\parbox{\\linewidth}{grp\\_b}} \\\\[2.5pt] "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [18] "  \\midrule\\addlinespace[2.5pt]"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [19] "  row\\_5 & 5.550e+03 & NA & five & 2015-05-15 & 17:55 & 2018-05-05 04:00 & 1325.810 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              
      [20] "  row\\_6 & NA & fig & six & 2015-06-15 & NA & 2018-06-06 16:11 & 13.255 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
      [21] "  row\\_7 & 7.770e+05 & grapefruit & seven & NA & 19:10 & 2018-07-07 05:22 & NA \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   
      [22] "  row\\_8 & 8.880e+06 & honeydew & eight & 2015-08-15 & 20:20 & NA & 0.440 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [23] "  \\bottomrule"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
      [24] "  \\end{longtable}"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      
      [25] "  \\endgroup"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            
      [26] "  \\pagebreak"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           
      [27] ""                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [28] "  \\begingroup"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
      [29] "  \\setlength\\LTleft{0\\linewidth}"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     
      [30] "  \\setlength\\LTright{0\\linewidth}\\fontsize{10.0pt}{12.0pt}\\selectfont"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              
      [31] "  \\begin{longtable}{@{\\extracolsep{\\fill}}>{\\raggedright\\arraybackslash}p{\\dimexpr 0.30\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}|>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedright\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\centering\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}>{\\raggedleft\\arraybackslash}p{\\dimexpr 0.09\\linewidth -2\\tabcolsep-1.5\\arrayrulewidth}}"
      [32] "  \\caption{\\label{tab:new_chunk2}test} \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
      [33] "  \\toprule"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             
      [34] "   & num & char & fctr & date & time & datetime & currency \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [35] "  \\midrule\\endhead\\addlinespace[2.5pt]"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               
      [36] "  \\multicolumn{8}{>{\\raggedright\\arraybackslash}m{1\\linewidth}}{\\parbox{\\linewidth}{grp\\_a}} \\\\[2.5pt] "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [37] "  \\midrule\\addlinespace[2.5pt]"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [38] "  row\\_1 & 1.111e-01 & apricot & one & 2015-01-15 & 13:35 & 2018-01-01 02:22 & 49.950 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            
      [39] "  row\\_2 & 2.222e+00 & banana & two & 2015-02-15 & 14:40 & 2018-02-02 14:33 & 17.950 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             
      [40] "  row\\_3 & 3.333e+01 & coconut & three & 2015-03-15 & 15:45 & 2018-03-03 03:44 & 1.390 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           
      [41] "  row\\_4 & 4.444e+02 & durian & four & 2015-04-15 & 16:50 & 2018-04-04 15:55 & 65100.000 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         
      [42] "  \\midrule\\addlinespace[2.5pt]"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [43] "  \\multicolumn{8}{>{\\raggedright\\arraybackslash}m{1\\linewidth}}{\\parbox{\\linewidth}{grp\\_b}} \\\\[2.5pt] "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [44] "  \\midrule\\addlinespace[2.5pt]"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [45] "  row\\_5 & 5.550e+03 & NA & five & 2015-05-15 & 17:55 & 2018-05-05 04:00 & 1325.810 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              
      [46] "  row\\_6 & NA & fig & six & 2015-06-15 & NA & 2018-06-06 16:11 & 13.255 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
      [47] "  row\\_7 & 7.770e+05 & grapefruit & seven & NA & 19:10 & 2018-07-07 05:22 & NA \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   
      [48] "  row\\_8 & 8.880e+06 & honeydew & eight & 2015-08-15 & 20:20 & NA & 0.440 \\\\ "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
      [49] "  \\bottomrule"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          
      [50] "  \\end{longtable}"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      
      [51] "  \\endgroup"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            
      [52] "  "                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      

# Header and footer processes into latex

    Code
      hf_process("My first line")
    Message
      i Coercing `header` from <character> to <fancyhead> with 1 row
    Output
      [1] "\\fancyhead[L]{\\begin{tabular}[b]{@{}l@{}}{My first line}\\end{tabular}}\\fancyhead[C]{\\begin{tabular}[b]{@{}c@{}}{\\phantom{}}\\end{tabular}}\\fancyhead[R]{\\begin{tabular}[b]{@{}r@{}}{\\phantom{}}\\end{tabular}}"

---

    Code
      hf_process(c("Two", "Lines"))
    Message
      i Coercing `header` from <character> to <fancyhead> with 2 rows
    Output
      [1] "\\fancyhead[L]{\\begin{tabular}[b]{@{}l@{}}{Two}\\\\{Lines}\\end{tabular}}\\fancyhead[C]{\\begin{tabular}[b]{@{}c@{}}{\\phantom{}}\\\\{\\phantom{}}\\end{tabular}}\\fancyhead[R]{\\begin{tabular}[b]{@{}r@{}}{\\phantom{}}\\\\{\\phantom{}}\\end{tabular}}"

---

    Code
      hf_process(fancyhead(fancyrow(right = doc_pagenum()), fancyrow(left = "a",
        center = "b", right = "c"), fancyrow(left = "something very longgggggggggggggggggggg")))
    Output
      [1] "\\fancyhead[L]{\\begin{tabular}[b]{@{}l@{}}{\\phantom{}}\\\\{a}\\\\{something very longgggggggggggggggggggg}\\end{tabular}}\\fancyhead[C]{\\begin{tabular}[b]{@{}c@{}}{\\phantom{}}\\\\{b}\\\\{\\phantom{}}\\end{tabular}}\\fancyhead[R]{\\begin{tabular}[b]{@{}r@{}}{Page \\thepage\\ of \\pageref*{LastPage}}\\\\{c}\\\\{\\phantom{}}\\end{tabular}}"

---

    Code
      hf_process(fancyfoot(fancyrow(left = "something very longgggggggggggggggggggg"),
      fancyrow(right = "something else"), fancyrow(center = "middle")))
    Output
      [1] "\\fancyfoot[L]{\\begin{tabular}[b]{@{}l@{}}{something very longgggggggggggggggggggg}\\\\{\\phantom{}}\\\\{\\phantom{}}\\end{tabular}}\\fancyfoot[C]{\\begin{tabular}[b]{@{}c@{}}{\\phantom{}}\\\\{\\phantom{}}\\\\{middle}\\end{tabular}}\\fancyfoot[R]{\\begin{tabular}[b]{@{}r@{}}{\\phantom{}}\\\\{something else}\\\\{\\phantom{}}\\end{tabular}}"

# Footnote characters are escaped correctly

    Code
      pagenum
    Output
      [1] "\\fancyhead[L]{\\begin{tabular}[b]{@{}l@{}}{Page \\thepage\\ of \\pageref*{LastPage}}\\end{tabular}}\\fancyhead[C]{\\begin{tabular}[b]{@{}c@{}}{\\phantom{}}\\end{tabular}}\\fancyhead[R]{\\begin{tabular}[b]{@{}r@{}}{\\phantom{}}\\end{tabular}}"

# gt to latex conversion preserves formatting

    Code
      preamble[1:6]
    Output
      [1] "\\begingroup"                                                                                                                      
      [2] "\\fontsize{10.0pt}{13.0pt}\\selectfont"                                                                                            
      [3] "\\begin{longtable}{llll}"                                                                                                          
      [4] "\\toprule"                                                                                                                         
      [5] " & \\multicolumn{1}{c}{{Placebo}} & \\multicolumn{1}{c}{{Xanomeline Low Dose}} & \\multicolumn{1}{c}{{Xanomeline High Dose}} \\\\ "
      [6] "\\midrule\\endhead\\addlinespace[2.5pt]"                                                                                           

# geometry processing - docx

    Code
      geom <- geom_process(geometry = geom_set(left = "1.27cm"), engine = "docx")
    Message
      Geometry value "1.27cm" changed to 1.27 inches for docx output

---

    Code
      geom <- geom_process(geometry = list(left = "1.27in"), engine = "docx")
    Message
      Geometry values for "right", "top", and "bottom" are not specified. Using
      defaults from geom_set()

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_fancyhdr.R
\name{fancyrow}
\alias{fancyrow}
\title{Construct document header row}
\usage{
fancyrow(left = NA, center = NA, right = NA)
}
\arguments{
\item{left}{Character string to be aligned to the left side of the row.}

\item{center}{Character string to be aligned to the center of the row.}

\item{right}{Character string to be aligned to the right side of the row.}
}
\value{
Object of class \code{fancyrow}
}
\description{
Define a single row in the document header/footer. Each row represents
a single line of text, with options for positioning text at left, center,
and/or right.
}
\examples{
fancyrow(left = "Left most text", right = "Right most text")

fancyrow(center = "Just text in the center")

fancyrow(left = "All", center = "Three", right = "Positions filled")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/docorate.R
\name{docorate}
\alias{docorate}
\title{Decorate and output a table, listing, or figure to a file}
\usage{
docorate(
  x,
  filename,
  path = NULL,
  header = fancyhead(fancyrow(right = doc_pagenum())),
  footer = fancyfoot(fancyrow(left = doc_path(filename, path), right = doc_datetime())),
  ...,
  fontsize = 10,
  geometry = geom_set(),
  fig_dim = c(5, 8),
  tbl_scale = TRUE,
  tbl_stub_pct = 0.3
)
}
\arguments{
\item{x}{object containing the display. See @details for more information.}

\item{filename}{required name of file including extension (note: only PDF supported currently)}

\item{path}{optional path to save the output pdf to}

\item{header}{Document header. Accepts a \code{fancyhead} object. If \code{NULL}, no header will be displayed.}

\item{footer}{Document footer Accepts a \code{fancyfoot} object. If \code{NULL}, no footer will be displayed.}

\item{...}{These dots are for future extensions and must be empty.}

\item{fontsize}{Font size (pt). Defaults to \code{10}. Accepted values: 10, 11, 12.}

\item{geometry}{Document sizing options based on the \code{geometry} latex
package. Accepts a named list. Default is \code{geom_set()}.}

\item{fig_dim}{vector containing figure height and width in inches. Defaults
to \code{c(5,8)}}

\item{tbl_scale}{Boolean for whether or not to automatically scale table columns to fit display area. Defaults to TRUE. Note that this will overwrite scaling set in the table directly unless set to FALSE.}

\item{tbl_stub_pct}{percent of total width that should be dedicated to stub column(s). If more than 1 stub column then this is the total for both.}
}
\value{
This function is called for its side effects
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

This function was deprecated and replaced with \code{as_docorator} and a corresponding render function
i.e \code{render_pdf}
}
\details{
While the \code{x} argument flexibly accepts many different R objects, the
following classes/types are recommended:
\itemize{
\item \code{gt}
\item \code{gt_group} (list of \code{gt} objects)
\item \code{ggplot}
\item list of \code{ggplot}s
\item path to PNG file created via \code{png_path()}
\item list of paths to PNG files created via \code{png_path()}
}
}
\examples{
\dontrun{
gt::gtcars |>
  dplyr::slice_head(n = 10) |>
  dplyr::select(mfr, model, year, msrp) |>
  gt::gt(groupname_col = "mfr",
         row_group_as_column = TRUE) |>
  docorate(
   header = fancyhead(fancyrow("Header 1"), fancyrow("Header 2")),
   filename = "mytbl.pdf")
 }

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_gt.R
\name{apply_to_gt_group}
\alias{apply_to_gt_group}
\title{apply a gt function to a gt_group}
\usage{
apply_to_gt_group(x, func, args, call = rlang::caller_env())
}
\arguments{
\item{x}{gt group object}

\item{func}{string with function name}

\item{args}{named list of function arguments with gt_tbl or gt_group as first arg}

\item{call}{caller env}
}
\description{
apply a gt function to a gt_group
}
\examples{
gt_tbl <- gt::exibble|> gt::gt()
gt_group <- gt::gt_group(gt_tbl, gt_tbl)

func <- gt::tab_options
arg_list_group <- list(page.header.use_tbl_headings = c(TRUE))

apply_to_gt_group(gt_group, func,arg_list_group)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_fancyhdr.R
\name{fancyfoot}
\alias{fancyfoot}
\title{Construct document footer}
\usage{
fancyfoot(...)
}
\arguments{
\item{...}{Series of objects of class \code{fancyrow}. Each entry represents a new
row in the document footer.}
}
\value{
Character string containing latex code for the \code{fancyfoot} entries as
part of the \code{fancyhdr} latex framework
}
\description{
Define document footer through a series of \code{fancyrow}s. Each row represents
a new line in the footer with options for positioning text at left, center,
and/or right positions.
}
\examples{
fancyfoot(
 fancyrow(left = "My first footnote", right = doc_datetime())
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prep_obj_tex.R
\name{prep_obj_tex}
\alias{prep_obj_tex}
\alias{prep_obj_tex.default}
\alias{prep_obj_tex.character}
\alias{prep_obj_tex.PNG}
\alias{prep_obj_tex.gt_tbl}
\title{Prepare table, listing, figure object for inclusion in the template Rmd}
\usage{
prep_obj_tex(x, transform = NULL, ...)

\method{prep_obj_tex}{default}(x, transform = NULL, ...)

\method{prep_obj_tex}{character}(x, transform = NULL, ...)

\method{prep_obj_tex}{PNG}(x, transform = NULL, ...)

\method{prep_obj_tex}{gt_tbl}(x, transform = NULL, ...)
}
\arguments{
\item{x}{docorator object containing info about the table, listing or figure}

\item{transform}{optional latex transformation function to apply to a gt latex string}

\item{...}{additional args}
}
\value{
object to be included as-is in render engine
}
\description{
Prepare table, listing, figure object for inclusion in the template Rmd
}
\examples{
docorator <- gt::exibble |>
gt::gt() |>
as_docorator(
display_name = "mytbl", footer = NULL,
save_object = FALSE)

prepared_obj <- prep_obj_tex(docorator)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale_gt.R
\name{apply_scale}
\alias{apply_scale}
\alias{apply_scale.default}
\alias{apply_scale.gt_tbl}
\alias{apply_scale.gt_group}
\alias{apply_scale.list}
\title{Apply scaling to gt objects}
\usage{
apply_scale(x, fontsize, tbl_scale, tbl_stub_pct)

\method{apply_scale}{default}(x, fontsize, tbl_scale, tbl_stub_pct)

\method{apply_scale}{gt_tbl}(x, fontsize, tbl_scale, tbl_stub_pct)

\method{apply_scale}{gt_group}(x, fontsize, tbl_scale, tbl_stub_pct)

\method{apply_scale}{list}(x, fontsize, tbl_scale, tbl_stub_pct)
}
\arguments{
\item{x}{gt object}

\item{fontsize}{document font size}

\item{tbl_scale}{Boolean for whether or not to automatically scale table columns to fit display area. Defaults to TRUE. Note that this will overwrite scaling set in the table directly unless set to FALSE.}

\item{tbl_stub_pct}{percent of total width that should be dedicated to stub column(s). If more than 1 stub column then this is the total for both.}
}
\value{
scaled gt object
}
\description{
Apply scaling to gt objects
}
\examples{
gt <- gt::exibble |>
gt::gt()

apply_scale(gt, fontsize = 10, tbl_scale = FALSE, tbl_stub_pct = "20\%")
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_fancyhdr.R
\name{fancyhead}
\alias{fancyhead}
\title{Construct document header}
\usage{
fancyhead(...)
}
\arguments{
\item{...}{Series of objects of class \code{fancyrow}. Each entry represents a new
row in the document header.}
}
\value{
Character string containing latex code for the \code{fancyhead} entries as
part of the \code{fancyhdr} latex framework
}
\description{
Define document header through a series of \code{fancyrow}s. Each row represents
a new line in the header with options for positioning text at left, center,
and/or right positions.
}
\examples{
fancyhead(
 fancyrow(left = "Protocol: 12345", right = doc_pagenum()),
 fancyrow(center = "Demographic Summary")
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_fancyhdr.R
\name{hf_height}
\alias{hf_height}
\alias{hf_height.default}
\alias{hf_height.fancyhdr}
\title{Calculate desired header or footer height for the document}
\usage{
hf_height(x, fontsize)

\method{hf_height}{default}(x, fontsize)

\method{hf_height}{fancyhdr}(x, fontsize)
}
\arguments{
\item{x}{header or footer}

\item{fontsize}{Document font size (pt)}
}
\value{
Numeric value
}
\description{
Calculate desired header or footer height for the document
}
\examples{
header <- fancyhead(
fancyrow(left = "Protocol: 12345", right = doc_pagenum()),
fancyrow(center = "Demographic Summary"))

hf_height(header, 10)

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale_gt.R
\name{scale_gt}
\alias{scale_gt}
\title{Scale gt table contents for document}
\usage{
scale_gt(x, tbl_stub_pct = 0.3)
}
\arguments{
\item{x}{table of class \code{gt_tbl}}

\item{tbl_stub_pct}{percent of total width that should be dedicated to stub column(s). If more than 1 stub column then this is the total for both.}
}
\value{
Table with col_widths settings applied
}
\description{
Scale gt table contents for document
}
\examples{

gt::gtcars |>
  dplyr::slice_head(n = 10) |>
  dplyr::select(mfr, model, year, msrp, ctry_origin) |>
  gt::gt(
      groupname_col = "ctry_origin",
      rowname_col = "mfr",
      row_group_as_column = TRUE) |>
  scale_gt(tbl_stub_pct = 0.4)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/docorate.R
\name{as_docorator}
\alias{as_docorator}
\title{Create docorator object}
\usage{
as_docorator(
  x,
  display_name,
  display_loc = NULL,
  header = fancyhead(fancyrow(right = doc_pagenum())),
  footer = NULL,
  save_object = TRUE,
  object_loc = display_loc,
  ...,
  fontsize = 10,
  geometry = geom_set(),
  fig_dim = c(5, 8),
  convert_ggplot = TRUE,
  tbl_scale = TRUE,
  tbl_stub_pct = 0.3
)
}
\arguments{
\item{x}{object containing the display. See @details for more information.}

\item{display_name}{required name of file (excluding extension)}

\item{display_loc}{optional path to save the output file to}

\item{header}{Document header. Accepts a \code{fancyhead} object. If \code{NULL}, no header will be displayed.}

\item{footer}{Document footer Accepts a \code{fancyfoot} object. If \code{NULL}, no footer will be displayed.}

\item{save_object}{Boolean indicating if a docorator object should be saved.}

\item{object_loc}{path for the docorator object -  defaults to display_loc argument.}

\item{...}{These dots are for future extensions and must be empty.}

\item{fontsize}{Font size (pt). Defaults to \code{10}. Accepted values: 10, 11, 12.}

\item{geometry}{Document sizing options based on the \code{geometry} latex
package. Accepts a named list. Default is \code{geom_set()}.}

\item{fig_dim}{vector containing figure height and width in inches. Defaults
to \code{c(5,8)}}

\item{convert_ggplot}{Boolean for whether or not to convert ggplot objects to PNG files to preserve scaling for all render engines. Defaults to TRUE.}

\item{tbl_scale}{Boolean for whether or not to automatically scale table columns to fit display area. Defaults to TRUE. Note that this will overwrite scaling set in the table directly unless set to FALSE.}

\item{tbl_stub_pct}{percent of total width that should be dedicated to stub column(s). If more than 1 stub column then this is the total for both.}
}
\value{
docorator object
}
\description{
Create docorator object
}
\details{
While the \code{x} argument flexibly accepts many different R objects, the
following classes/types are recommended:
\itemize{
\item \code{gt}
\item \code{gt_group} (list of \code{gt} objects)
\item \code{ggplot}
\item list of \code{ggplot}s
\item path to PNG file created via \code{png_path()}
\item list of paths to PNG files created via \code{png_path()}
}
}
\examples{

\dontrun{
gt::gtcars |>
  dplyr::slice_head(n = 10) |>
  dplyr::select(mfr, model, year, msrp) |>
  gt::gt(groupname_col = "mfr",
         row_group_as_column = TRUE) |>
  as_docorator(
   header = fancyhead(fancyrow("Header 1"), fancyrow("Header 2")),
   display_name = "mytbl",
   footer = NULL)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{doc_datetime}
\alias{doc_datetime}
\title{Date and time of program run}
\usage{
doc_datetime()
}
\value{
character string
}
\description{
Date and time of program run
}
\examples{
\donttest{
 doc_datetime()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_pkg_version}
\alias{check_pkg_version}
\title{Check package versions for docorator object are the same as loaded}
\usage{
check_pkg_version(x)
}
\arguments{
\item{x}{docorator object}
}
\description{
Check package versions for docorator object are the same as loaded
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{png_path}
\alias{png_path}
\title{Path of png file}
\usage{
png_path(path = NULL)
}
\arguments{
\item{path}{path to png}
}
\value{
object with png attribute
}
\description{
Path of png file
}
\examples{
\dontrun{
png_path <- png_path(path = "path_to_my_png.png")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geometry.R
\name{geom_set}
\alias{geom_set}
\title{Set document geometry defaults}
\usage{
geom_set(...)
}
\arguments{
\item{...}{Series of named value pairs for latex geometry options}
}
\value{
Named list
}
\description{
Set document geometry defaults
}
\details{
Type geom_set() in console to view package defaults. Use of the
function will add to the defaults and/or override included defaults of the
same name. For values that are \code{NULL}, such as for \code{headheight} and
\code{footskip}, the values will be calculated automatically based on the number
of header and/or footer lines. For all geometry settings, reference the
documentation here: https://texdoc.org/serve/geometry.pdf/0
}
\examples{
# view defaults
geom_set()

# Update the defaults
geom_set(left="0.5in", right="0.5in")

# add new defaults
geom_set(paper = "legalpaper")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{create_chunks_all}
\alias{create_chunks_all}
\title{Create code chunks}
\usage{
create_chunks_all(x, transform)
}
\arguments{
\item{x}{docorator object}

\item{transform}{optional latex transformation function to apply to a gt latex string}
}
\value{
printed code chunk(s) to be included as-is in the render engine
}
\description{
Create code chunks
}
\examples{
\dontrun{
docorator <- gt::exibble |>
  gt::gt() |>
  as_docorator(save_object = FALSE)
create_chunks_all(docorator, transform = NULL)
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{doc_pagenum}
\alias{doc_pagenum}
\title{Automatic page numbering}
\usage{
doc_pagenum()
}
\value{
character string containing latex code
}
\description{
Automatic page numbering
}
\examples{
doc_pagenum()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{doc_path}
\alias{doc_path}
\title{Path of program}
\usage{
doc_path(filename = NULL, path = NULL)
}
\arguments{
\item{filename}{Name of output file}

\item{path}{program path}
}
\value{
character string
}
\description{
Path of program
}
\examples{
\dontrun{
 doc_path(filename = "my_tbl.pdf", path = NULL)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prep_obj_docx.R
\name{prep_obj_docx}
\alias{prep_obj_docx}
\alias{prep_obj_docx.default}
\alias{prep_obj_docx.PNG}
\alias{prep_obj_docx.gt_tbl}
\alias{prep_obj_docx.list}
\title{Prepare table, listing, figure object for inclusion in the template Rmd}
\usage{
prep_obj_docx(x, ...)

\method{prep_obj_docx}{default}(x, ...)

\method{prep_obj_docx}{PNG}(x, ...)

\method{prep_obj_docx}{gt_tbl}(x, ...)

\method{prep_obj_docx}{list}(x, ...)
}
\arguments{
\item{x}{docorator object containing info about the table, listing or figure}
}
\value{
xml to be inserted into word document
}
\description{
Prepare table, listing, figure object for inclusion in the template Rmd
}
\examples{
docorator <- gt::exibble |>
gt::gt() |>
as_docorator(
display_name = "mytbl", footer = NULL,
save_object = FALSE)

prepared_obj <- prep_obj_docx(docorator)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_fancyhdr.R
\name{hf_process}
\alias{hf_process}
\alias{hf_process.default}
\alias{hf_process.character}
\alias{hf_process.fancyhead}
\alias{hf_process.fancyfoot}
\title{Process headers/footers}
\usage{
hf_process(x, escape_latex = TRUE, fontsize = 10, engine = "pdf")

\method{hf_process}{default}(x, escape_latex = TRUE, fontsize = 10, engine = "pdf")

\method{hf_process}{character}(x, escape_latex = TRUE, fontsize = 10, engine = "pdf")

\method{hf_process}{fancyhead}(x, escape_latex = TRUE, fontsize = 10, engine = "pdf")

\method{hf_process}{fancyfoot}(x, escape_latex = TRUE, fontsize = fontsize, engine = "pdf")
}
\arguments{
\item{x}{header or footer}

\item{escape_latex}{Boolean to escape latex in header/footer}

\item{engine}{render engine to dictate how the headers and footers should be constructed}
}
\value{
character string containing headers and footers latex code
}
\description{
Process headers/footers
}
\examples{
header <- fancyhead(
fancyrow(left = "Protocol: 12345", right = doc_pagenum()),
fancyrow(center = "Demographic Summary"))

hf_process(header)

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/render.R
\name{render_rtf}
\alias{render_rtf}
\title{Render to rtf}
\usage{
render_rtf(
  x,
  display_loc = NULL,
  remove_unicode_ws = TRUE,
  use_page_header = FALSE,
  version_check = TRUE
)
}
\arguments{
\item{x}{\code{docorator} object}

\item{display_loc}{path to save the output rtf to}

\item{remove_unicode_ws}{Option to remove unicode white space from text.}

\item{use_page_header}{If \code{TRUE} then all table headings will be migrated to the page header. See https://gt.rstudio.com/reference/tab_options.html#arg-page-header-use-tbl-headings}

\item{version_check}{Boolean indicating whether to print a note if gt or ggplot versions dont match between the original docorator object and the one being used for rendering}
}
\value{
This function saves an rtf to a specified location
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}
}
\details{
Option \code{remove_unicode_ws} serves as a workaround for this
\href{https://github.com/rstudio/gt/issues/1437}{issue} in gt
}
\section{Examples}{


\if{html}{\out{<div class="sourceCode r">}}\preformatted{gt::gtcars |>
  dplyr::slice_head(n = 10) |>
  dplyr::select(mfr, model, year, msrp) |>
  gt::gt(groupname_col = "mfr",
         row_group_as_column = TRUE) |>
  as_docorator(
   header = fancyhead(fancyrow("Header 1"), fancyrow("Header 2")),
   display_name = "mytbl") |>
 render_rtf()
}\if{html}{\out{</div>}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_gt.R
\name{hf_extract}
\alias{hf_extract}
\title{extract header footer info from docorator object}
\usage{
hf_extract(x)
}
\arguments{
\item{x}{a docorator object}
}
\value{
list of header footer info
}
\description{
extract header footer info from docorator object
}
\examples{
docorator <- gt::exibble |>
  gt::gt()|>
  as_docorator(
  display_name = "mytbl",
  header = fancyhead(
  fancyrow(left = "Protocol: 12345", right = doc_pagenum()),
  fancyrow(center = "Demographic Summary")
  ),
  footer = NULL,
  save_object = FALSE)

hf_extract(docorator)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_gt.R
\name{hf_to_gt}
\alias{hf_to_gt}
\title{Apply headers and footnotes to gt object}
\usage{
hf_to_gt(x)
}
\arguments{
\item{x}{a docorator object}
}
\value{
gt object (either tbl or group)
}
\description{
Pulls headers and footnotes from the docorator object and applies them to the gt object
Note: this function is only for use by downstream tools and is not intended for general users
}
\examples{
docorator <- gt::exibble |>
  gt::gt()|>
  as_docorator(
  display_name = "my_tbl",
  header = fancyhead(
  fancyrow(left = "Protocol: 12345", right = doc_pagenum()),
  fancyrow(center = "Demographic Summary")
  ),
  footer = NULL,
  save_object = FALSE)

hf_to_gt(docorator)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/render_docx.R
\name{render_docx}
\alias{render_docx}
\title{Render to docx}
\usage{
render_docx(x, display_loc = NULL, version_check = TRUE)
}
\arguments{
\item{x}{\code{docorator} object}

\item{display_loc}{optional path to save the output docx to}

\item{version_check}{Boolean indicating whether to print a note if gt or ggplot versions dont match between the original docorator object and the one being used for rendering}
}
\value{
This function saves a docx to a specified location
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}
}
\section{Examples}{


\if{html}{\out{<div class="sourceCode r">}}\preformatted{gt::gtcars |>
  dplyr::slice_head(n = 10) |>
  dplyr::select(mfr, model, year, msrp) |>
  gt::gt(groupname_col = "mfr",
         row_group_as_column = TRUE) |>
  as_docorator(
   header = fancyhead(fancyrow("Header 1"), fancyrow("Header 2")),
   display_name = "mytbl") |>
 render_docx()
}\if{html}{\out{</div>}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prep_obj_rtf.R
\name{prep_obj_rtf}
\alias{prep_obj_rtf}
\alias{prep_obj_rtf.default}
\alias{prep_obj_rtf.character}
\alias{prep_obj_rtf.PNG}
\alias{prep_obj_rtf.ggplot}
\alias{prep_obj_rtf.gt_tbl}
\alias{prep_obj_rtf.list}
\title{Prepare table, listing, figure object for output to rtf}
\usage{
prep_obj_rtf(x, ...)

\method{prep_obj_rtf}{default}(x, ...)

\method{prep_obj_rtf}{character}(x, ...)

\method{prep_obj_rtf}{PNG}(x, ...)

\method{prep_obj_rtf}{ggplot}(x, ...)

\method{prep_obj_rtf}{gt_tbl}(x, ...)

\method{prep_obj_rtf}{list}(x, ...)
}
\arguments{
\item{x}{docorator object containing display information}

\item{...}{additional args}
}
\value{
gt object to be included as-is to render engine
}
\description{
Prepare table, listing, figure object for output to rtf
}
\examples{
docorator <- gt::exibble |>
gt::gt() |>
as_docorator(
display_name = "my_tbl",
footer=NULL,
save_object = FALSE)

prepared_obj <- prep_obj_rtf(docorator)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/render.R
\name{render_pdf}
\alias{render_pdf}
\title{Render to pdf}
\usage{
render_pdf(
  x,
  display_loc = NULL,
  transform = NULL,
  header_latex = NULL,
  keep_tex = FALSE,
  escape_latex = TRUE,
  quarto = FALSE,
  version_check = TRUE,
  fancywrap = TRUE
)
}
\arguments{
\item{x}{\code{docorator} object}

\item{display_loc}{optional path to save the output pdf to}

\item{transform}{optional latex transformation function to apply to a gt latex string}

\item{header_latex}{optional .tex file of header latex}

\item{keep_tex}{Boolean indicating if to keep resulting .tex file from latex conversion. Defaults to FALSE.}

\item{escape_latex}{Boolean indicating if headers and footers of a gt table should be escaped with gt::escape_latex}

\item{quarto}{Boolean indicating whether to use Quarto as the rendering engine. Defaults to \code{FALSE}, which uses Rmarkdown to render. \ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}}

\item{version_check}{Boolean indicating whether to print a note if gt or ggplot versions dont match between the original docorator object and the one being used for rendering}

\item{fancywrap}{Boolean indicating if headers and footers should be split to fit the page. Defaults to \code{TRUE}. Note that only fancyrows with one \code{left}, \code{right} OR \code{center} element will be wrapped. \ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}}
}
\value{
This function saves a pdf to a specified location
}
\description{
Render to pdf
}
\section{Examples}{


\if{html}{\out{<div class="sourceCode r">}}\preformatted{gt::gtcars |>
  dplyr::slice_head(n = 10) |>
  dplyr::select(mfr, model, year, msrp) |>
  gt::gt(groupname_col = "mfr",
         row_group_as_column = TRUE) |>
  as_docorator(
   header = fancyhead(fancyrow("Header 1"), fancyrow("Header 2")),
   display_name = "mytbl") |>
 render_pdf()
}\if{html}{\out{</div>}}
}
